@@ -6,7 +6,6 @@
 # - if cinnamon --replace was called from Melange, it will be killed when this process is closed
 #   - Currently only occurs when Melange is started by Geany.
 # - List extensions that failed to load ?
-# - Check for issues with multiple monitors
 # - Add insert button to "simple types" inspect dialog ? is there actual use for these types inserted as results ?
 # - Remove javascript version ?
 # - Load all enabled log categories and window height from gsettings
@@ -15,6 +14,7 @@
 #   - When pressing ctrl + r, search history
 #   - auto-completion ?
 
+import sys
 import os
 import pyinotify
 import gi
@@ -22,10 +22,36 @@ gi.require_version('Gtk', '3.0')
 from gi.repository import Gio, Gtk, GObject, Gdk, Pango, GLib
 import dbus, dbus.service, dbus.glib
 from pageutils import *
+from lookingglass_proxy import LookingGlassProxy
 from dbus.mainloop.glib import DBusGMainLoop
 
-LG_DBUS_NAME = "org.Cinnamon.LookingGlass"
-LG_DBUS_PATH = "/org/Cinnamon/LookingGlass"
+MELANGE_DBUS_NAME = "org.Cinnamon.Melange"
+MELANGE_DBUS_PATH = "/org/Cinnamon/Melange"
+
+class MenuButton(Gtk.Button):
+    def __init__(self, text):
+        Gtk.Button.__init__(self, text)
+        self.connect("clicked", self.onClicked)
+
+    def set_popup(self, menu):
+        self.menu = menu
+
+    def onClicked(self, widget):
+        x, y, w, h = self.getScreenCoordinates()
+        self.menu.popup(None, None, lambda menu, data: (x, y+h, True), None, 1, 0)
+                
+    def getScreenCoordinates(self):
+        parent = self.get_parent_window()
+        x, y = parent.get_root_origin()
+        w = parent.get_width()
+        h = parent.get_height()
+        extents = parent.get_frame_extents()
+        allocation = self.get_allocation()
+        return (x + (extents.width-w)/2 + allocation.x,
+                y + (extents.height-h)-(extents.width-w)/2 + allocation.y,
+                allocation.width,
+                allocation.height)
+
 
 class ResizeGrip(Gtk.Widget):
     def __init__(self, parent):
@@ -158,7 +184,7 @@ class CommandLine(Gtk.Entry):
             self.set_text("")
             self.settings.set_strv("looking-glass-history", self.history)
 
-            dbusManager.cinnamonDBus.lgEval(command)
+            lookingGlassProxy.Eval(command)
 
 class NewLogDialog(Gtk.Dialog):
     def __init__(self, parent):
@@ -337,32 +363,46 @@ class ClosableTabLabel(Gtk.Box):
 class CinnamonLog(dbus.service.Object):
     def __init__ (self):
         self.window = None
-        dbus.service.Object.__init__ (self, dbusManager.sessionBus, LG_DBUS_PATH, LG_DBUS_NAME)
+        dbus.service.Object.__init__ (self, dbus.SessionBus (), MELANGE_DBUS_PATH, MELANGE_DBUS_NAME)
 
-    @dbus.service.method (LG_DBUS_NAME, in_signature='', out_signature='')
-    def show(self):
+    @dbus.service.method (MELANGE_DBUS_NAME, in_signature='b', out_signature='')
+    def show(self, startInspector):
         if self.window is not None:
-            if self.window.get_visible():
+            if startInspector and lookingGlassProxy:
+                lookingGlassProxy.StartInspector()
+                self.window.hide()
+            elif self.window.get_visible():
                 self.window.hide()
             else:
-                self.window.present()
-                self.window.move(0,0)
-                self.window.focus()
+                self.showAndFocus()
         else:
             self.run()
+            if startInspector and lookingGlassProxy:
+                lookingGlassProxy.StartInspector()
+                self.window.hide()
             Gtk.main()
 
+    def showAndFocus(self):
+        self.window.present()
+        screen = self.window.get_screen()
+        geom = screen.get_monitor_geometry(screen.get_primary_monitor())
+        w, h = self.window.get_size()
+        self.window.resize(geom.width, h)
+        self.window.move(geom.x, geom.y)
+        self.window.set_focus(self.commandline)
+
     def run(self):
         self.window = Gtk.Window(Gtk.WindowType.TOPLEVEL)
         screen = self.window.get_screen()
+        geom = screen.get_monitor_geometry(screen.get_primary_monitor())
 
         self.window.set_border_width(0)
         self.window.set_decorated(False)
         self.window.set_skip_taskbar_hint(True)
         self.window.set_keep_above(True)
-        self.window.set_default_size(screen.get_width(), 200)
+        self.window.set_default_size(geom.width, 200)
         self.window.set_has_resize_grip(False)
-        self.window.move(0,0)
+        self.window.move(geom.x,geom.y)
 
         self.window.connect("delete_event", self.onDelete)
         self.window.connect("key-press-event", self.onKeyPress)
@@ -402,7 +442,8 @@ class CinnamonLog(dbus.service.Object):
         table.attach(Gtk.Label("Exec:"), column, column+1, 1, 2, 0, 0, 3)
         column += 1
 
-        table.attach(CommandLine(), column, column+1, 1, 2, Gtk.AttachOptions.EXPAND|Gtk.AttachOptions.FILL, 0, 3, 2)
+        self.commandline = CommandLine()
+        table.attach(self.commandline, column, column+1, 1, 2, Gtk.AttachOptions.EXPAND|Gtk.AttachOptions.FILL, 0, 3, 2)
         column += 1
 
         global statusLabel
@@ -421,6 +462,7 @@ class CinnamonLog(dbus.service.Object):
         self.window.show_all()
         self.activatePage("results")
         setStatus(True)
+        self.window.set_focus(self.commandline)
 
     def createMenuItem(self, text, callback):
         item = Gtk.MenuItem(text)
@@ -438,7 +480,10 @@ class CinnamonLog(dbus.service.Object):
         menu.append(self.createMenuItem('Quit', self.onExitClicked))
         menu.show_all()
 
-        button = Gtk.MenuButton(u"Actions \u25BE")
+        if hasattr(Gtk, 'MenuButton'):
+            button = Gtk.MenuButton(u"Actions \u25BE")
+        else:
+            button = MenuButton(u"Actions \u25BE")
         button.set_popup(menu)
         return button
 
@@ -500,7 +545,7 @@ class CinnamonLog(dbus.service.Object):
         return False
 
     def onPickerClicked(self, widget):
-        dbusManager.cinnamonDBus.lgStartInspector()
+        lookingGlassProxy.StartInspector()
         self.window.hide()
 
     def createDummyPage(self, text, description):
@@ -509,7 +554,7 @@ class CinnamonLog(dbus.service.Object):
 
     def createPage(self, text, moduleName):
         module = __import__("page_%s" % moduleName)
-        module.dbusManager = dbusManager
+        module.lookingGlassProxy = lookingGlassProxy
         module.cinnamonLog = self
         label = Gtk.Label(text)
         page = module.ModulePage()
@@ -517,7 +562,7 @@ class CinnamonLog(dbus.service.Object):
         self.notebook.append_page(page, label)
 
     def activatePage(self, moduleName):
-        self.window.present()
+        self.showAndFocus()
         page = self.notebook.page_num(self.pages[moduleName])
         self.notebook.set_current_page(page)
 
@@ -527,60 +572,21 @@ def setStatus(status):
     else:
         statusLabel.show()
 
-class DBusManager:
-    def __init__ (self):
-        self.sessionBus = dbus.SessionBus ()
-        self.dbus = prox = self.createSessionDBusProxy("org.freedesktop.DBus", "/org/freedesktop/DBus")
-        self.dbus.connect_to_signal("NameOwnerChanged", self.onNameOwnerChanged)
-
-        self.cinnamonSignals = []
-        self.cinnamonReconnectCallback = []
-        self.initCinnamonProxy()
-
-    def initCinnamonProxy(self):
-        self.cinnamonDBus = self.createSessionDBusProxy("org.Cinnamon", "/org/Cinnamon")
-        for callback in self.cinnamonReconnectCallback:
-            callback()
-        for name, callback in self.cinnamonSignals:
-            self.cinnamonDBus.connect_to_signal(name, callback)
-
-    def addReconnectCallback(self, callback):
-        self.cinnamonReconnectCallback.append(callback)
-
-    def connectToCinnamonSignal(self, name, callback):
-        self.cinnamonDBus.connect_to_signal(name, callback)
-        self.cinnamonSignals.append((name, callback))
-
-    def createSessionDBusProxy(self, name, path):
-        try:
-            proxy = self.sessionBus.get_object(name, path)
-            return proxy
-        except dbus.exceptions.DBusException as e:
-            print(e)
-            return None
-
-    def onNameOwnerChanged(self, name, old, new):
-        if name == "org.Cinnamon":
-            if new == "":
-                print "Cinnamon offline"
-                setStatus(False)
-            else:
-                print "Cinnamon online"
-                setStatus(True)
-                self.initCinnamonProxy()
-
 if __name__ == "__main__":
     GObject.type_register(ResizeGrip)
     DBusGMainLoop(set_as_default=True)
 
-    global dbusManager
-    dbusManager = DBusManager()
+    global lookingGlassProxy
+    lookingGlassProxy = LookingGlassProxy()
+    lookingGlassProxy.addStatusChangeCallback(setStatus)
 
-    request = dbusManager.sessionBus.request_name(LG_DBUS_NAME, dbus.bus.NAME_FLAG_DO_NOT_QUEUE)
+    sessionBus = dbus.SessionBus ()
+    request = sessionBus.request_name(MELANGE_DBUS_NAME, dbus.bus.NAME_FLAG_DO_NOT_QUEUE)
     if request != dbus.bus.REQUEST_NAME_REPLY_EXISTS:
         app = CinnamonLog()
     else:
-        object = dbusManager.sessionBus.get_object(LG_DBUS_NAME, LG_DBUS_PATH)
-        app = dbus.Interface(object, LG_DBUS_NAME)
+        object = sessionBus.get_object(MELANGE_DBUS_NAME, MELANGE_DBUS_PATH)
+        app = dbus.Interface(object, MELANGE_DBUS_NAME)
 
-    app.show()
+    startInspector = len(sys.argv) == 2 and sys.argv[1] == 'inspect'
+    app.show(startInspector)
