@@ -17,11 +17,12 @@
 #   License along with this library; if not, write to the Free Software
 #   Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
 
-from gi.repository import Gtk, GObject, Gio, GdkPixbuf, Gdk, GMenu, GLib
+from gi.repository import Gtk, GObject, Gio, GdkPixbuf, Gdk, CMenu, GLib
 import cgi
 import os
 import gettext
 import subprocess
+import shutil
 
 from cme import config
 gettext.bindtextdomain(config.GETTEXT_PACKAGE, config.localedir)
@@ -50,8 +51,15 @@ class MainWindow(object):
         self.setupItemTree()
         self.tree.get_object('edit_delete').set_sensitive(False)
         self.tree.get_object('edit_properties').set_sensitive(False)
+        self.tree.get_object('edit_cut').set_sensitive(False)
+        self.tree.get_object('edit_copy').set_sensitive(False)
+        self.tree.get_object('edit_paste').set_sensitive(False)
         self.tree.get_object('move_up_button').set_sensitive(False)
         self.tree.get_object('move_down_button').set_sensitive(False)
+        self.cut_copy_buffer = None
+        self.file_id = None
+        self.last_tree = None
+        self.main_window = self.tree.get_object('mainwindow')
 
     def run(self):
         self.loadMenus()
@@ -70,23 +78,23 @@ class MainWindow(object):
         item_id = None
         if iter:
             update_items = True
-            if isinstance(items[iter][3], GMenu.TreeEntry):
+            if isinstance(items[iter][3], CMenu.TreeEntry):
                 item_id = items[iter][3].get_desktop_file_id()
-                update_type = GMenu.TreeItemType.ENTRY
-            elif isinstance(items[iter][3], GMenu.TreeDirectory):
+                update_type = CMenu.TreeItemType.ENTRY
+            elif isinstance(items[iter][3], CMenu.TreeDirectory):
                 item_id = os.path.split(items[iter][3].get_desktop_file_path())[1]
-                update_type = GMenu.TreeItemType.DIRECTORY
-            elif isinstance(items[iter][3], GMenu.TreeSeparator):
+                update_type = CMenu.TreeItemType.DIRECTORY
+            elif isinstance(items[iter][3], CMenu.TreeSeparator):
                 item_id = items.get_path(iter)
-                update_type = GMenu.TreeItemType.SEPARATOR
+                update_type = CMenu.TreeItemType.SEPARATOR
         menus, iter = menu_tree.get_selection().get_selected()
         update_menus = False
         menu_id = None
         if iter:
-            if menus[iter][2].get_desktop_file_path():
-                menu_id = os.path.split(menus[iter][2].get_desktop_file_path())[1]
+            if menus[iter][3].get_desktop_file_path():
+                menu_id = os.path.split(menus[iter][3].get_desktop_file_path())[1]
             else:
-                menu_id = menus[iter][2].get_menu_id()
+                menu_id = menus[iter][3].get_menu_id()
             update_menus = True
         self.loadMenus()
         #find current menu in new tree
@@ -100,13 +108,13 @@ class MainWindow(object):
             i = 0
             for item in item_tree.get_model():
                 found = False
-                if update_type != GMenu.TreeItemType.SEPARATOR:
-                    if isinstance (item[3], GMenu.TreeEntry) and item[3].get_desktop_file_id() == item_id:
+                if update_type != CMenu.TreeItemType.SEPARATOR:
+                    if isinstance (item[3], CMenu.TreeEntry) and item[3].get_desktop_file_id() == item_id:
                         found = True
-                    if isinstance (item[3], GMenu.TreeDirectory) and item[3].get_desktop_file_path() and update_type == GMenu.TreeItemType.DIRECTORY:
+                    if isinstance (item[3], CMenu.TreeDirectory) and item[3].get_desktop_file_path() and update_type == CMenu.TreeItemType.DIRECTORY:
                         if os.path.split(item[3].get_desktop_file_path())[1] == item_id:
                             found = True
-                if isinstance(item[3], GMenu.TreeSeparator):
+                if isinstance(item[3], CMenu.TreeSeparator):
                     if not isinstance(item_id, tuple):
                         #we may not skip the increment via "continue"
                         i += 1
@@ -127,27 +135,27 @@ class MainWindow(object):
         return False
 
     def findMenu(self, menus, path, iter, menu_id):
-        if not menus[path][2].get_desktop_file_path():
-            if menu_id == menus[path][2].get_menu_id():
+        if not menus[path][3].get_desktop_file_path():
+            if menu_id == menus[path][3].get_menu_id():
                 menu_tree = self.tree.get_object('menu_tree')
                 menu_tree.expand_to_path(path)
                 menu_tree.get_selection().select_path(path)
                 return True
             return False
-        if os.path.split(menus[path][2].get_desktop_file_path())[1] == menu_id:
+        if os.path.split(menus[path][3].get_desktop_file_path())[1] == menu_id:
             menu_tree = self.tree.get_object('menu_tree')
             menu_tree.expand_to_path(path)
             menu_tree.get_selection().select_path(path)
             return True
 
     def setupMenuTree(self):
-        self.menu_store = Gtk.TreeStore(GdkPixbuf.Pixbuf, str, object)
-        menus = self.tree.get_object('menu_tree')
-        column = Gtk.TreeViewColumn(_('Name'))
+        self.menu_store = Gtk.TreeStore(object, str, bool, object) # bool is unused, just a placeholder
+        menus = self.tree.get_object('menu_tree')                            # so object is the same index for
+        column = Gtk.TreeViewColumn(_("Name"))                               # the menu tree and item tree
         column.set_spacing(4)
         cell = Gtk.CellRendererPixbuf()
         column.pack_start(cell, False)
-        column.add_attribute(cell, 'pixbuf', 0)
+        column.set_cell_data_func(cell, self.icon_data_func, 0)
         cell = Gtk.CellRendererText()
         column.pack_start(cell, True)
         column.add_attribute(cell, 'markup', 1)
@@ -156,7 +164,7 @@ class MainWindow(object):
 
     def setupItemTree(self):
         items = self.tree.get_object('item_tree')
-        column = Gtk.TreeViewColumn(_('Show'))
+        column = Gtk.TreeViewColumn(_("Show"))
         cell = Gtk.CellRendererToggle()
         cell.connect('toggled', self.on_item_tree_show_toggled)
         column.pack_start(cell, True)
@@ -164,20 +172,25 @@ class MainWindow(object):
         #hide toggle for separators
         column.set_cell_data_func(cell, self._cell_data_toggle_func)
         items.append_column(column)
-        column = Gtk.TreeViewColumn(_('Item'))
+        column = Gtk.TreeViewColumn(_("Item"))
         column.set_spacing(4)
         cell = Gtk.CellRendererPixbuf()
         column.pack_start(cell, False)
-        column.add_attribute(cell, 'pixbuf', 1)
+        column.set_cell_data_func(cell, self.icon_data_func, 1)
         cell = Gtk.CellRendererText()
         column.pack_start(cell, True)
         column.add_attribute(cell, 'markup', 2)
         items.append_column(column)
-        self.item_store = Gtk.ListStore(bool, GdkPixbuf.Pixbuf, str, object)
+        self.item_store = Gtk.ListStore(bool, object, str, object)
         items.set_model(self.item_store)
 
+    def icon_data_func(self, column, cell, model, iter, data=None):
+        wrapper = model.get_value(iter, data)
+        if wrapper:
+            cell.set_property("surface", wrapper.surface)
+
     def _cell_data_toggle_func(self, tree_column, renderer, model, treeiter, data=None):
-        if isinstance(model[treeiter][3], GMenu.TreeSeparator):
+        if isinstance(model[treeiter][3], CMenu.TreeSeparator):
             renderer.set_property('visible', False)
         else:
             renderer.set_property('visible', True)
@@ -199,19 +212,19 @@ class MainWindow(object):
             if not show:
                 name = "<small><i>%s</i></small>" % (name,)
 
-            icon = util.getIcon(menu)
-            iters[menu] = self.menu_store.append(iters[parent], (icon, name, menu))
+            icon = util.getIcon(menu, self.main_window)
+            iters[menu] = self.menu_store.append(iters[parent], (icon, name, False, menu))
             self.loadMenu(iters, menu)
 
     def loadItems(self, menu):
         self.item_store.clear()
         for item, show in self.editor.getItems(menu):
-            icon = util.getIcon(item)
-            if isinstance(item, GMenu.TreeDirectory):
+            icon = util.getIcon(item, self.main_window)
+            if isinstance(item, CMenu.TreeDirectory):
                 name = item.get_name()
-            elif isinstance(item, GMenu.TreeEntry):
+            elif isinstance(item, CMenu.TreeEntry):
                 name = item.get_app_info().get_display_name()
-            elif isinstance(item, GMenu.TreeSeparator):
+            elif isinstance(item, CMenu.TreeSeparator):
                 name = '---'
             else:
                 assert False, 'should not be reached'
@@ -249,26 +262,26 @@ class MainWindow(object):
         menu_tree = self.tree.get_object('menu_tree')
         menus, iter = menu_tree.get_selection().get_selected()
         if not iter:
-            parent = menus[(0,)][2]
+            parent = menus[(0,)][3]
             menu_tree.expand_to_path((0,))
             menu_tree.get_selection().select_path((0,))
         else:
-            parent = menus[iter][2]
+            parent = menus[iter][3]
         file_path = os.path.join(util.getUserDirectoryPath(), util.getUniqueFileId('alacarte-made', '.directory'))
-        process = subprocess.Popen(['gnome-desktop-item-edit', file_path], env=os.environ)
+        process = subprocess.Popen(['cinnamon-desktop-editor', '-mdirectory', '-o' + file_path], env=os.environ)
         GObject.timeout_add(100, self.waitForNewMenuProcess, process, parent.get_menu_id(), file_path)
 
     def on_new_item_button_clicked(self, button):
         menu_tree = self.tree.get_object('menu_tree')
         menus, iter = menu_tree.get_selection().get_selected()
         if not iter:
-            parent = menus[(0,)][2]
+            parent = menus[(0,)][3]
             menu_tree.expand_to_path((0,))
             menu_tree.get_selection().select_path((0,))
         else:
-            parent = menus[iter][2]
+            parent = menus[iter][3]
         file_path = os.path.join(util.getUserItemPath(), util.getUniqueFileId('alacarte-made', '.desktop'))
-        process = subprocess.Popen(['gnome-desktop-item-edit', file_path], env=os.environ)
+        process = subprocess.Popen(['cinnamon-desktop-editor', '-mlauncher', '-o' + file_path], env=os.environ)
         GObject.timeout_add(100, self.waitForNewItemProcess, process, parent.get_menu_id(), file_path)
 
     def on_edit_delete_activate(self, menu):
@@ -277,38 +290,74 @@ class MainWindow(object):
         if not iter:
             return
         item = items[iter][3]
-        if isinstance(item, GMenu.TreeEntry):
+        if isinstance(item, CMenu.TreeEntry):
             self.editor.deleteItem(item)
-        elif isinstance(item, GMenu.TreeDirectory):
+        elif isinstance(item, CMenu.TreeDirectory):
             self.editor.deleteMenu(item)
-        elif isinstance(item, GMenu.TreeSeparator):
+        elif isinstance(item, CMenu.TreeSeparator):
             self.editor.deleteSeparator(item)
 
     def on_edit_properties_activate(self, menu):
-        item_tree = self.tree.get_object('item_tree')
+        item_tree = self.tree.get_object(self.last_tree)
         items, iter = item_tree.get_selection().get_selected()
         if not iter:
             return
         item = items[iter][3]
-        if not isinstance(item, GMenu.TreeEntry) and not isinstance(item, GMenu.TreeDirectory):
+        if not isinstance(item, CMenu.TreeEntry) and not isinstance(item, CMenu.TreeDirectory):
             return
 
-        if isinstance(item, GMenu.TreeEntry):
+        if isinstance(item, CMenu.TreeEntry):
             file_path = os.path.join(util.getUserItemPath(), item.get_desktop_file_id())
-            file_type = 'Item'
-        elif isinstance(item, GMenu.TreeDirectory):
+            file_type = 'launcher'
+        elif isinstance(item, CMenu.TreeDirectory):
             file_path = os.path.join(util.getUserDirectoryPath(), os.path.split(item.get_desktop_file_path())[1])
-            file_type = 'Menu'
+            file_type = 'directory'
 
         if not os.path.isfile(file_path):
-            data = open(item.get_desktop_file_path()).read()
-            open(file_path, 'w').write(data)
+            shutil.copy(item.get_desktop_file_path(), file_path)
 
         if file_path not in self.edit_pool:
             self.edit_pool.append(file_path)
-            process = subprocess.Popen(['gnome-desktop-item-edit', file_path], env=os.environ)
+            process = subprocess.Popen(['cinnamon-desktop-editor', '-m' + file_type, '-o' + file_path], env=os.environ)
             GObject.timeout_add(100, self.waitForEditProcess, process, file_path)
 
+    def on_edit_cut_activate(self, menu):
+        item_tree = self.tree.get_object('item_tree')
+        items, iter = item_tree.get_selection().get_selected()
+        item = items[iter][3]
+        if not iter:
+            return
+        if not isinstance(item, CMenu.TreeEntry):
+            return
+        (self.cut_copy_buffer, self.file_id) = self.editor.cutItem(item)
+
+    def on_edit_copy_activate(self, menu):
+        item_tree = self.tree.get_object('item_tree')
+        items, iter = item_tree.get_selection().get_selected()
+        item = items[iter][3]
+        if not iter:
+            return
+        if not isinstance(item, CMenu.TreeEntry):
+            return
+        (self.cut_copy_buffer, self.file_id) = self.editor.copyItem(item)
+
+    def on_edit_paste_activate(self, menu):
+        item_tree = self.tree.get_object('item_tree')
+        items, iter = item_tree.get_selection().get_selected()
+        if not iter:
+            menu_tree = self.tree.get_object('menu_tree')
+            items, iter = menu_tree.get_selection().get_selected()
+            if not iter:
+                return
+        item = items[iter][3]
+        if not isinstance(item, CMenu.TreeDirectory):
+            return
+        if self.cut_copy_buffer is not None:
+            success = self.editor.pasteItem(self.cut_copy_buffer, item, self.file_id)
+            if success:
+                self.cut_copy_buffer = None
+                self.file_id = None
+
     def on_menu_tree_cursor_changed(self, treeview):
         selection = treeview.get_selection()
         if selection is None:
@@ -316,20 +365,38 @@ class MainWindow(object):
         menus, iter = selection.get_selected()
         if iter is None:
             return
+        menu = menus[iter][3]
+
         menu_path = menus.get_path(iter)
+
         item_tree = self.tree.get_object('item_tree')
         item_tree.get_selection().unselect_all()
-        self.loadItems(self.menu_store[menu_path][2])
+        self.loadItems(self.menu_store[menu_path][3])
         self.tree.get_object('edit_delete').set_sensitive(False)
-        self.tree.get_object('edit_properties').set_sensitive(False)
-        self.tree.get_object('move_up_button').set_sensitive(False)
-        self.tree.get_object('move_down_button').set_sensitive(False)
-        self.tree.get_object('properties_button').set_sensitive(False)
+        self.tree.get_object('edit_properties').set_sensitive(True)
+        self.tree.get_object('edit_cut').set_sensitive(False)
+        self.tree.get_object('edit_copy').set_sensitive(False)
+        self.tree.get_object('properties_button').set_sensitive(True)
         self.tree.get_object('delete_button').set_sensitive(False)
+        self.tree.get_object('cut_button').set_sensitive(False)
+        self.tree.get_object('copy_button').set_sensitive(False)
+
+        can_paste = isinstance(menu, CMenu.TreeDirectory) and self.cut_copy_buffer is not None
+        self.tree.get_object('edit_paste').set_sensitive(can_paste)
+        self.tree.get_object('paste_button').set_sensitive(can_paste)
+
+        index = menus.get_path(iter).get_indices()[menus.get_path(iter).get_depth() - 1]
+        parent_iter = menus.iter_parent(iter)
+        count =  menus.iter_n_children(parent_iter)
+        can_go_up = index > 0 and isinstance(menu, CMenu.TreeDirectory)
+        can_go_down = index < count - 1 and isinstance(menu, CMenu.TreeDirectory)
+        self.tree.get_object('move_up_button').set_sensitive(can_go_up)
+        self.tree.get_object('move_down_button').set_sensitive(can_go_down)
+        self.last_tree = "menu_tree"
 
     def on_item_tree_show_toggled(self, cell, path):
         item = self.item_store[path][3]
-        if isinstance(item, GMenu.TreeSeparator):
+        if isinstance(item, CMenu.TreeSeparator):
             return
         if self.item_store[path][0]:
             self.editor.setVisible(item, False)
@@ -349,15 +416,27 @@ class MainWindow(object):
         self.tree.get_object('edit_delete').set_sensitive(True)
         self.tree.get_object('delete_button').set_sensitive(True)
 
-        can_edit = not isinstance(item, GMenu.TreeSeparator)
+        can_edit = not isinstance(item, CMenu.TreeSeparator)
         self.tree.get_object('edit_properties').set_sensitive(can_edit)
         self.tree.get_object('properties_button').set_sensitive(can_edit)
 
+        can_cut_copy = not isinstance(item, CMenu.TreeDirectory)
+        self.tree.get_object('cut_button').set_sensitive(can_cut_copy)
+        self.tree.get_object('copy_button').set_sensitive(can_cut_copy)
+        self.tree.get_object('edit_cut').set_sensitive(can_cut_copy)
+        self.tree.get_object('edit_copy').set_sensitive(can_cut_copy)
+
+        can_paste = isinstance(item, CMenu.TreeDirectory) and self.cut_copy_buffer is not None
+
+        self.tree.get_object('edit_paste').set_sensitive(can_paste)
+        self.tree.get_object('paste_button').set_sensitive(can_paste)
+
         index = items.get_path(iter).get_indices()[0]
-        can_go_up = index > 0 and isinstance(item, GMenu.TreeDirectory)
-        can_go_down = index < len(items) - 1 and isinstance(item, GMenu.TreeDirectory)
+        can_go_up = index > 0 and isinstance(item, CMenu.TreeDirectory)
+        can_go_down = index < len(items) - 1 and isinstance(item, CMenu.TreeDirectory)
         self.tree.get_object('move_up_button').set_sensitive(can_go_up)
         self.tree.get_object('move_down_button').set_sensitive(can_go_down)
+        self.last_tree = "item_tree"
 
     def on_item_tree_row_activated(self, treeview, path, column):
         self.on_edit_properties_activate(None)
@@ -386,34 +465,53 @@ class MainWindow(object):
         #without this shift-f10 won't work
         return True
 
+    def on_menu_tree_popup_menu(self, menu_tree, event=None):
+        model, iter = menu_tree.get_selection().get_selected()
+        if event:
+            #don't show if it's not the right mouse button
+            if event.button != 3:
+                return
+            button = event.button
+            event_time = event.time
+            info = menu_tree.get_path_at_pos(int(event.x), int(event.y))
+            if info is not None:
+                path, col, cellx, celly = info
+                menu_tree.grab_focus()
+                menu_tree.set_cursor(path, col, 0)
+        else:
+            path = model.get_path(iter)
+            button = 0
+            event_time = 0
+            menu_tree.grab_focus()
+            menu_tree.set_cursor(path, menu_tree.get_columns()[0], 0)
+        popup = self.tree.get_object('edit_menu')
+        popup.popup(None, None, None, None, button, event_time)
+        #without this shift-f10 won't work
+        return True
+
     def on_item_tree_key_press_event(self, item_tree, event):
         if event.keyval == Gdk.KEY_Delete:
             self.on_edit_delete_activate(item_tree)
 
     def on_move_up_button_clicked(self, button):
-        item_tree = self.tree.get_object('item_tree')
+        item_tree = self.tree.get_object(self.last_tree)
         items, iter = item_tree.get_selection().get_selected()
         if not iter:
             return
-        path = items.get_path(iter)
-        #at top, can't move up
-        if path.get_indices()[0] == 0:
-            return
-        item = items[path][3]
-        before = items[(path.get_indices()[0] - 1,)][3]
+
+        item = items[iter][3]
+        before = items[items.iter_previous(iter)][3]
+
         self.editor.moveItem(item.get_parent(), item, before=before)    
 
     def on_move_down_button_clicked(self, button):
-        item_tree = self.tree.get_object('item_tree')
+        item_tree = self.tree.get_object(self.last_tree)
         items, iter = item_tree.get_selection().get_selected()
         if not iter:
             return
-        path = items.get_path(iter)
-        #at bottom, can't move down
-        if path.get_indices()[0] == (len(items) - 1):
-            return
-        item = items[path][3]
-        after = items[path][3]
+
+        item = items[iter][3]
+        after = items[iter][3]
         self.editor.moveItem(item.get_parent(), item, after=after)
 
     def on_restore_button_clicked(self, button):
@@ -424,8 +522,18 @@ class MainWindow(object):
 
     def on_properties_button_clicked(self, button):
         self.on_edit_properties_activate(None)
+
     def on_delete_button_clicked(self, button):
         self.on_edit_delete_activate(None)
 
+    def on_cut_button_clicked(self, button):
+        self.on_edit_cut_activate(None)
+
+    def on_copy_button_clicked(self, button):
+        self.on_edit_copy_activate(None)
+
+    def on_paste_button_clicked(self, button):
+        self.on_edit_paste_activate(None)
+
     def quit(self):
         Gtk.main_quit()
