@@ -19,14 +19,14 @@
 import os
 import xml.dom.minidom
 import xml.parsers.expat
-from gi.repository import GMenu, GLib
+from gi.repository import CMenu, GLib
 from cme import util
 
 class MenuEditor(object):
     def __init__(self, name='cinnamon-applications.menu'):
         self.name = name
 
-        self.tree = GMenu.Tree.new(name, GMenu.TreeFlags.SHOW_EMPTY|GMenu.TreeFlags.INCLUDE_EXCLUDED|GMenu.TreeFlags.INCLUDE_NODISPLAY|GMenu.TreeFlags.SHOW_ALL_SEPARATORS|GMenu.TreeFlags.SORT_DISPLAY_NAME)
+        self.tree = CMenu.Tree.new(name, CMenu.TreeFlags.SHOW_EMPTY|CMenu.TreeFlags.INCLUDE_EXCLUDED|CMenu.TreeFlags.INCLUDE_NODISPLAY|CMenu.TreeFlags.SHOW_ALL_SEPARATORS|CMenu.TreeFlags.SORT_DISPLAY_NAME)
         self.tree.connect('changed', self.menuChanged)
         self.load()
 
@@ -66,11 +66,11 @@ class MenuEditor(object):
     def restoreTree(self, menu):
         item_iter = menu.iter()
         item_type = item_iter.next()
-        while item_type != GMenu.TreeItemType.INVALID:
-            if item_type == GMenu.TreeItemType.DIRECTORY:
+        while item_type != CMenu.TreeItemType.INVALID:
+            if item_type == CMenu.TreeItemType.DIRECTORY:
                 item = item_iter.get_directory()
                 self.restoreTree(item)
-            elif item_type == GMenu.TreeItemType.ENTRY:
+            elif item_type == CMenu.TreeItemType.ENTRY:
                 item = item_iter.get_entry()
                 self.restoreItem(item)
             item_type = item_iter.next()
@@ -106,8 +106,8 @@ class MenuEditor(object):
 
         item_iter = parent.iter()
         item_type = item_iter.next()
-        while item_type != GMenu.TreeItemType.INVALID:
-            if item_type == GMenu.TreeItemType.DIRECTORY:
+        while item_type != CMenu.TreeItemType.INVALID:
+            if item_type == CMenu.TreeItemType.DIRECTORY:
                 item = item_iter.get_directory()
                 yield (item, self.isVisible(item))
             item_type = item_iter.next()
@@ -117,17 +117,17 @@ class MenuEditor(object):
         item_iter = item.iter()
         item_type = item_iter.next()
 
-        while item_type != GMenu.TreeItemType.INVALID:
+        while item_type != CMenu.TreeItemType.INVALID:
             item = None
-            if item_type == GMenu.TreeItemType.DIRECTORY:
+            if item_type == CMenu.TreeItemType.DIRECTORY:
                 item = item_iter.get_directory()
-            elif item_type == GMenu.TreeItemType.ENTRY:
+            elif item_type == CMenu.TreeItemType.ENTRY:
                 item = item_iter.get_entry()
-            elif item_type == GMenu.TreeItemType.HEADER:
+            elif item_type == CMenu.TreeItemType.HEADER:
                 item = item_iter.get_header()
-            elif item_type == GMenu.TreeItemType.ALIAS:
+            elif item_type == CMenu.TreeItemType.ALIAS:
                 item = item_iter.get_alias()
-            elif item_type == GMenu.TreeItemType.SEPARATOR:
+            elif item_type == CMenu.TreeItemType.SEPARATOR:
                 item = item_iter.get_separator()
             if item:
                 contents.append(item)
@@ -137,28 +137,28 @@ class MenuEditor(object):
     def getItems(self, menu):
         item_iter = menu.iter()
         item_type = item_iter.next()
-        while item_type != GMenu.TreeItemType.INVALID:
+        while item_type != CMenu.TreeItemType.INVALID:
             item = None
-            if item_type == GMenu.TreeItemType.ENTRY:
+            if item_type == CMenu.TreeItemType.ENTRY:
                 item = item_iter.get_entry()
-            elif item_type == GMenu.TreeItemType.DIRECTORY:
+            elif item_type == CMenu.TreeItemType.DIRECTORY:
                 item = item_iter.get_directory()
-            elif item_type == GMenu.TreeItemType.HEADER:
+            elif item_type == CMenu.TreeItemType.HEADER:
                 item = item_iter.get_header()
-            elif item_type == GMenu.TreeItemType.ALIAS:
+            elif item_type == CMenu.TreeItemType.ALIAS:
                 item = item_iter.get_alias()
-            elif item_type == GMenu.TreeItemType.SEPARATOR:
+            elif item_type == CMenu.TreeItemType.SEPARATOR:
                 item = item_iter.get_separator()
             yield (item, self.isVisible(item))
             item_type = item_iter.next()
 
     def canRevert(self, item):
-        if isinstance(item, GMenu.TreeEntry):
+        if isinstance(item, CMenu.TreeEntry):
             if util.getItemPath(item.get_desktop_file_id()) is not None:
                 path = util.getUserItemPath()
                 if os.path.isfile(os.path.join(path, item.get_desktop_file_id())):
                     return True
-        elif isinstance(item, GMenu.TreeDirectory):
+        elif isinstance(item, CMenu.TreeDirectory):
             if item.get_desktop_file_path():
                 file_id = os.path.split(item.get_desktop_file_path())[1]
             else:
@@ -171,7 +171,7 @@ class MenuEditor(object):
 
     def setVisible(self, item, visible):
         dom = self.dom
-        if isinstance(item, GMenu.TreeEntry):
+        if isinstance(item, CMenu.TreeEntry):
             menu_xml = self.getXmlMenu(self.getPath(item.get_parent()), dom.documentElement, dom)
             if visible:
                 self.addXmlFilename(menu_xml, dom, item.get_desktop_file_id(), 'Include')
@@ -179,11 +179,11 @@ class MenuEditor(object):
             else:
                 self.addXmlFilename(menu_xml, dom, item.get_desktop_file_id(), 'Exclude')
             self.addXmlTextElement(menu_xml, 'AppDir', util.getUserItemPath(), dom)
-        elif isinstance(item, GMenu.TreeDirectory):
+        elif isinstance(item, CMenu.TreeDirectory):
             item_iter = item.iter()
             first_child_type = item_iter.next()
             #don't mess with it if it's empty
-            if first_child_type == GMenu.TreeItemType.INVALID:
+            if first_child_type == CMenu.TreeItemType.INVALID:
                 return
             menu_xml = self.getXmlMenu(self.getPath(item), dom.documentElement, dom)
             for node in self.getXmlNodesByName(['Deleted', 'NotDeleted'], menu_xml):
@@ -241,32 +241,46 @@ class MenuEditor(object):
             self.addXmlTextElement(menu_xml, 'DirectoryDir', util.getUserDirectoryPath(), dom)
         self.save()
 
-    def copyItem(self, item, new_parent, before=None, after=None):
+    def copyItem(self, item):
         dom = self.dom
         file_path = item.get_desktop_file_path()
-        keyfile = GLib.KeyFile()
-        keyfile.load_from_file(file_path, util.KEY_FILE_FLAGS)
+        copy_buffer = GLib.KeyFile()
+        copy_buffer.load_from_file(file_path, util.KEY_FILE_FLAGS)
+        return (copy_buffer, None)
 
-        util.fillKeyFile(keyfile, dict(Categories=[], Hidden=False))
+    def cutItem(self, item):
+        copy_buffer, file_id = self.copyItem(item)
+        file_id = self.deleteItem(item)
+        return (copy_buffer, file_id)
 
-        app_info = item.get_app_info()
-        file_id = util.getUniqueFileId(app_info.get_name().replace(os.sep, '-'), '.desktop')
-        out_path = os.path.join(util.getUserItemPath(), file_id)
-
-        contents, length = keyfile.to_data()
-
-        f = open(out_path, 'w')
-        f.write(contents)
-        f.close()
-
-        self.addItem(new_parent, file_id, dom)
-        self.positionItem(new_parent, ('Item', file_id), before, after)
-        self.save()
-        return file_id
+    def pasteItem(self, cut_copy_buffer, menu, file_id = None):
+        try:
+            path = self.getPath(menu)
+            util.fillKeyFile(cut_copy_buffer, dict(Hidden=False, NoDisplay=False))
+            name = util.getNameFromKeyFile(cut_copy_buffer)
+            if file_id is None:
+                file_id = util.getUniqueFileId(name.replace(os.sep, '-'), '.desktop')
+            out_path = os.path.join(util.getUserItemPath(), file_id)
+            contents, length = cut_copy_buffer.to_data()
+            f = open(out_path, 'w')
+            f.write(contents)
+            f.close()
+            menu_xml = self.getXmlMenu(path, self.dom.documentElement, self.dom)
+            self.addXmlFilename(menu_xml, self.dom, file_id, 'Include')
+            self.addXmlTextElement(menu_xml, 'AppDir', util.getUserItemPath(), self.dom)
+            self.save()
+            return True
+        except:
+            return False
 
     def deleteItem(self, item):
-        self.writeItem(item, Hidden=True)
+        file_id = self.writeItem(item, Hidden=True)
+        item_xml = self.getXmlMenu(self.getPath(item.get_parent()), self.dom.documentElement, self.dom)
+
+        self.removeXmlFilename(item_xml, self.dom, file_id)
+
         self.save()
+        return file_id
 
     def deleteMenu(self, menu):
         dom = self.dom
@@ -293,8 +307,8 @@ class MenuEditor(object):
 
         item_iter = parent.iter()
         item_type = item_iter.next()
-        while item_type != GMenu.TreeItemType.INVALID:
-            if item_type == GMenu.TreeItemType.DIRECTORY:
+        while item_type != CMenu.TreeItemType.INVALID:
+            if item_type == CMenu.TreeItemType.DIRECTORY:
                 item = item_iter.get_directory()
                 if item.get_menu_id() == menu_id:
                     return item
@@ -304,10 +318,10 @@ class MenuEditor(object):
             item_type = item_iter.next()
 
     def isVisible(self, item):
-        if isinstance(item, GMenu.TreeEntry):
+        if isinstance(item, CMenu.TreeEntry):
             app_info = item.get_app_info()
             return not (item.get_is_excluded() or app_info.get_nodisplay())
-        elif isinstance(item, GMenu.TreeDirectory):
+        elif isinstance(item, CMenu.TreeDirectory):
             return not item.get_is_nodisplay()
         return True
 
@@ -315,7 +329,10 @@ class MenuEditor(object):
         names = []
         current = menu
         while current is not None:
-            names.append(current.get_menu_id())
+            try:
+                names.append(current.get_menu_id())
+            except:
+                names.append(current.get_desktop_file_id())
             current = current.get_parent()
 
         # XXX - don't append root menu name, alacarte doesn't
@@ -365,6 +382,11 @@ class MenuEditor(object):
         node.appendChild(self.addXmlTextElement(node, 'Filename', filename, dom))
         return element.appendChild(node)
 
+    def removeXmlFilename(self, element, dom, filename):
+        for node in self.getXmlNodesByName(['Include'], element):
+            if node.childNodes[0].nodeName == 'Filename' and node.childNodes[0].childNodes[0].nodeValue == filename:
+                element.removeChild(node)
+
     def addDeleted(self, element, dom):
         node = dom.createElement('Deleted')
         return element.appendChild(node)
@@ -474,11 +496,11 @@ class MenuEditor(object):
         layout = []
         layout.append(('Merge', 'menus'))
         for item in items:
-            if isinstance(item, GMenu.TreeDirectory):
+            if isinstance(item, CMenu.TreeDirectory):
                 layout.append(('Menuname', item.get_menu_id()))
-            elif isinstance(item, GMenu.TreeEntry):
+            elif isinstance(item, CMenu.TreeEntry):
                 layout.append(('Filename', item.get_desktop_file_id()))
-            elif isinstance(item, GMenu.TreeSeparator):
+            elif isinstance(item, CMenu.TreeSeparator):
                 layout.append(('Separator',))
             else:
                 layout.append(item)
@@ -495,14 +517,14 @@ class MenuEditor(object):
 
     def getIndex(self, item, contents):
         index = -1
-        if isinstance(item, GMenu.TreeDirectory):
+        if isinstance(item, CMenu.TreeDirectory):
             for i in range(len(contents)):
                 if type(item) is not type(contents[i]):
                     continue
                 if item.get_menu_id() == contents[i].get_menu_id():
                     index = i
                     return index
-        elif isinstance(item, GMenu.TreeEntry):
+        elif isinstance(item, CMenu.TreeEntry):
             for i in range(len(contents)):
                 if type(item) is not type(contents[i]):
                     continue
