@@ -29,7 +29,7 @@ except Exception, detail:
     sys.exit(1)
 
 class SidePage:
-    def __init__(self, name, icon, keywords, advanced, content_box, is_c_mod = False, is_standalone = False, exec_name = None):
+    def __init__(self, name, icon, keywords, content_box = None, size = None, is_c_mod = False, is_standalone = False, exec_name = None, module=None):
         self.name = name
         self.icon = icon
         self.content_box = content_box
@@ -37,32 +37,39 @@ class SidePage:
         self.is_c_mod = is_c_mod
         self.is_standalone = is_standalone
         self.exec_name = exec_name
+        self.module = module # Optionally set by the module so we can call on_module_selected() on it when we show it.
         self.keywords = keywords
-        self.advanced = advanced
+        self.size = size
         self.topWindow = None
         self.builder = None
+        if self.module != None:
+            self.module.loaded = False
 
-    def add_widget(self, widget, advanced = False):
-        self.widgets.append(widget)
-        widget.advanced = advanced
+    def add_widget(self, widget):
+        self.widgets.append(widget)        
 
-    def build(self, mode_advanced):
+    def build(self):        
         # Clear all the widgets from the content box
         widgets = self.content_box.get_children()
         for widget in widgets:
             self.content_box.remove(widget)
+
+        if (self.module is not None):
+            self.module.on_module_selected()
+            self.module.loaded = True
+
         # Add our own widgets
         # C modules are sort of messy - they check the desktop type
         # (for Unity or GNOME) and show/hide UI items depending on
-        # the result - so we can't just show_all on the widget, it will
+        # the result - so we cannot just show_all on the widget, it will
         # mess up these modifications - so for these, we just show the
         # top-level widget
         if not self.is_standalone:
             for widget in self.widgets:
-                if widget.advanced:
-                    if not mode_advanced:
-                        continue
-                self.content_box.pack_start(widget, False, False, 2)
+                if hasattr(widget, 'expand'):
+                    self.content_box.pack_start(widget, True, True, 2)
+                else:
+                    self.content_box.pack_start(widget, False, False, 2)
             if self.is_c_mod:
                 self.content_box.show()
                 children = self.content_box.get_children()
@@ -80,12 +87,16 @@ class SidePage:
                                 c_widget.show()
             else:
                 self.content_box.show_all()
+                try:
+                    self.check_third_arg()
+                except:
+                    pass
         else:
             subprocess.Popen(self.exec_name.split())
 
 class CCModule:
-    def __init__(self, label, mod_id, icon, category, advanced, keywords, content_box):
-        sidePage = SidePage(label, icon, keywords, advanced, content_box, True, False, mod_id)
+    def __init__(self, label, mod_id, icon, category, keywords, content_box):
+        sidePage = SidePage(label, icon, keywords, content_box, False, True, False, mod_id)
         self.sidePage = sidePage
         self.name = mod_id
         self.category = category
@@ -101,14 +112,16 @@ class CCModule:
             return False
 
 class SAModule:
-    def __init__(self, label, mod_id, icon, category, advanced, keywords, content_box):
-        sidePage = SidePage(label, icon, keywords, advanced, content_box, False, True, mod_id)
+    def __init__(self, label, mod_id, icon, category, keywords, content_box):
+        sidePage = SidePage(label, icon, keywords, content_box, False, False, True, mod_id)
         self.sidePage = sidePage
         self.name = mod_id
         self.category = category
 
     def process (self):
-        return fileexists(self.name.split()[0])
+        name = self.name.replace("gksudo ", "")
+        name = name.replace("gksu ", "")
+        return fileexists(name.split()[0])
 
 def fileexists(program):
 
@@ -145,20 +158,73 @@ def rec_mkdir(path):
         return
     os.mkdir(path)
 
+class Section(Gtk.Box):
+    def __init__(self, name):
+        self.name = name
+        super(Section, self).__init__()
+        self.set_orientation(Gtk.Orientation.VERTICAL)
+        self.set_border_width(6)
+        self.set_spacing(6)
+        self.label = Gtk.Label()
+        self.label.set_markup("<b>%s</b>" % self.name)
+        hbox = Gtk.Box()
+        hbox.set_orientation(Gtk.Orientation.HORIZONTAL)
+        hbox.pack_start(self.label, False, False, 0)
+        self.pack_start(hbox, False, True, 0)
+
+    def add(self, widget):
+        box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
+        box.set_margin_left(40)
+        box.set_margin_right(40)
+        box.pack_start(widget, False, True, 0)
+        self.pack_start(box, False, False, 0)
+
+    def add_expand(self, widget):
+        box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
+        box.set_margin_left(40)
+        box.set_margin_right(40)
+        box.pack_start(widget, True, True, 0)
+        self.pack_start(box, False, False, 0)
+
+    def add_indented(self, widget):
+        box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
+        box.set_margin_left(80)
+        box.set_margin_right(10)
+        box.pack_start(widget, False, True, 0)
+        self.pack_start(box, False, False, 0)
+
+    def add_indented_expand(self, widget):
+        box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
+        box.set_margin_left(80)
+        box.set_margin_right(10)
+        box.pack_start(widget, True, True, 0)
+        self.pack_start(box, False, False, 0)
+
+class SectionBg(Gtk.Viewport):
+    def __init__(self):
+        Gtk.Viewport.__init__(self)
+        self.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
+        style = self.get_style_context()
+        style.add_class("section-bg")
+        self.expand = True # Tells CS to give expand us to the whole window
+
 class IndentedHBox(Gtk.HBox):
     def __init__(self):
         super(IndentedHBox, self).__init__()
-        indent = Gtk.Label('\t')
+        indent = Gtk.Label.new('\t')
         self.pack_start(indent, False, False, 0)
 
     def add(self, item):
-        self.pack_start(item, False, False, 0)
+        self.pack_start(item, False, True, 0)
+
+    def add_expand(self, item):
+        self.pack_start(item, True, True, 0)
 
 class GSettingsCheckButton(Gtk.CheckButton):    
     def __init__(self, label, schema, key, dep_key):
         self.key = key
         self.dep_key = dep_key
-        super(GSettingsCheckButton, self).__init__(label)
+        super(GSettingsCheckButton, self).__init__(label = label)
         self.settings = Gio.Settings.new(schema)        
         self.set_active(self.settings.get_boolean(self.key))
         self.settings.connect("changed::"+self.key, self.on_my_setting_changed)
@@ -195,9 +261,9 @@ class GSettingsSpinButton(Gtk.HBox):
         self.max = max
         self.dep_key = dep_key
         super(GSettingsSpinButton, self).__init__()        
-        self.label = Gtk.Label(label)       
+        self.label = Gtk.Label.new(label)       
         self.content_widget = Gtk.SpinButton()
-        self.units = Gtk.Label(units)        
+        self.units = Gtk.Label.new(units)        
         if (label != ""):       
             self.pack_start(self.label, False, False, 2)
         self.pack_start(self.content_widget, False, False, 2)
@@ -257,7 +323,7 @@ class GSettingsEntry(Gtk.HBox):
         self.key = key
         self.dep_key = dep_key
         super(GSettingsEntry, self).__init__()
-        self.label = Gtk.Label(label)       
+        self.label = Gtk.Label.new(label)       
         self.content_widget = Gtk.Entry()
         self.pack_start(self.label, False, False, 5)        
         self.add(self.content_widget)     
@@ -294,25 +360,27 @@ class GSettingsFileChooser(Gtk.HBox):
     def __init__(self, label, schema, key, dep_key, show_none_cb = False):
         self.key = key
         self.dep_key = dep_key
+        self.show_none_cb = show_none_cb
         super(GSettingsFileChooser, self).__init__()
-        self.label = Gtk.Label(label)       
+        self.label = Gtk.Label.new(label)       
         self.content_widget = Gtk.FileChooserButton()
         self.pack_start(self.label, False, False, 2)
         self.add(self.content_widget)
         self.settings = Gio.Settings.new(schema)
         value = self.settings.get_string(self.key)     
-        if show_none_cb:
-            self.show_none_cb = Gtk.CheckButton(_("None"))
-            self.show_none_cb.set_active(value=="")
-            self.pack_start(self.show_none_cb, False, False, 5)
-        else:
-            self.show_none_cb = None
-        if value=="":
-            self.content_widget.set_sensitive(False)
-        else:
+        if value != "":
             self.content_widget.set_filename(value)
+
+        if self.show_none_cb:
+            self.show_none_cb_widget = Gtk.CheckButton(_("None"))
+            self.show_none_cb_widget.set_active(value=="")
+            self.pack_start(self.show_none_cb_widget, False, False, 5)        
+            if value=="":
+                self.content_widget.set_sensitive(False)
+        
         self.content_widget.connect('file-set', self.on_my_value_changed)
-        self.show_none_cb.connect('toggled', self.on_my_value_changed)
+        if self.show_none_cb:
+            self.show_none_cb_widget.connect('toggled', self.on_my_value_changed)
         self.content_widget.show_all()
         self.dependency_invert = False
         if self.dep_key is not None:
@@ -326,17 +394,22 @@ class GSettingsFileChooser(Gtk.HBox):
             self.on_dependency_setting_changed(self, None)
 
     def on_my_value_changed(self, widget):
-        if self.show_none_cb.get_active():
-            value = ""
-            self.content_widget.set_sensitive(False)
+        if self.show_none_cb:
+            if self.show_none_cb_widget.get_active():
+                value = ""
+                self.content_widget.set_sensitive(False)
+            else:
+                value = self.content_widget.get_filename()
+                if value==None:
+                    value = ""
+                self.content_widget.set_sensitive(True)
         else:
             value = self.content_widget.get_filename()
             if value==None:
                 value = ""
-            self.content_widget.set_sensitive(True)
         self.settings.set_string(self.key, value)
 
-    def on_dependency_setting_changed(self, settings, dep_key):
+    def on_dependency_setting_changed(self, settings, dep_key):        
         if not self.dependency_invert:
             self.set_sensitive(self.dep_settings.get_boolean(self.dep_key))
         else:
@@ -350,7 +423,7 @@ class GSettingsFontButton(Gtk.HBox):
         self.settings = Gio.Settings.new(schema)
         self.value = self.settings.get_string(key)
         
-        self.label = Gtk.Label(label)
+        self.label = Gtk.Label.new(label)
 
         self.content_widget = Gtk.FontButton()
         self.content_widget.set_font_name(self.value)
@@ -388,7 +461,7 @@ class GSettingsFontButton(Gtk.HBox):
 #        self.settings = Gio.Settings.new(schema)
 #        self.value = self.settings.get_double(self.key)
 #        
-#        self.label = Gtk.Label(label)
+#        self.label = Gtk.Label.new(label)
 #
 #        #returned variant is range:(min, max)
 #        _min, _max = self.settings.get_range(self.key)[1]
@@ -427,15 +500,22 @@ class GSettingsRange(Gtk.HBox):
         self.dep_key = dep_key
         self.settings = Gio.Settings.new(schema)
         self.valtype = valtype
+
         if self.valtype == "int":
             self.value = self.settings.get_int(self.key) * 1.0
         elif self.valtype == "uint":
             self.value = self.settings.get_uint(self.key) * 1.0
         elif self.valtype == "double":
             self.value = self.settings.get_double(self.key) * 1.0
-        self.label = Gtk.Label(label)
+        self.label = Gtk.Label.new(label)
+        self.label.set_alignment(1.0, 0.5)
+        self.label.set_size_request(150, -1)
         self.low_label = Gtk.Label()
+        self.low_label.set_alignment(0.5, 0.5)
+        self.low_label.set_size_request(60, -1)
         self.hi_label = Gtk.Label()
+        self.hi_label.set_alignment(0.5, 0.5)
+        self.hi_label.set_size_request(60, -1)
         self.low_label.set_markup("<i><small>%s</small></i>" % low_label)
         self.hi_label.set_markup("<i><small>%s</small></i>" % hi_label)
         self.inverted = inverted
@@ -445,19 +525,24 @@ class GSettingsRange(Gtk.HBox):
         self._min = low_limit * 1.0
         self._max = hi_limit * 1.0
         self.content_widget = Gtk.Scale.new_with_range(Gtk.Orientation.HORIZONTAL, 0, 1, (self._step / self._range))
+        self.content_widget.set_size_request(300, 0)
         self.content_widget.set_value(self.to_corrected(self.value))
         self.content_widget.set_draw_value(False);
+
+        self.grid = Gtk.Grid()
         if (label != ""):
-            self.pack_start(self.label, False, False, 2)
+            self.grid.attach(self.label, 0, 0, 1, 1)
         if (low_label != ""):
-            self.pack_start(self.low_label, False, False, 2)
-        self.pack_start(self.content_widget, True, True, 2)
+            self.grid.attach(self.low_label, 1, 0, 1, 1)
+        self.grid.attach(self.content_widget, 2, 0, 1, 1)
         if (hi_label != ""):
-            self.pack_start(self.hi_label, False, False, 2)
+            self.grid.attach(self.hi_label, 3, 0, 1, 1)
+        self.pack_start(self.grid, True, True, 2)
         self._dragging = False
         self.content_widget.connect('value-changed', self.on_my_value_changed)
         self.content_widget.connect('button-press-event', self.on_mouse_down)
         self.content_widget.connect('button-release-event', self.on_mouse_up)
+        self.content_widget.connect("scroll-event", self.on_mouse_scroll_event)
         self.content_widget.show_all()
         self.dependency_invert = False
         if self.dep_key is not None:
@@ -481,6 +566,18 @@ class GSettingsRange(Gtk.HBox):
         self._dragging = False
         self.on_my_value_changed(widget)
 
+    def on_mouse_scroll_event(self, widget, event):
+        found, delta_x, delta_y = event.get_scroll_deltas()
+        if found:
+            add = delta_y < 0
+            uncorrected = self.from_corrected(widget.get_value())
+            if add:
+                corrected = self.to_corrected(uncorrected + self._step)
+            else:
+                corrected = self.to_corrected(uncorrected - self._step)
+            widget.set_value(corrected)
+        return True
+
     def on_my_value_changed(self, widget):
         if self._dragging:
             return
@@ -533,12 +630,15 @@ class GSettingsRange(Gtk.HBox):
                 result =  (value * self._range) + self._min
         return round(result)
 
+    def add_mark(self, value, position, markup):
+        self.content_widget.add_mark((value - self._min) / self._range, position, markup)
+
 class GSettingsRangeSpin(Gtk.HBox):
     def __init__(self, label, schema, key, dep_key, **options):
         self.key = key
         self.dep_key = dep_key
         super(GSettingsRangeSpin, self).__init__()
-        self.label = Gtk.Label(label)
+        self.label = Gtk.Label.new(label)
         self.content_widget = Gtk.SpinButton()
 
         if (label != ""):
@@ -570,7 +670,9 @@ class GSettingsRangeSpin(Gtk.HBox):
             self.on_dependency_setting_changed(self, None)
 
     def on_my_setting_changed(self, settings, key):
-        self.content_widget.set_value(self.settings.get_double(self.key))
+        value = self.settings.get_double(self.key)
+        if value != self.content_widget.get_value():
+            self.content_widget.set_value(value)
 
     def on_my_value_changed(self, widget):
         self.settings.set_double(self.key, self.content_widget.get_value())
@@ -589,7 +691,7 @@ class GSettingsComboBox(Gtk.HBox):
         self.settings = Gio.Settings.new(schema)        
         self.value = self.settings.get_string(self.key)
                       
-        self.label = Gtk.Label(label)       
+        self.label = Gtk.Label.new(label)       
         self.model = Gtk.ListStore(str, str)
         selected = None
         for option in options:
@@ -609,11 +711,10 @@ class GSettingsComboBox(Gtk.HBox):
         
         if (label != ""):
             self.pack_start(self.label, False, False, 2)                
-        self.pack_start(self.content_widget, False, False, 2)                     
+        self.pack_start(self.content_widget, True, True, 2)                     
         self.content_widget.connect('changed', self.on_my_value_changed)
         self.content_widget.show_all()
         self.dependency_invert = False
-        self.dependency_invert = False
         if self.dep_key is not None:
             if self.dep_key[0] == '!':
                 self.dependency_invert = True
@@ -637,13 +738,18 @@ class GSettingsComboBox(Gtk.HBox):
             self.set_sensitive(not self.dep_settings.get_boolean(self.dep_key))
 
 class GSettingsIntComboBox(Gtk.HBox):
-    def __init__(self, label, schema, key, options):
+    def __init__(self, label, schema, key, dep_key, options, use_uint=False):
         self.key = key
+        self.dep_key = dep_key
+        self.use_uint = use_uint
         super(GSettingsIntComboBox, self).__init__()
         self.settings = Gio.Settings.new(schema)
-        self.value = self.settings.get_int(self.key)
+        if self.use_uint:
+            self.value = self.settings.get_uint(self.key)
+        else:
+            self.value = self.settings.get_int(self.key)
 
-        self.label = Gtk.Label(label)
+        self.label = Gtk.Label.new(label)
         self.model = Gtk.ListStore(int, str)
         selected = None
         for option in options:
@@ -663,7 +769,63 @@ class GSettingsIntComboBox(Gtk.HBox):
 
         if (label != ""):
             self.pack_start(self.label, False, False, 2)
-        self.pack_start(self.content_widget, False, False, 2)
+        self.pack_start(self.content_widget, False, True, 2)
+        self.content_widget.connect('changed', self.on_my_value_changed)
+        self.content_widget.show_all()
+        self.dependency_invert = False
+        if self.dep_key is not None:
+            if self.dep_key[0] == '!':
+                self.dependency_invert = True
+                self.dep_key = self.dep_key[1:]
+            split = self.dep_key.split('/')
+            self.dep_settings = Gio.Settings.new(split[0])
+            self.dep_key = split[1]
+            self.dep_settings.connect("changed::"+self.dep_key, self.on_dependency_setting_changed)
+            self.on_dependency_setting_changed(self, None)
+
+    def on_my_value_changed(self, widget):
+        tree_iter = widget.get_active_iter()
+        if tree_iter != None:
+            value = self.model[tree_iter][0]
+            if self.use_uint:
+                self.settings.set_uint(self.key, value)
+            else:
+                self.settings.set_int(self.key, value)
+
+    def on_dependency_setting_changed(self, settings, dep_key):
+        if not self.dependency_invert:
+            self.set_sensitive(self.dep_settings.get_boolean(self.dep_key))
+        else:
+            self.set_sensitive(not self.dep_settings.get_boolean(self.dep_key))
+
+class GSettingsUIntComboBox(Gtk.HBox):
+    def __init__(self, label, schema, key, options):
+        self.key = key
+        super(GSettingsUIntComboBox, self).__init__()
+        self.settings = Gio.Settings.new(schema)
+        self.value = self.settings.get_uint(self.key)
+
+        self.label = Gtk.Label.new(label)
+        self.model = Gtk.ListStore(int, str)
+        selected = None
+        for option in options:
+            iter = self.model.insert_before(None, None)
+            self.model.set_value(iter, 0, option[0])
+            self.model.set_value(iter, 1, option[1])
+            if (option[0] == self.value):
+                selected = iter
+
+        self.content_widget = Gtk.ComboBox.new_with_model(self.model)
+        renderer_text = Gtk.CellRendererText()
+        self.content_widget.pack_start(renderer_text, True)
+        self.content_widget.add_attribute(renderer_text, "text", 1)
+
+        if selected is not None:
+            self.content_widget.set_active_iter(selected)
+
+        if (label != ""):
+            self.pack_start(self.label, False, False, 2)
+        self.pack_start(self.content_widget, False, True, 2)
         self.content_widget.connect('changed', self.on_my_value_changed)
         self.content_widget.show_all()
 
@@ -671,7 +833,8 @@ class GSettingsIntComboBox(Gtk.HBox):
         tree_iter = widget.get_active_iter()
         if tree_iter != None:
             value = self.model[tree_iter][0]
-            self.settings.set_int(self.key, value)
+            self.settings.set_uint(self.key, value)
+
 
 class GSettingsColorChooser(Gtk.ColorButton):
     def __init__(self, schema, key, dep_key):
@@ -715,7 +878,7 @@ class GSettingsColorChooser(Gtk.ColorButton):
 #         self.settings = gconf.client_get_default()
 #         self.value = self.settings.get_string(key)
         
-#         self.label = Gtk.Label(label)
+#         self.label = Gtk.Label.new(label)
 
 #         self.content_widget = Gtk.FontButton()
 #         self.content_widget.set_font_name(self.value)
@@ -737,7 +900,7 @@ class GSettingsColorChooser(Gtk.ColorButton):
 #         if not self.value:
 #             self.value = init_value
                       
-#         self.label = Gtk.Label(label)       
+#         self.label = Gtk.Label.new(label)       
 #         self.model = Gtk.ListStore(str, str)
 #         selected = None
 #         for option in options:
