@@ -21,6 +21,7 @@ setting_dict = {
     "header"          :   "Header", # Not a setting, just a boldface header text
     "separator"       :   "Separator", # not a setting, a horizontal separator
     "entry"           :   "Entry",
+    "textview"        :   "TextView",    
     "checkbox"        :   "CheckButton",
     "spinbutton"      :   "SpinButton",
     "filechooser"     :   "FileChooser",
@@ -29,14 +30,14 @@ setting_dict = {
     "colorchooser"    :   "ColorChooser",
     "radiogroup"      :   "RadioGroup",
     "iconfilechooser" :   "IconFileChooser",
+    "keybinding"      :   "Keybinding",
     "button"          :   "Button" # Not a setting, provides a button which triggers a callback in the applet/desklet
 }
 
-
 class Factory():
-    def __init__(self, file_name, instance_id, multi_instance):
+    def __init__(self, file_name, instance_id, multi_instance, uuid):
         self.file = file_name
-        self.settings = Settings(file_name, self, instance_id, multi_instance)
+        self.settings = Settings(file_name, self, instance_id, multi_instance, uuid)
         self.widgets = collections.OrderedDict()
         self.file_obj = Gio.File.new_for_path(self.file)
         self.file_monitor = self.file_obj.monitor_file(Gio.FileMonitorFlags.SEND_MOVED, None)
@@ -48,7 +49,7 @@ class Factory():
         try:
             self.widgets[key] = eval(setting_dict[setting_type])(key, self.settings, uuid)
         except Exception, detail:
-            print (_("Invalid setting type '%s' supplied - please check your json file for %s" % (setting_type, uuid)))
+            print ("Invalid setting type '%s' supplied - please check your json file for %s" % (setting_type, uuid))
             print detail
 
     def on_file_changed(self, file, other, event, data):
@@ -109,18 +110,30 @@ class Factory():
 
 
 class Settings():
-    def __init__(self, file_name, factory, instance_id, multi_instance):
+    def __init__(self, file_name, factory, instance_id, multi_instance, uuid):
         self.file_name = file_name
         self.factory = factory
         self.instance_id = instance_id
         self.multi_instance = multi_instance
+        self.uuid = uuid
+        try:
+            self.tUser = gettext.translation(self.uuid, home+"/.local/share/locale").ugettext
+        except IOError:
+            try:
+                self.tUser = gettext.translation(self.uuid, "/usr/share/locale").ugettext
+            except IOError:
+                self.tUser = None
+        try:
+            self.t = gettext.translation("cinnamon", "/usr/share/cinnamon/locale").ugettext
+        except IOError:
+            self.t = None
         self.reload()
 
     def reload (self):
         _file = open(self.file_name)
         raw_data = _file.read()
         self.data = {}
-        self.data = json.loads(raw_data, object_pairs_hook=collections.OrderedDict)
+        self.data = json.loads(raw_data.decode('utf-8'), object_pairs_hook=collections.OrderedDict)
         _file.close()
 
     def save (self, name = None):
@@ -140,24 +153,52 @@ class Settings():
     def get_key_exists(self, key):
         return key in self.data.keys()
 
+    def real_update_dbus(self, key):
+        self.factory.pause_monitor()
+        session_bus = dbus.SessionBus()
+        cinnamon_dbus = session_bus.get_object("org.Cinnamon", "/org/Cinnamon")
+        setter = cinnamon_dbus.get_dbus_method('updateSetting', 'org.Cinnamon')
+        payload = json.dumps(self.data[key])
+        setter(self.uuid, self.instance_id, key, payload)
+        self.factory.resume_monitor()
+
+    def try_update_dbus(self, key):
+        try:
+            self.real_update_dbus(key)
+        except Exception, e:
+            print "Cinnamon not running, falling back to python settings engine: ", e
+            self.save()
+
+    def try_update_dbus_foreach(self):
+        failed = False
+        for key in self.data.keys():
+            if "value" in self.data[key] and "default" in self.data[key]:
+                try:
+                    self.real_update_dbus(key)
+                except Exception, e:
+                    failed = True;
+                    print "Cinnamon not running, falling back to python settings engine: ", e
+        if failed:
+            self.save()
+
     def set_value(self, key, val):
         self.data[key]["value"] = val
-        self.save()
+        self.try_update_dbus(key)
 
     def set_custom_value(self, key, val):
         self.data[key]["last-custom-value"] = val
-        self.save()
+        self.try_update_dbus(key)
 
     def reset_to_defaults(self):
         for key in self.data.keys():
             if "value" in self.data[key] and "default" in self.data[key]:
                 self.data[key]["value"] = self.data[key]["default"]
-        self.save()
+        self.try_update_dbus_foreach()
 
     def load_from_file(self, filename):
         new_file = open(filename)
         new_raw = new_file.read()
-        new_json = json.loads(new_raw, object_pairs_hook=collections.OrderedDict)
+        new_json = json.loads(new_raw.decode('utf-8'), object_pairs_hook=collections.OrderedDict)
         new_file.close()
         copy = self.data
         if copy["__md5__"] != new_json["__md5__"]:
@@ -165,7 +206,7 @@ class Settings():
                                    None, 0,
                                   (Gtk.STOCK_NO, Gtk.ResponseType.NO,
                                    Gtk.STOCK_YES, Gtk.ResponseType.YES))
-            text = Gtk.Label(_("The MD5 tags for the file you are trying to import and the existing file do not match.\n"
+            text = Gtk.Label.new(_("The MD5 tags for the file you are trying to import and the existing file do not match.\n"
                              "This means the two files were generated by different versions of this applet, desklet or extension,\n"
                              "or possibly from a different one entirely.  Continuing with this procedure could yield unpredictable results.\n\n"
                              "Are you sure you want to proceed?"))
@@ -182,9 +223,17 @@ class Settings():
         self.save()
 
 
-class BaseWidget():
+class BaseWidget(object):
     def __init__(self, key, settings_obj, uuid):
         self.settings_obj = settings_obj
+        if self.settings_obj.tUser:
+            self.tUser = self.settings_obj.tUser
+        else:
+            self.tUser = None
+        if self.settings_obj.t:
+            self.t = self.settings_obj.t
+        else:
+            self.t = None
         self.key = key
         self.uuid = uuid
         self.handler = None
@@ -222,6 +271,14 @@ class BaseWidget():
 
     def get_desc(self):
         try:
+            if self.tUser:
+                result = self.tUser(self.settings_obj.get_data(self.key)["description"])
+                if result != self.settings_obj.get_data(self.key)["description"]:
+                    print result
+                    return result
+            if self.t:
+                print self.t(self.settings_obj.get_data(self.key)["description"])
+                return self.t(self.settings_obj.get_data(self.key)["description"])
             return self.settings_obj.get_data(self.key)["description"]
         except:
             print ("Could not find description for key '%s' in xlet '%s'" % (self.key, self.uuid))
@@ -229,15 +286,27 @@ class BaseWidget():
 
     def get_tooltip(self):
         try:
+            if self.tUser:
+                result = self.tUser(self.settings_obj.get_data(self.key)["tooltip"])
+                if result != self.settings_obj.get_data(self.key)["tooltip"]:
+                    return result
+            if self.t:
+                return self.t(self.settings_obj.get_data(self.key)["tooltip"])
             return self.settings_obj.get_data(self.key)["tooltip"]
         except:
             return ""
 
     def get_units(self):
         try:
+            if self.tUser:
+                result = self.tUser(self.settings_obj.get_data(self.key)["units"])
+                if result != self.settings_obj.get_data(self.key)["units"]:
+                    return result
+            if self.t:
+                return self.t(self.settings_obj.get_data(self.key)["units"])
             return self.settings_obj.get_data(self.key)["units"]
         except:
-            print ("Could not find description for key '%s' in xlet '%s'" % (self.key, self.uuid))
+            print ("Could not find units for key '%s' in xlet '%s'" % (self.key, self.uuid))
             return ""
 
     def get_val(self):
@@ -269,9 +338,26 @@ class BaseWidget():
 
     def get_options(self):
         try:
-            return self.settings_obj.get_data(self.key)["options"]
-        except:
+            if self.t or self.tUser:
+                ret = collections.OrderedDict()
+                d = self.settings_obj.get_data(self.key)["options"]
+                for key in d.keys():
+                    if self.tUser:
+                        translated_key = self.tUser(key)
+                        if translated_key != key or not self.t:
+                            ret[translated_key] = d[key]
+                        elif self.t:
+                            translated_key = self.t(key)
+                            ret[translated_key] = d[key]
+                    elif self.t:
+                        translated_key = self.t(key)
+                        ret[translated_key] = d[key]
+                return ret
+            else:
+                return self.settings_obj.get_data(self.key)["options"]
+        except Exception, detail:
             print ("Could not find options for key '%s' in xlet '%s'" % (self.key, self.uuid))
+            print detail
 
     def get_custom_val(self):
         try:
@@ -315,6 +401,12 @@ class BaseWidget():
             return self.settings_obj.get_data(self.key)["indent"]
         except:
             return False
+            
+    def get_height(self):
+        try:
+            return self.settings_obj.get_data(self.key)["height"]
+        except:
+            return 200            
 
 def set_tt(tt, *widgets):
     for widget in widgets:
@@ -324,7 +416,7 @@ def set_tt(tt, *widgets):
 class IndentedHBox(Gtk.HBox):
     def __init__(self):
         super(IndentedHBox, self).__init__()
-        indent = Gtk.Label('\t')
+        indent = Gtk.Label.new('\t')
         self.pack_start(indent, False, False, 0)
 
     def add(self, item):
@@ -354,21 +446,13 @@ class CheckButton(Gtk.CheckButton, BaseWidget):
         self.set_active(self.get_val())
         self.handler = self.connect('toggled', self.on_my_value_changed)
         set_tt(self.get_tooltip(), self)
-        self._value_changed_timer = None
 
     def add_dependent(self, widget):
         self.dependents.append(widget)
 
     def on_my_value_changed(self, widget):
-        if self._value_changed_timer:
-            GObject.source_remove(self._value_changed_timer)
-        self._value_changed_timer = GObject.timeout_add(300, self.update_settings_value)
-
-    def update_settings_value(self):
         self.set_val(self.get_active())
         self.update_dependents()
-        self._value_changed_timer = None
-        return False
 
     def update_dependents(self):
         for dep in self.dependents:
@@ -387,13 +471,13 @@ class SpinButton(Gtk.HBox, BaseWidget):
         BaseWidget.__init__(self, key, settings_obj, uuid)
         super(SpinButton, self).__init__()
 
-        self.label = Gtk.Label(self.get_desc())
+        self.label = Gtk.Label.new(self.get_desc())
         self.spinner = Gtk.SpinButton()
         try: # Guess if number is float and set digits to step size decimal places
             self.spinner.set_digits(len(str(self.get_step()).split(".")[1]))
         except:
             pass
-        self.units = Gtk.Label(self.get_units())
+        self.units = Gtk.Label.new(self.get_units())
 
         if self.get_desc() != "":
             self.pack_start(self.label, False, False, 2)
@@ -410,7 +494,7 @@ class SpinButton(Gtk.HBox, BaseWidget):
     def on_my_value_changed(self, widget):
         if self._value_changed_timer:
             GObject.source_remove(self._value_changed_timer)
-        self._value_changed_timer = GObject.timeout_add(300, self.update_settings_value)
+        self._value_changed_timer = GObject.timeout_add(100, self.update_settings_value)
 
     def update_settings_value(self):
         self.set_val(self.spinner.get_value())
@@ -429,7 +513,7 @@ class Entry(Gtk.HBox, BaseWidget):
     def __init__(self, key, settings_obj, uuid):
         BaseWidget.__init__(self, key, settings_obj, uuid)
         super(Entry, self).__init__()
-        self.label = Gtk.Label(self.get_desc())
+        self.label = Gtk.Label.new(self.get_desc())
         self.entry = Gtk.Entry()
         self.pack_start(self.label, False, False, 2)
         self.add(self.entry)
@@ -441,7 +525,7 @@ class Entry(Gtk.HBox, BaseWidget):
     def on_my_value_changed(self, widget):
         if self._value_changed_timer:
             GObject.source_remove(self._value_changed_timer)
-        self._value_changed_timer = GObject.timeout_add(300, self.update_settings_value)
+        self._value_changed_timer = GObject.timeout_add(100, self.update_settings_value)
 
     def update_settings_value(self):
         self.set_val(self.entry.get_text())
@@ -456,11 +540,53 @@ class Entry(Gtk.HBox, BaseWidget):
     def update_dep_state(self, active):
         self.entry.set_sensitive(active)
 
+class TextView(Gtk.HBox, BaseWidget):
+    def __init__(self, key, settings_obj, uuid):
+        BaseWidget.__init__(self, key, settings_obj, uuid)
+        super(TextView, self).__init__()
+        self.label = Gtk.Label.new(self.get_desc())
+        self.label.set_alignment(xalign=1, yalign=0)
+        self.scrolledwindow = Gtk.ScrolledWindow(hadjustment=None, vadjustment=None)
+        self.scrolledwindow.set_size_request(width=-1, height=self.get_height())
+        self.scrolledwindow.set_policy(hscrollbar_policy=Gtk.PolicyType.AUTOMATIC,
+                                       vscrollbar_policy=Gtk.PolicyType.AUTOMATIC)
+        self.scrolledwindow.set_shadow_type(type=Gtk.ShadowType.ETCHED_IN)
+        self.textview = Gtk.TextView()
+        self.textview.set_border_width(3)
+        self.textview.set_wrap_mode(wrap_mode=Gtk.WrapMode.NONE)
+        self.buffer = self.textview.get_buffer()
+        self.buffer.set_text(self.get_val())
+        self.pack_start(self.label, False, False, 2)
+        self.add(self.scrolledwindow)
+        self.scrolledwindow.add(self.textview)
+        self.handler = self.buffer.connect("changed", self.on_my_value_changed)
+        set_tt(self.get_tooltip(), self.label, self.textview)
+        self._value_changed_timer = None
+
+    def on_my_value_changed(self, widget):
+        if self._value_changed_timer:
+            GObject.source_remove(self._value_changed_timer)
+        self._value_changed_timer = GObject.timeout_add(100, self.update_settings_value)
+
+    def update_settings_value(self):
+        [start, end] = self.buffer.get_bounds()
+        self.set_val(self.buffer.get_text(start, end, False))
+        self._value_changed_timer = None
+        return False
+
+    def on_settings_file_changed(self):
+        self.textview.handler_block(self.handler)
+        self.buffer.set_text(self.get_val())
+        self.textview.handler_unblock(self.handler)
+
+    def update_dep_state(self, active):
+        self.textview.set_sensitive(active)        
+
 class ColorChooser(Gtk.HBox, BaseWidget):
     def __init__(self, key, settings_obj, uuid):
         BaseWidget.__init__(self, key, settings_obj, uuid)
         super(ColorChooser, self).__init__()
-        self.label = Gtk.Label(self.get_desc())
+        self.label = Gtk.Label.new(self.get_desc())
         self.chooser = Gtk.ColorButton()
         self.chooser.set_use_alpha(True)
         self.eyedropper = eyedropper.EyeDropper()
@@ -481,8 +607,11 @@ class ColorChooser(Gtk.HBox, BaseWidget):
         self.set_val(color)
 
     def on_my_value_changed(self, *args):
-        color = Gdk.RGBA()
-        self.chooser.get_rgba(color)
+        try:
+            color = Gdk.RGBA()
+            self.chooser.get_rgba(color)
+        except TypeError:
+            color = self.chooser.get_rgba()
         self.set_val(color.to_string())
 
     def on_settings_file_changed(self):
@@ -499,7 +628,7 @@ class ComboBox(Gtk.HBox, BaseWidget):
     def __init__(self, key, settings_obj, uuid):
         BaseWidget.__init__(self, key, settings_obj, uuid)
         super(ComboBox, self).__init__()
-        self.label = Gtk.Label(self.get_desc())
+        self.label = Gtk.Label.new(self.get_desc())
         options = self.get_options()
         for option_name in options.keys():
             if isinstance(options[option_name], basestring):
@@ -554,7 +683,7 @@ class RadioGroup(Gtk.VBox, BaseWidget):
     def __init__(self, key, settings_obj, uuid):
         BaseWidget.__init__(self, key, settings_obj, uuid)
         super(RadioGroup, self).__init__()
-        self.label = Gtk.Label(self.get_desc())
+        self.label = Gtk.Label.new(self.get_desc())
         self.model = self.get_options()
         self.entry = None
         self.custom_key = None
@@ -578,6 +707,9 @@ class RadioGroup(Gtk.VBox, BaseWidget):
                 hbox.add(button)
                 self.entry = Gtk.Entry()
                 hbox.add(self.entry)
+                if self.get_val() == self.get_custom_val():
+                    button.set_active(True)
+                button.orig_key = key
             else:
                 if group is None:
                     button = Gtk.RadioButton.new_with_label_from_widget(None, key)
@@ -585,6 +717,10 @@ class RadioGroup(Gtk.VBox, BaseWidget):
                 else:
                     button = Gtk.RadioButton.new_with_label_from_widget(group, key)
                 hbox.add(button)
+                if self.get_val() == self.model[key]:
+                    button.set_active(True)
+                button.orig_key = key
+
             self.pack_start(hbox, False, False, 2)
 
         if self.entry is not None:
@@ -595,13 +731,6 @@ class RadioGroup(Gtk.VBox, BaseWidget):
 
         self.group = group.get_group()
         for button in self.group:
-            label = button.get_label()
-            if label == "":
-                label = self.custom_key
-            if label == self.custom_key:
-                self.custom_button.set_active(True)
-            elif self.get_val() == self.model[label]:
-                button.set_active(True)
             button.handler = button.connect("toggled", self.on_button_activated)
             set_tt(self.get_tooltip(), button)
         self._value_changed_timer = None
@@ -619,7 +748,7 @@ class RadioGroup(Gtk.VBox, BaseWidget):
             if widget is self.custom_button:
                 self.update_custom_settings_value()
             else:
-                self.update_settings_value(widget.get_label())
+                self.update_settings_value(widget.orig_key)
 
     def update_custom_settings_value(self):
         self.set_val(self.entry.get_text())
@@ -669,7 +798,7 @@ class FileChooser(Gtk.HBox, BaseWidget):
         BaseWidget.__init__(self, key, settings_obj, uuid)
         super(FileChooser, self).__init__()
 
-        self.label = Gtk.Label(self.get_desc())
+        self.label = Gtk.Label.new(self.get_desc())
         self.entry = Gtk.Entry()
         self.button = Gtk.Button("")
         self.button.set_image(Gtk.Image().new_from_stock(Gtk.STOCK_OPEN, Gtk.IconSize.BUTTON))
@@ -744,7 +873,7 @@ class IconFileChooser(Gtk.HBox, BaseWidget):
 
         valid, self.width, self.height = Gtk.icon_size_lookup(Gtk.IconSize.BUTTON)
 
-        self.label = Gtk.Label(self.get_desc())
+        self.label = Gtk.Label.new(self.get_desc())
         self.entry = Gtk.Entry()
         self.image_button = Gtk.Button()
 
@@ -774,7 +903,7 @@ class IconFileChooser(Gtk.HBox, BaseWidget):
             self.preview.set_from_icon_name(val, Gtk.IconSize.BUTTON)
 
     def on_button_pressed(self, widget):
-        dialog = Gtk.FileChooserDialog(_("Pick a new icon file"),
+        dialog = Gtk.FileChooserDialog(_("Choose an Icon"),
                                            None,
                                            Gtk.FileChooserAction.OPEN,
                                            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
@@ -784,6 +913,10 @@ class IconFileChooser(Gtk.HBox, BaseWidget):
         filter_text.set_name(_("Image files"))
         filter_text.add_mime_type("image/*")
         dialog.add_filter(filter_text)
+        
+        preview = Gtk.Image()
+        dialog.set_preview_widget(preview)
+        dialog.connect("update-preview", self.update_icon_preview_cb, preview)
 
         response = dialog.run()
 
@@ -798,7 +931,7 @@ class IconFileChooser(Gtk.HBox, BaseWidget):
     def on_entry_changed(self, widget):
         if self._value_changed_timer:
             GObject.source_remove(self._value_changed_timer)
-        self._value_changed_timer = GObject.timeout_add(300, self.update_from_entry)
+        self._value_changed_timer = GObject.timeout_add(100, self.update_from_entry)
 
     def update_from_entry(self):
         self.set_val(self.entry.get_text())
@@ -815,13 +948,26 @@ class IconFileChooser(Gtk.HBox, BaseWidget):
     def update_dep_state(self, active):
         self.entry.set_sensitive(active)
         self.image_button.set_sensitive(active)
-
+        
+    #Updates the preview widget
+    def update_icon_preview_cb(self, dialog, preview):
+        filename = dialog.get_preview_filename()
+        dialog.set_preview_widget_active(False)
+        if os.path.isfile(filename):
+            pixbuf = GdkPixbuf.Pixbuf.new_from_file(filename)
+            if pixbuf is not None:
+                if pixbuf.get_width() > 128:
+                    pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(filename, 128, -1)
+                elif pixbuf.get_height() > 128:
+                    pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(filename, -1, 128)
+                preview.set_from_pixbuf(pixbuf)
+                dialog.set_preview_widget_active(True)
 
 class Scale(Gtk.HBox, BaseWidget):
     def __init__(self, key, settings_obj, uuid):
         BaseWidget.__init__(self, key, settings_obj, uuid)
         super(Scale, self).__init__()
-        self.label = Gtk.Label(self.get_desc())
+        self.label = Gtk.Label.new(self.get_desc())
         self.scale = Gtk.HScale.new_with_range(self.get_min(), self.get_max(), self.get_step())
         self.scale.set_value(self.get_val())
         if (self.get_desc() != ""):
@@ -830,12 +976,27 @@ class Scale(Gtk.HBox, BaseWidget):
         self.handler = self.scale.connect('value-changed', self.on_my_value_changed)
         self.scale.show_all()
         set_tt(self.get_tooltip(), self.label, self.scale)
+        self.scale.connect("scroll-event", self.on_mouse_scroll_event)
         self._value_changed_timer = None
 
+# TODO: Should we fix this in GTK? upscrolling should slide the slider to the right..right?
+# This is already adjusted in Nemo as well.
+    def on_mouse_scroll_event(self, widget, event):
+        found, delta_x, delta_y = event.get_scroll_deltas()
+        if found:
+            add = delta_y < 0
+            val = widget.get_value()
+            if add:
+                val += self.get_step()
+            else:
+                val -= self.get_step()
+            widget.set_value(val)
+        return True
+
     def on_my_value_changed(self, widget):
         if self._value_changed_timer:
             GObject.source_remove(self._value_changed_timer)
-        self._value_changed_timer = GObject.timeout_add(300, self.update_settings_value)
+        self._value_changed_timer = GObject.timeout_add(100, self.update_settings_value)
 
     def update_settings_value(self):
         self.set_val(self.scale.get_value())
@@ -850,6 +1011,105 @@ class Scale(Gtk.HBox, BaseWidget):
     def update_dep_state(self, active):
         self.scale.set_sensitive(active)
 
+
+SPECIAL_MODS = (["Super_L",    "<Super>"],
+                ["Super_R",    "<Super>"],
+                ["Alt_L",      "<Alt>"],
+                ["Alt_R",      "<Alt>"],
+                ["Control_L",  "<Primary>"],
+                ["Control_R",  "<Primary>"],
+                ["Shift_L",    "<Shift>"],
+                ["Shift_R",    "<Shift>"])
+
+class Keybinding(Gtk.HBox, BaseWidget):
+    def __init__(self, key, settings_obj, uuid):
+        BaseWidget.__init__(self, key, settings_obj, uuid)
+        super(Keybinding, self).__init__()
+        self.label = Gtk.Label.new(self.get_desc())
+        self.value = self.get_val()
+        if self.get_desc() != "":
+            self.pack_start(self.label, False, False, 2)
+
+        self.button = Gtk.Button(self.value)
+        self.button.set_tooltip_text(_("Click to set a new accelerator key.") +
+                                     _("  Press Escape or click again to cancel the operation." +
+                                       "  Press Backspace to clear the existing keybinding."))
+        self.button.connect("clicked", self.clicked)
+        self.button.set_size_request(200, -1)
+        self.pack_start(self.button, False, False, 4)
+        self.set_button_text()
+        self.show_all()
+        self.event_id = None
+        self.teaching = False
+
+    def clicked(self, widget):
+        if not self.teaching:
+            device = Gtk.get_current_event_device()
+            if device.get_source() == Gdk.InputSource.KEYBOARD:
+                self.keyboard = device
+            else:
+                self.keyboard = device.get_associated_device()
+
+            self.keyboard.grab(self.get_window(), Gdk.GrabOwnership.WINDOW, False,
+                               Gdk.EventMask.KEY_PRESS_MASK | Gdk.EventMask.KEY_RELEASE_MASK,
+                               None, Gdk.CURRENT_TIME)
+
+            self.button.set_label(_("Pick an accelerator"))
+            self.event_id = self.connect( "key-release-event", self.on_key_release )
+            self.teaching = True
+        else:
+            if self.event_id:
+                self.disconnect(self.event_id)
+            self.ungrab()
+            self.set_button_text()
+            self.teaching = False
+
+    def on_key_release(self, widget, event):
+        self.disconnect(self.event_id)
+        self.ungrab()
+        self.event_id = None
+        if event.keyval == Gdk.KEY_Escape:
+            self.button.set_label(self.value)
+            self.teaching = False
+            return True
+        if event.keyval == Gdk.KEY_BackSpace:
+            self.teaching = False
+            self.value = ""
+            self.set_button_text()
+            self.set_val(self.value)
+            return True
+        accel_string = Gtk.accelerator_name(event.keyval, event.state)
+        accel_string = self.sanitize(accel_string)
+        self.value = accel_string
+        self.set_button_text()
+        self.set_val(self.value)
+        self.teaching = False
+        return True
+
+    def sanitize(self, string):
+        accel_string = string.replace("<Mod2>", "")
+        accel_string = accel_string.replace("<Mod4>", "")
+        for single, mod in SPECIAL_MODS:
+            if single in accel_string and mod in accel_string:
+                accel_string = accel_string.replace(mod, "")
+        return accel_string
+
+    def set_button_text(self):
+        if self.value == "":
+            self.button.set_label(_("<not set>"))
+        else:
+            self.button.set_label(self.value)
+
+    def ungrab(self):
+        self.keyboard.ungrab(Gdk.CURRENT_TIME)
+
+    def on_settings_file_changed(self):
+        self.value = self.get_val()
+        self.set_button_text()
+
+    def update_dep_state(self, active):
+        self.button.set_sensitive(active)
+
 class Button(Gtk.Button, BaseWidget):
     def __init__(self, key, settings_obj, uuid):
         BaseWidget.__init__(self, key, settings_obj, uuid)
