@@ -8,7 +8,6 @@ const Signals = imports.signals;
 const St = imports.gi.St;
 const Mainloop = imports.mainloop;
 const Main = imports.ui.main;
-const PanelMenu = imports.ui.panelMenu;
 const PopupMenu = imports.ui.popupMenu;
 const MessageTray = imports.ui.messageTray;
 const ModemManager = imports.misc.modemManager;
@@ -389,7 +388,6 @@ NMDevice.prototype = {
     },
 
     deactivate: function() {	
-		log("DISCONNECT");	
         this.device.disconnect(function() {});
     },
 
@@ -757,7 +755,23 @@ NMDeviceModem.prototype = {
         this._connectionType = 'ppp';
 
         this._capabilities = device.current_capabilities;
-        if (this._capabilities & NetworkManager.DeviceModemCapabilities.GSM_UMTS) {
+        // Support new ModemManager1 devices
+        if (device.udi.indexOf('/org/freedesktop/ModemManager1/Modem') == 0) {
+            try {
+                is_wwan = true;
+                this.mobileDevice = new ModemManager.BroadbandModem(device.udi, device.current_capabilities);
+                if (this._capabilities & NetworkManager.DeviceModemCapabilities.GSM_UMTS) {
+                    this._connectionType = NetworkManager.SETTING_GSM_SETTING_NAME;
+                } else if (this._capabilities & NetworkManager.DeviceModemCapabilities.LTE) {
+                    this._connectionType = NetworkManager.SETTING_GSM_SETTING_NAME;
+                } else if (this._capabilities & NetworkManager.DeviceModemCapabilities.CDMA_EVDO) {
+                    this._connectionType = NetworkManager.SETTING_CDMA_SETTING_NAME;
+                }
+            }
+            catch (e){
+                global.logError(e);
+            }
+        } else if (this._capabilities & NetworkManager.DeviceModemCapabilities.GSM_UMTS) {
             is_wwan = true;
             this.mobileDevice = new ModemManager.ModemGsm(device.udi);
             this._connectionType = NetworkManager.SETTING_GSM_SETTING_NAME;
@@ -1169,7 +1183,6 @@ NMDeviceWireless.prototype = {
     _networkSortFunction: function(one, two) {
         let oneHasConnection = one.connections.length != 0;
         let twoHasConnection = two.connections.length != 0;
-
         // place known connections first
         // (-1 = good order, 1 = wrong order)
         if (oneHasConnection && !twoHasConnection)
@@ -1177,6 +1190,8 @@ NMDeviceWireless.prototype = {
         else if (!oneHasConnection && twoHasConnection)
             return 1;
 
+        return two.accessPoints[0].strength - one.accessPoints[0].strength;
+/**
         let oneHasSecurity = one.security != NMAccessPointSecurity.NONE;
         let twoHasSecurity = two.security != NMAccessPointSecurity.NONE;
 
@@ -1190,6 +1205,7 @@ NMDeviceWireless.prototype = {
 
         // sort alphabetically
         return GLib.utf8_collate(one.ssidText, two.ssidText);
+**/
     },
 
     _networkCompare: function(network, accessPoint) {
@@ -1598,17 +1614,20 @@ NMMessageTraySource.prototype = {
     }
 };
 
-function MyApplet(orientation, panel_height) {
-    this._init(orientation, panel_height);
+function MyApplet(metadata, orientation, panel_height) {
+    this._init(metadata, orientation, panel_height);
 }
 
 MyApplet.prototype = {
     __proto__: Applet.IconApplet.prototype,
 
-    _init: function(orientation, panel_height) {        
+    _init: function(metadata, orientation, panel_height) {        
         Applet.IconApplet.prototype._init.call(this, orientation, panel_height);
         
-        try {                                
+        try {
+            this.metadata = metadata;
+            Main.systrayManager.registerRole("network", metadata.uuid);
+
             this.menuManager = new PopupMenu.PopupMenuManager(this);
             this.menu = new Applet.AppletPopupMenu(this, orientation);
             this.menuManager.addMenu(this.menu);            
@@ -1676,6 +1695,9 @@ MyApplet.prototype = {
             this.menu.addMenuItem(this._devices.vpn.section);
             this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
             this.menu.addSettingsAction(_("Network Settings"), 'network');
+            this.menu.addAction(_("Network Connections"), Lang.bind(this, function() {
+				Util.spawnCommandLine("nm-connection-editor");
+            }));
 
             this._activeConnections = [ ];
             this._connections = [ ];
@@ -2200,6 +2222,7 @@ MyApplet.prototype = {
     },
 
     on_applet_removed_from_panel: function() {
+        Main.systrayManager.unregisterRole("network", this.metadata.uuid);
         if (this._periodicTimeoutId){
             Mainloop.source_remove(this._periodicTimeoutId);
         }
@@ -2208,6 +2231,6 @@ MyApplet.prototype = {
 };
 
 function main(metadata, orientation, panel_height) {  
-    let myApplet = new MyApplet(orientation, panel_height);
+    let myApplet = new MyApplet(metadata, orientation, panel_height);
     return myApplet;      
 }
