@@ -4,23 +4,25 @@ const Main = imports.ui.main;
 const Gtk = imports.gi.Gtk;
 const PopupMenu = imports.ui.popupMenu;
 const St = imports.gi.St;
-const Gio = imports.gi.Gio;
 const Clutter = imports.gi.Clutter;
 const Mainloop = imports.mainloop;
 const MessageTray = imports.ui.messageTray;
 const Urgency = imports.ui.messageTray.Urgency;
 const NotificationDestroyedReason = imports.ui.messageTray.NotificationDestroyedReason;
+const Settings = imports.ui.settings;
 
-function MyApplet(metadata, orientation, panel_height) {
-    this._init(metadata, orientation, panel_height);
+function MyApplet(metadata, orientation, panel_height, instanceId) {
+    this._init(metadata, orientation, panel_height, instanceId);
 }
 
 MyApplet.prototype = {
     __proto__: Applet.TextIconApplet.prototype,
 
-    _init: function(metadata, orientation, panel_height) {
+    _init: function(metadata, orientation, panel_height, instanceId) {
         Applet.TextIconApplet.prototype._init.call(this, orientation, panel_height);
 
+        this.settings = new Settings.AppletSettings(this, metadata.uuid, instanceId);
+        this.settings.bindProperty(Settings.BindingDirection.IN, "ignoreTransientNotifications", "ignoreTransientNotifications", null, null);
         Gtk.IconTheme.get_default().append_search_path(metadata.path);
         this._orientation = orientation;
         this.menuManager = new PopupMenu.PopupMenuManager(this);
@@ -28,8 +30,6 @@ MyApplet.prototype = {
         this.notifications = [];
         Main.messageTray.connect('notify-applet-update', Lang.bind(this, this._notification_added));
         global.settings.connect('changed::panel-edit-mode', Lang.bind(this, this.on_panel_edit_mode_changed));
-        this._calendarSettings = new Gio.Settings({ schema: 'org.cinnamon.calendar' });
-        this._calendarSettings.connect('changed', Lang.bind(this, this._update_timestamp));
         this._blinking = false;
         this._blink_toggle = false;
     },
@@ -45,51 +45,87 @@ MyApplet.prototype = {
         this.menu_label.actor.reactive = false;
         this.menu_label.actor.can_focus = false;
         this.menu_label.label.add_style_class_name('popup-subtitle-menu-item');
-        this.menu.addMenuItem(this.menu_label);
-        this.menu.addActor(this._maincontainer);
         this.clear_separator = new PopupMenu.PopupSeparatorMenuItem();
-        this.menu.addMenuItem(this.clear_separator);
         this.clear_action = new PopupMenu.PopupMenuItem(_("Clear notifications"));
-        this.menu.addMenuItem(this.clear_action);
         this.clear_action.connect('activate', Lang.bind(this, this._clear_all));
         this.clear_action.actor.hide();
-        this.scrollview = new St.ScrollView({ x_fill: true, y_fill: true, y_align: St.Align.START});
+
+        if (this._orientation == St.Side.BOTTOM) {
+            this.menu.addMenuItem(this.menu_label);
+            this.menu.addActor(this._maincontainer);
+            this.menu.addMenuItem(this.clear_separator);
+            this.menu.addMenuItem(this.clear_action);
+        } else {
+            this.menu.addMenuItem(this.clear_action);
+            this.menu.addMenuItem(this.clear_separator);
+            this.menu.addMenuItem(this.menu_label);
+            this.menu.addActor(this._maincontainer);
+        }
+
+        this.scrollview = new St.ScrollView({ x_fill: true, y_fill: true, y_align: St.Align.START, style_class: "vfade"});
         this._maincontainer.add(this.scrollview);
         this.scrollview.add_actor(this._notificationbin);
         this.scrollview.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
+
+        let vscroll = this.scrollview.get_vscroll_bar();
+        vscroll.connect('scroll-start',
+                        Lang.bind(this, function() {
+                                      this.menu.passEvents = true;
+                                  }));
+        vscroll.connect('scroll-stop',
+                        Lang.bind(this, function() {
+                                      this.menu.passEvents = false;
+                                  }));
+
         this._crit_icon = new St.Icon({icon_name: 'critical-notif', icon_type: St.IconType.SYMBOLIC, reactive: true, track_hover: true, style_class: 'system-status-icon' });
         this._alt_crit_icon = new St.Icon({icon_name: 'alt-critical-notif', icon_type: St.IconType.SYMBOLIC, reactive: true, track_hover: true, style_class: 'system-status-icon' });
         this.update_list();
     },
 
     _notification_added: function (mtray, notification) {
+        
+        // Ignore transient notifications?
+        if (this.ignoreTransientNotifications && notification.isTransient) {
+            return;
+        }
+
         notification.actor.unparent();
         let existing_index = this.notifications.indexOf(notification);
         if (existing_index != -1) {
-            notification._inNotificationBin = true;
-            notification.actor.reparent(this._notificationbin);
-            notification.expand();
-            notification._timeLabel.show();
+            if (notification._destroyed) {
+                this.notifications.splice(existing_index, 1);
+            }
+            else {
+                notification._inNotificationBin = true;
+                global.reparentActor(notification.actor, this._notificationbin);
+                notification.expand();
+                notification._timeLabel.show();
+            }
             this.update_list();
             return;
         }
+        if (notification._destroyed) {
+            return;
+        }
         notification._inNotificationBin = true;
         this.notifications.push(notification);
         notification.expand();
         this._notificationbin.add(notification.actor)
         notification.actor._parent_container = this._notificationbin;
         notification.actor.add_style_class_name('notification-applet-padding');
-        notification.connect('clicked', Lang.bind(this, this._item_clicked));
-        notification.connect('destroy', Lang.bind(this, this._item_clicked));
+        notification.connect('clicked', Lang.bind(this, this._item_clicked, false));
+        notification.connect('destroy', Lang.bind(this, this._item_clicked, true));
         notification._timeLabel.show();
         this.update_list();
     },
 
-    _item_clicked: function(notification) {
+    _item_clicked: function(notification, destroyed) {
         let i = this.notifications.indexOf(notification);
         if (i != -1) {
             this.notifications.splice(i, 1);
-            notification.destroy(NotificationDestroyedReason.DISMISSED);
+            if (!destroyed) {
+                notification.destroy(NotificationDestroyedReason.DISMISSED);
+            }
         }
         this.update_list();
     },
@@ -174,7 +210,7 @@ MyApplet.prototype = {
     },
 
     _update_timestamp: function () {
-        let dateFormat = this._calendarSettings.get_string('date-format');       
+        let dateFormat = _("%l:%M %p");
         let actors = this._notificationbin.get_children();
         if (actors) {
             for (let i = 0; i < actors.length; i++) {
@@ -198,8 +234,8 @@ MyApplet.prototype = {
     }
 };
 
-function main(metadata, orientation, panel_height) {
-    let myApplet = new MyApplet(metadata, orientation, panel_height);
+function main(metadata, orientation, panel_height, instanceId) {
+    let myApplet = new MyApplet(metadata, orientation, panel_height, instanceId);
     return myApplet;
 }
 
