@@ -5,11 +5,11 @@ const Lang = imports.lang;
 const St = imports.gi.St;
 const PopupMenu = imports.ui.popupMenu;
 const Pango = imports.gi.Pango;
+const Main = imports.ui.main;
+const Settings = imports.ui.settings;
 
-const POWER_SCHEMA = "org.cinnamon.power"
-const SHOW_PERCENTAGE_KEY = "power-label";
-const BUS_NAME = 'org.gnome.SettingsDaemon';
-const OBJECT_PATH = '/org/gnome/SettingsDaemon/Power';
+const BUS_NAME = 'org.cinnamon.SettingsDaemon';
+const OBJECT_PATH = '/org/cinnamon/SettingsDaemon/Power';
 
 const UPDeviceType = {
     UNKNOWN: 0,
@@ -36,14 +36,8 @@ const UPDeviceState = {
     PENDING_DISCHARGE: 6
 };
 
-const LabelDisplay = {
-    NONE: 'none',
-    PERCENT: 'percent',
-    TIME: 'time'
-};
-
 const PowerManagerInterface = {
-    name: 'org.gnome.SettingsDaemon.Power',
+    name: 'org.cinnamon.SettingsDaemon.Power',
     methods: [
         { name: 'GetDevices', inSignature: '', outSignature: 'a(susdut)' },
         { name: 'GetPrimaryDevice', inSignature: '', outSignature: '(susdut)' },
@@ -60,9 +54,7 @@ let PowerManagerProxy = DBus.makeProxyClass(PowerManagerInterface);
 const SettingsManagerInterface = {
 	name: 'org.freedesktop.DBus.Properties',
 	methods: [
-		{ name: 'Get', inSignature: 's,s', outSignature: 'v' },
-		{ name: 'GetAll', inSignature: 's', outSignature: 'a{sv}' },
-		{ name: 'Set', inSignature: 's,s,v', outSignature: '' }
+		{ name: 'GetAll', inSignature: 's', outSignature: 'a{sv}' }
 	],
 	signals: [
 	{name: 'PropertiesChanged', inSignature:'s,a{sv},a[s]', outSignature:''}
@@ -81,7 +73,7 @@ DeviceItem.prototype = {
     _init: function(device) {
         PopupMenu.PopupBaseMenuItem.prototype._init.call(this, { reactive: false });
 
-        let [device_id, device_type, icon, percentage, state, time] = device;
+        let [device_id, device_type, icon, percentage, state, time, timepercentage] = device;
 
         this._box = new St.BoxLayout({ style_class: 'popup-device-menu-item' });
         this._label = new St.Label({ text: this._deviceTypeToString(device_type) });
@@ -128,23 +120,31 @@ DeviceItem.prototype = {
     }
 }
 
-function MyApplet(orientation, panel_height) {
-    this._init(orientation, panel_height);
+function MyApplet(metadata, orientation, panel_height, instanceId) {
+    this._init(metadata, orientation, panel_height, instanceId);
 }
 
 
 MyApplet.prototype = {
     __proto__: Applet.TextIconApplet.prototype,
 
-    _init: function(orientation, panel_height) {        
+    _init: function(metadata, orientation, panel_height, instanceId) {        
         Applet.TextIconApplet.prototype._init.call(this, orientation, panel_height);
         
-        try {                                
+        try {
+            this.metadata = metadata;
+
+            this.settings = new Settings.AppletSettings(this, metadata.uuid, instanceId);
+            this.settings.bindProperty(Settings.BindingDirection.IN, "labelinfo", "labelinfo", this._updateLabel, null);
+            
+            Main.systrayManager.registerRole("power", metadata.uuid);
+            Main.systrayManager.registerRole("battery", metadata.uuid);
+            
             this.menuManager = new PopupMenu.PopupMenuManager(this);
             this.menu = new Applet.AppletPopupMenu(this, orientation);
             this.menuManager.addMenu(this.menu);            
             
-            this.set_applet_icon_symbolic_name('battery-missing');            
+            //this.set_applet_icon_symbolic_name('battery-missing');            
             this._proxy = new PowerManagerProxy(DBus.session, BUS_NAME, OBJECT_PATH);
             this._smProxy = new SettingsManagerProxy(DBus.session, BUS_NAME, OBJECT_PATH);
             
@@ -163,41 +163,15 @@ MyApplet.prototype = {
             this._hasPrimary = false;
             this._primaryDeviceId = null;
             
-            let settings = new Gio.Settings({ schema: POWER_SCHEMA }); 
-            this._labelDisplay = settings.get_string(SHOW_PERCENTAGE_KEY);
             let applet = this;
-            settings.connect('changed::'+SHOW_PERCENTAGE_KEY, function() {
-                applet._switchLabelDisplay(settings.get_string(SHOW_PERCENTAGE_KEY));
-            });
 
             this._batteryItem = new PopupMenu.PopupMenuItem('', { reactive: false });
             this._primaryPercentage = new St.Label();
             this._batteryItem.addActor(this._primaryPercentage, { align: St.Align.END });
             this.menu.addMenuItem(this._batteryItem);
 
-            this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
-            this._otherDevicePosition = 2;
+            this._otherDevicePosition = 1;
             
-            // Setup label display settings
-            this._displayItem = new PopupMenu.PopupSubMenuMenuItem(_("Display"));
-            this.menu.addMenuItem(this._displayItem);
-            this._displayPercentageItem = new PopupMenu.PopupMenuItem(_("Show percentage"));
-            this._displayPercentageItem.connect('activate', Lang.bind(this, function() {
-                settings.set_string(SHOW_PERCENTAGE_KEY, LabelDisplay.PERCENT);
-            }));
-            this._displayItem.menu.addMenuItem(this._displayPercentageItem);
-            this._displayTimeItem = new PopupMenu.PopupMenuItem(_("Show time remaining"));
-            this._displayTimeItem.connect('activate', Lang.bind(this, function() {
-                settings.set_string(SHOW_PERCENTAGE_KEY, LabelDisplay.TIME);
-            }));
-            this._displayItem.menu.addMenuItem(this._displayTimeItem);
-            this._displayNoneItem = new PopupMenu.PopupMenuItem(_("Hide label"));
-            this._displayNoneItem.connect('activate', Lang.bind(this, function() {
-                settings.set_string(SHOW_PERCENTAGE_KEY, LabelDisplay.NONE);
-            }));
-            this._displayItem.menu.addMenuItem(this._displayNoneItem);
-            this._switchLabelDisplay(this._labelDisplay);
-
             this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
             this.menu.addSettingsAction(_("Power Settings"), 'power');
 
@@ -212,26 +186,6 @@ MyApplet.prototype = {
     on_applet_clicked: function(event) {
         this.menu.toggle();        
     },
-
-    _switchLabelDisplay: function(display) {
-            this._labelDisplay = display;
-
-            this._displayPercentageItem.setShowDot(false);
-            this._displayNoneItem.setShowDot(false);
-            this._displayTimeItem.setShowDot(false);
-
-            if (this._labelDisplay == LabelDisplay.PERCENT) {
-                this._displayPercentageItem.setShowDot(true);
-            }
-            else if (this._labelDisplay == LabelDisplay.TIME) {
-                this._displayTimeItem.setShowDot(true);
-            }
-            else {
-                this._displayNoneItem.setShowDot(true);
-            }
-
-            this._updateLabel();
-    },
     
     _readPrimaryDevice: function() {
         this._proxy.GetPrimaryDeviceRemote(Lang.bind(this, function(device, error) {
@@ -290,11 +244,20 @@ MyApplet.prototype = {
             let position = 0;
             for (let i = 0; i < devices.length; i++) {
                 let [device_id, device_type] = devices[i];
-                if (device_type == UPDeviceType.AC_POWER || device_id == this._primaryDeviceId)
+
+                if (this._hasPrimary == false) {
+                	if (device_type == UPDeviceType.AC_POWER) {
+                    	this.set_applet_tooltip(_("AC adapter"));
+                	}
+                	else if (device_type == UPDeviceType.BATTERY) {
+                    	this.set_applet_tooltip(_("Laptop battery"));
+               		}
+               	}
+
+                if (device_type == UPDeviceType.AC_POWER || (this._hasPrimary && device_id == this._primaryDeviceId))
                     continue;
 
                 let item = new DeviceItem (devices[i]);
-                this.set_applet_tooltip(item._label.text);
                 this._deviceItems.push(item);
                 this.menu.addMenuItem(item, this._otherDevicePosition + position);
                 position++;
@@ -306,10 +269,10 @@ MyApplet.prototype = {
         this._devicesChanged();
     },
 
-    _devicesChanged: function() {
-        this.set_applet_icon_symbolic_name('battery-missing');
+    _devicesChanged: function() {        
         this._proxy.GetRemote('Icon', Lang.bind(this, function(icon, error) {
             if (icon) {    
+                this.set_applet_icon_symbolic_name('battery-missing');
                 let gicon = Gio.icon_new_for_string(icon);
                 this._applet_icon.gicon = gicon;
                 this.actor.show();
@@ -326,39 +289,47 @@ MyApplet.prototype = {
     _updateLabel: function() {
         this._proxy.GetDevicesRemote(Lang.bind(this, function(devices, error) {
             if (error) {
-                this._mainLabel.set_text("");
+            	this._mainLabel.set_text("");
                 return;
             }
+            if (this.labelinfo != "nothing") {
+            	for (let i = 0; i < devices.length; i++) {
+	                let [device_id, device_type, icon, percentage, state, time] = devices[i];
+	                if (device_type == UPDeviceType.BATTERY || device_id == this._primaryDeviceId) {
+	                    let labelText = "";
 
-            if (this._labelDisplay != LabelDisplay.NONE) {
-                for (let i = 0; i < devices.length; i++) {
-                    let [device_id, device_type, icon, percentage, state, time] = devices[i];
-                    if (device_type == UPDeviceType.BATTERY || device_id == this._primaryDeviceId) {
-                        let labelText = "";
-
-                        if (this._labelDisplay == LabelDisplay.PERCENT || time == 0) {
-                            labelText = C_("percent of battery remaining", "%d%%").format(Math.round(percentage));
-                        }
-                        else if (this._labelDisplay == LabelDisplay.TIME) {
-                            let seconds = time / 60;
-                            let minutes = Math.floor(seconds % 60);
-                            let hours = Math.floor(seconds / 60);
-                            labelText = C_("time of battery remaining", "%d:%02d").format(hours,minutes);
-                        }
-
-                        this._mainLabel.set_text(labelText);
-                        return;
-                    }
+	                    if (this.labelinfo == "percentage" || time == 0) {
+	                        labelText = C_("percent of battery remaining", "%d%%").format(Math.round(percentage));
+	                    }
+	                    else if (this.labelinfo == "time") {
+	                        let seconds = Math.round(time / 60);
+	                        let minutes = Math.floor(seconds % 60);
+	                        let hours = Math.floor(seconds / 60);
+	                        labelText = C_("time of battery remaining", "%d:%02d").format(hours,minutes);
+	                    }
+	                    else if (this.labelinfo == "percentage_time") {
+	                        let seconds = Math.round(time / 60);
+	                        let minutes = Math.floor(seconds % 60);
+	                        let hours = Math.floor(seconds / 60);
+	                        labelText = C_("percent of battery remaining", "%d%%").format(Math.round(percentage)) + " (" +
+	                                    C_("time of battery remaining", "%d:%02d").format(hours,minutes) + ")";
+	                    }
+	                    this._mainLabel.set_text(labelText);
+	                    if (device_id == this._primaryDeviceId) {
+	                    	return;
+	                    }
+	            	}
                 }
             }
-            // Display disabled or no battery found... hot-unplugged?
-            this._mainLabel.set_text("");
         }));
-    }
+    },
     
+    on_applet_removed_from_panel: function() {
+        Main.systrayManager.unregisterId(this.metadata.uuid);
+    }
 };
 
-function main(metadata, orientation, panel_height) {  
-    let myApplet = new MyApplet(orientation, panel_height);
+function main(metadata, orientation, panel_height, instanceId) {  
+    let myApplet = new MyApplet(metadata, orientation, panel_height, instanceId);
     return myApplet;      
 }
