@@ -18,6 +18,42 @@ const DEFAULT_ICON_SIZE = 16; // too bad this can't be defined in theme (cinnamo
 const SPINNER_ANIMATION_TIME = 1;
 const ICON_HEIGHT_FACTOR = .64;
 
+/* TODO: dragHelper will need to be reworked once more flexible panel configuration is merged */
+
+function dragHelper() {
+    this._init();
+}
+
+dragHelper.prototype = {
+    _init: function() {
+        this.dragging = false;
+        this.panel_show_id = 0;
+    },
+
+    temp_show_panels: function() {
+        if (Main.panel && !this.dragging)
+            Main.panel._enterPanel();
+        if (Main.panel2 && !this.dragging)
+            Main.panel2._enterPanel();
+        if (this.panel_show_id > 0) {
+            Mainloop.source_remove(this.panel_show_id);
+            this.panel_show_id = 0;
+        }
+        this.dragging = true;
+        this.panel_show_id = Mainloop.timeout_add(2000, Lang.bind(this, this.temp_unshow_panels));
+    },
+
+    temp_unshow_panels: function() {
+        if (Main.panel)
+            Main.panel._leavePanel();
+        if (Main.panel2)
+            Main.panel2._leavePanel();
+        this.dragging = false;
+        return false;
+    }
+}
+
+let drag_helper = new dragHelper();
 
 function AppMenuButtonRightClickMenu(actor, metaWindow, orientation) {
     this._init(actor, metaWindow, orientation);
@@ -65,6 +101,13 @@ AppMenuButtonRightClickMenu.prototype = {
         let itemOnAllWorkspaces = new PopupMenu.PopupMenuItem(_("Visible on all workspaces"));
         itemOnAllWorkspaces.connect('activate', Lang.bind(this, this._toggleOnAllWorkspaces));
 
+        let itemRestoreOpacity = new PopupMenu.PopupMenuItem(_("Restore to full opacity"));
+        itemRestoreOpacity.connect('activate', Lang.bind(this, this._onRestoreOpacity));
+
+        if (this.metaWindow.get_compositor_private().opacity == 255) {
+            itemRestoreOpacity.actor.hide()
+        }
+
         if (mw.is_on_all_workspaces()) {
             itemOnAllWorkspaces.label.set_text(_("Only on this workspace"));
             itemMoveToLeftWorkspace.actor.hide();
@@ -105,6 +148,7 @@ AppMenuButtonRightClickMenu.prototype = {
             itemCloseAllWindows,
             itemCloseOtherWindows,
             new PopupMenu.PopupSeparatorMenuItem(),
+            itemRestoreOpacity,
             itemMinimizeWindow,
             itemMaximizeWindow,
             itemCloseWindow
@@ -114,12 +158,15 @@ AppMenuButtonRightClickMenu.prototype = {
         }, this);
      },
 
+     _onRestoreOpacity: function(actor, event) {
+        this.metaWindow.get_compositor_private().set_opacity(255);
+     },
+
      _onToggled: function(actor, isOpening){
         if (!isOpening) {
-            this.removeAll();
             return;
         }
-
+        this.removeAll();
         this._populateMenu();
     },
     
@@ -130,7 +177,7 @@ AppMenuButtonRightClickMenu.prototype = {
     _onCloseAllActivate: function(actor, event) {
         let metas = new Array();
         for (let i = 0; i < this.window_list.length; i++) {
-            if (this.window_list[i].actor.visible) {
+            if (this.window_list[i].actor.visible && !this.window_list[i]._needsAttention) {
                 metas.push(this.window_list[i].metaWindow);
             }
         }
@@ -142,7 +189,9 @@ AppMenuButtonRightClickMenu.prototype = {
     _onCloseOthersActivate: function(actor, event) {
         let metas = new Array();
         for (let i = 0; i < this.window_list.length; i++) {
-            if (this.window_list[i].metaWindow != this.metaWindow && this.window_list[i].actor.visible) {
+            if (this.window_list[i].metaWindow != this.metaWindow &&
+                                this.window_list[i].actor.visible &&
+                                !this.window_list[i]._needsAttention) {
                 metas.push(this.window_list[i].metaWindow);
             }
         }
@@ -216,15 +265,15 @@ AppMenuButtonRightClickMenu.prototype = {
 
 };
 
-function AppMenuButton(applet, metaWindow, animation, orientation, panel_height) {
-    this._init(applet, metaWindow, animation, orientation, panel_height);
+function AppMenuButton(applet, metaWindow, animation, orientation, panel_height, draggable) {
+    this._init(applet, metaWindow, animation, orientation, panel_height, draggable);
 }
 
 AppMenuButton.prototype = {
 //    __proto__ : AppMenuButton.prototype,
 
     
-    _init: function(applet, metaWindow, animation, orientation, panel_height) {
+    _init: function(applet, metaWindow, animation, orientation, panel_height, draggable) {
                
         this.actor = new St.Bin({ style_class: 'window-list-item-box',
 								  reactive: true,
@@ -240,11 +289,10 @@ AppMenuButton.prototype = {
       
         this.actor._delegate = this;
         this.actor.connect('button-release-event', Lang.bind(this, this._onButtonRelease));
-
+        this.actor.connect('button-press-event', Lang.bind(this, this._onButtonPress));
 		this.metaWindow = metaWindow;	
 
-        this._applet = applet;	
-		
+        this._applet = applet;
         let bin = new St.Bin({ name: 'appMenu' });
         this.actor.set_child(bin);
 
@@ -276,48 +324,58 @@ AppMenuButton.prototype = {
             this._label.set_text(title);
             if (this._tooltip) this._tooltip.set_text(title);
         }));
-                
+
+        this._updateTileTypeId = this.metaWindow.connect('notify::tile-type', Lang.bind(this, function () {
+            let title = this.getDisplayTitle();
+            this._label.set_text(title);
+            if (this._tooltip) this._tooltip.set_text(title);
+        }));
+        
+
         this._spinner = new Panel.AnimatedIcon('process-working.svg', PANEL_ICON_SIZE);
         this._container.add_actor(this._spinner.actor);
         this._spinner.actor.lower_bottom();
-
-        let tracker = Cinnamon.WindowTracker.get_default();
-        this.app = tracker.get_window_app(this.metaWindow);
+        
         this.set_icon(panel_height);
         let title = this.getDisplayTitle();
-        if (metaWindow.minimized)
-            this._label.set_text("[" + title + "]");
-        else
-            this._label.set_text(title);
+        this._label.set_text(title);        
         
         if(animation){
 			this.startAnimation(); 
 			this.stopAnimation();
 		}
-		
-        //set up the right click menu
-        this._menuManager = new PopupMenu.PopupMenuManager(this);
-        this.rightClickMenu = new AppMenuButtonRightClickMenu(this.actor, this.metaWindow, orientation);
-        this._menuManager.addMenu(this.rightClickMenu);
-        
+
         this._tooltip = new Tooltips.PanelItemTooltip(this, title, orientation);
-        
-        this._draggable = DND.makeDraggable(this.actor);
-        this._draggable.connect('drag-begin', Lang.bind(this, this._onDragBegin));
-        this._draggable.connect('drag-cancelled', Lang.bind(this, this._onDragCancelled));
-        this._draggable.connect('drag-end', Lang.bind(this, this._onDragEnd));
-        
+
+        if (draggable) {
+            //set up the right click menu
+            this._menuManager = new PopupMenu.PopupMenuManager(this);
+            this.rightClickMenu = new AppMenuButtonRightClickMenu(this.actor, this.metaWindow, orientation);
+            this._menuManager.addMenu(this.rightClickMenu);
+
+            this._draggable = DND.makeDraggable(this.actor);
+            this._draggable.connect('drag-begin', Lang.bind(this, this._onDragBegin));
+            this._draggable.connect('drag-cancelled', Lang.bind(this, this._onDragCancelled));
+            this._draggable.connect('drag-end', Lang.bind(this, this._onDragEnd));
+        } else {
+            this._draggable = null;
+        }
+
         this._inEditMode = undefined;
         this.on_panel_edit_mode_changed();
         global.settings.connect('changed::panel-edit-mode', Lang.bind(this, this.on_panel_edit_mode_changed));
         global.settings.connect('changed::window-list-applet-scroll', Lang.bind(this, this.on_scroll_mode_changed));
         this.window_list = this.actor._delegate._applet._windows;
+        this.alert_list = this.actor._delegate._applet._alertWindows;
         this.scroll_connector = null;
         this.on_scroll_mode_changed();
+        this._needsAttention = false;
     },
     
     on_panel_edit_mode_changed: function() {
-        this._inEditMode = this._draggable.inhibit = global.settings.get_boolean("panel-edit-mode");
+        if (this._draggable) {
+            this._inEditMode = this._draggable.inhibit = global.settings.get_boolean("panel-edit-mode");
+        }
     }, 
 
     on_scroll_mode_changed: function() {
@@ -375,14 +433,31 @@ AppMenuButton.prototype = {
 
     getDisplayTitle: function() {
         let title = this.metaWindow.get_title();
-        if (!title) title = this.app ? this.app.get_name() : '?';
-        return title;
+        let tracker = Cinnamon.WindowTracker.get_default();
+        let app = tracker.get_window_app(this.metaWindow);
+        if (!title) title = app ? app.get_name() : '?';
+
+        if (this.metaWindow.minimized) {
+            return "["+ title +"]";                        
+        }                    
+        else if (this.metaWindow.tile_type == Meta.WindowTileType.TILED) {
+            return "|"+ title;
+        }
+        else if (this.metaWindow.tile_type == Meta.WindowTileType.SNAPPED) {
+            return "||"+ title;
+        }
+        else {
+            return title;
+        }        
     },
 
     _onDestroy: function() {
         this.metaWindow.disconnect(this._updateCaptionId);
+        this.metaWindow.disconnect(this._updateTileTypeId);        
         this._tooltip.destroy();
-        this.rightClickMenu.destroy();
+        if (this.rightClickMenu) {
+            this.rightClickMenu.destroy();
+        }
     },
     
     _hasFocus: function(metaWindow) {
@@ -401,37 +476,63 @@ AppMenuButton.prototype = {
     },
     
     doFocus: function() {
-        if (this._hasFocus(this.metaWindow) && !this.metaWindow.minimized) {                                     
-        	this.actor.add_style_pseudo_class('focus');    
-            this.actor.remove_style_class_name("window-list-item-demands-attention");    	
+        if (this._hasFocus(this.metaWindow) && !this.metaWindow.minimized) {
+            this.actor.add_style_pseudo_class('focus');
+            this.actor.remove_style_class_name("window-list-item-demands-attention");
             this.actor.remove_style_class_name("window-list-item-demands-attention-top");
-        }        		    	        
-        else {            
-          	this.actor.remove_style_pseudo_class('focus');        		
-        }	    	                
+            this._needsAttention = false;
+            this._removeAlerts(this.metaWindow);
+        } else {
+            this.actor.remove_style_pseudo_class('focus');
+        }
     },
-    
+
     _onButtonRelease: function(actor, event) {
         this._tooltip.hide();
-        if ( Cinnamon.get_event_state(event) & Clutter.ModifierType.BUTTON1_MASK ) {
+        if (!this._draggable) {
+            /* non-draggable = off-workspace window demanding attention
+               the only action for these is activation */
+            if (event.get_button() == 1) {
+                this._windowHandle(false);
+            }
+            return false;
+        }
+        if (event.get_button() == 1) {
             if ( this.rightClickMenu.isOpen ) {
-                this.rightClickMenu.toggle();                
+                this.rightClickMenu.toggle();
             }
             this._windowHandle(false);
-        } else if (Cinnamon.get_event_state(event) & Clutter.ModifierType.BUTTON2_MASK) {
+        } else if (event.get_button() == 2)
             this.metaWindow.delete(global.get_current_time());
-        } else if (Cinnamon.get_event_state(event) & Clutter.ModifierType.BUTTON3_MASK) {
+        return true;
+    },
+
+    _onButtonPress: function(actor, event) {
+        this._tooltip.hide();
+        if (!this._draggable) {
+            return false;
+        }
+        if (event.get_button() == 3) {
             this.rightClickMenu.mouseEvent = event;
-            this.rightClickMenu.toggle();   
-        }   
+            this.rightClickMenu.toggle();
+            return true;
+        }
+        return false;
     },
 
     _windowHandle: function(fromDrag){
-        if ( this.metaWindow.has_focus() ) {
+        let has_focus = this.metaWindow.has_focus();
+        if (!this.metaWindow.minimized && !has_focus) {
+            this.metaWindow.foreach_transient(function(child) {
+                if (!child.minimized && child.has_focus()) {
+                    has_focus = true;
+                }
+            });
+        }
+        if ( has_focus ) {
             if (fromDrag){
                 return;
             }
-            
             this.metaWindow.minimize(global.get_current_time());
             this.actor.remove_style_pseudo_class('focus');
         }
@@ -439,15 +540,33 @@ AppMenuButton.prototype = {
             if (this.metaWindow.minimized) {
                 this.metaWindow.unminimize(global.get_current_time()); 
             }
+            let ws = this.metaWindow.get_workspace().index()
+            if (ws != global.screen.get_active_workspace_index()) {
+                global.screen.get_workspace_by_index(ws).activate(global.get_current_time());
+            }
             Main.activateWindow(this.metaWindow, global.get_current_time());
             this.actor.add_style_pseudo_class('focus');
+            this._removeAlerts(this.metaWindow);
+        }
+    },
+
+    _removeAlerts: function(metaWindow) {
+        for (let i = 0; i < this.alert_list.length; i++) {
+            if (metaWindow == this.alert_list[i].metaWindow) {
+                let alert = this.alert_list[i];
+                if (alert.actor.get_parent()) {
+                    alert.actor.get_parent().remove_actor(alert.actor);
+                }
+                alert.actor.destroy();
+                this.alert_list.splice(i, 1);
+            }
         }
     },
 
     handleDragOver: function(source, actor, x, y, time) {
         if (this._inEditMode) return DND.DragMotionResult.MOVE_DROP;
         if (source instanceof AppMenuButton) return DND.DragMotionResult.CONTINUE;
-        
+        drag_helper.temp_show_panels();
         if (typeof(this._applet.dragEnterTime) == 'undefined') {
             this._applet.dragEnterTime = time;
         } else {
@@ -456,7 +575,6 @@ AppMenuButton.prototype = {
                 this._applet.dragEnterTime = time;
             }
         }
-                
         if (time > (this._applet.dragEnterTime + 300)) {
             this._windowHandle(true);
         }
@@ -520,7 +638,7 @@ AppMenuButton.prototype = {
         alloc.natural_size = naturalSize;
         [minSize, naturalSize] = this._label.get_preferred_width(forHeight);
 	alloc.min_size = alloc.min_size + Math.max(0, minSize - Math.floor(alloc.min_size / 2));
-        alloc.natural_size = 150;
+        alloc.natural_size = 150 * global.ui_scale;
     },
 
     _getContentPreferredHeight: function(actor, forWidth, alloc) {
@@ -555,7 +673,7 @@ AppMenuButton.prototype = {
         }
         this._iconBox.allocate(childBox, flags);
 
-        let iconWidth = this.iconSize;
+        let iconWidth = this.iconSize * global.ui_scale;
 
         [minWidth, minHeight, naturalWidth, naturalHeight] = this._label.get_preferred_size();
 
@@ -596,20 +714,56 @@ AppMenuButton.prototype = {
     getDragActorSource: function() {
         return this.actor;
     },
-    
+
     set_icon: function(panel_height) {
+      let tracker = Cinnamon.WindowTracker.get_default();
+      let app = tracker.get_window_app(this.metaWindow);
+
       if (global.settings.get_boolean('panel-scale-text-icons') && global.settings.get_boolean('panel-resizable')) {
-        this.iconSize = Math.round(panel_height * ICON_HEIGHT_FACTOR);
+        this.iconSize = Math.round(panel_height * ICON_HEIGHT_FACTOR / global.ui_scale);
       }
       else {
         this.iconSize = DEFAULT_ICON_SIZE;
       }
-      let icon = this.app ?
-                            this.app.create_icon_texture(this.iconSize) :
+      let icon = app ?
+                            app.create_icon_texture(this.iconSize) :
                             new St.Icon({ icon_name: 'application-default-icon',
                                          icon_type: St.IconType.FULLCOLOR,
                                          icon_size: this.iconSize });
+
+
+      let old_child = this._iconBox.get_child();
       this._iconBox.set_child(icon);
+      if (old_child != null) { 
+        old_child.destroy();
+      }      
+    },
+
+    getAttention: function() {
+        if (this._needsAttention) {
+            return false;
+        }
+        this._needsAttention = true;
+        let counter = 0;
+        this._flashButton(counter);
+        return true;
+    },
+
+    _flashButton: function(counter) {
+        if (!this._needsAttention) {
+            return;
+        }
+        this.actor.add_style_class_name("window-list-item-demands-attention");
+        if (counter < 4) {
+            Mainloop.timeout_add(500, Lang.bind(this, function () {
+                if (this.actor.has_style_class_name("window-list-item-demands-attention")) {
+                    this.actor.remove_style_class_name("window-list-item-demands-attention");
+                }
+                Mainloop.timeout_add(500, Lang.bind(this, function () {
+                    this._flashButton(++counter)
+                }));
+            }));
+        }
     }
 };
 
@@ -620,7 +774,7 @@ function MyAppletBox(applet) {
 MyAppletBox.prototype = {
     _init: function(applet) {
         this.actor = new St.BoxLayout({ name: 'windowList',
-                                       	style_class: 'window-list-box' });
+                                        style_class: 'window-list-box' });
         this.actor._delegate = this;
         
         this._applet = applet;
@@ -632,8 +786,11 @@ MyAppletBox.prototype = {
     
     handleDragOver: function(source, actor, x, y, time) {
         if (this._inEditMode) return DND.DragMotionResult.MOVE_DROP;
-        if (!(source instanceof AppMenuButton)) return DND.DragMotionResult.NO_DROP;
-        
+        if (!(source instanceof AppMenuButton))  {
+            drag_helper.temp_show_panels();
+            return DND.DragMotionResult.NO_DROP;
+        }
+
         let children = this.actor.get_children();
         let windowPos = children.indexOf(source.actor);
         
@@ -704,6 +861,19 @@ MyAppletBox.prototype = {
     }
 }
 
+function MyAppletAlertBox(applet) {
+    this._init(applet);
+}
+
+MyAppletAlertBox.prototype = {
+    _init: function(applet) {
+        this.actor = new St.BoxLayout({ name: 'windowList',
+                                        style_class: 'window-list-box' });
+        this.actor._delegate = this;
+        this._applet = applet;
+    },
+}
+
 function MyApplet(orientation, panel_height) {
     this._init(orientation, panel_height);
 }
@@ -717,85 +887,163 @@ MyApplet.prototype = {
         try {                    
             this.orientation = orientation;
             this.dragInProgress = false;
-            
+
             this.myactorbox = new MyAppletBox(this);
+            this.leftAlertBox = new MyAppletAlertBox(this);
+            this.rightAlertBox = new MyAppletAlertBox(this);
+
             this.myactor = this.myactorbox.actor;
-        
+
+            this.actor.add(this.leftAlertBox.actor);
             this.actor.add(this.myactor);
+            this.actor.add(this.rightAlertBox.actor);
+
             this.actor.reactive = global.settings.get_boolean("panel-edit-mode");
-                                       	
-            if (orientation == St.Side.TOP) {
-                this.myactor.add_style_class_name('window-list-box-top');
-                this.myactor.set_style('margin-top: 0px;');
-                this.myactor.set_style('padding-top: 0px;');
-                this.actor.set_style('margin-top: 0px;');
-                this.actor.set_style('padding-top: 0px;');
-            }
-            else {
-                this.myactor.add_style_class_name('window-list-box-bottom');
-                this.myactor.set_style('margin-bottom: 0px;');
-                this.myactor.set_style('padding-bottom: 0px;');
-                this.actor.set_style('margin-bottom: 0px;');
-                this.actor.set_style('padding-bottom: 0px;');
-            }
+            this.on_orientation_changed(orientation);
 
             this._windows = new Array();
-
+            this._alertWindows = new Array();
             let tracker = Cinnamon.WindowTracker.get_default();
             tracker.connect('notify::focus-app', Lang.bind(this, this._onFocus));
+            global.screen.get_display().connect('notify::focus-window', Lang.bind(this, this._onFocusWindow));
 
             this.switchWorkspaceHandler = global.window_manager.connect('switch-workspace',
                                             Lang.bind(this, this._refreshItems));
+            
             global.window_manager.connect('minimize',
-                                            Lang.bind(this, this._onMinimize));
+                                            Lang.bind(this, this._onWindowStateChange));
             global.window_manager.connect('maximize',
-                                            Lang.bind(this, this._onMaximize));
+                                            Lang.bind(this, this._onWindowStateChange));
             global.window_manager.connect('unmaximize',
-                                            Lang.bind(this, this._onMaximize));
+                                            Lang.bind(this, this._onWindowStateChange));
             global.window_manager.connect('map',
-                                            Lang.bind(this, this._onMap));
+                                            Lang.bind(this, this._onWindowStateChange));
+            global.window_manager.connect('tile',
+                                            Lang.bind(this, this._onWindowStateChange));
                                             
             this._workspaces = [];
             this._changeWorkspaces();
             global.screen.connect('notify::n-workspaces',
                                     Lang.bind(this, this._changeWorkspaces));
-            global.display.connect('window-demands-attention', Lang.bind(this, this._onWindowDemandsAttention));
-            global.display.connect('window-marked-urgent', Lang.bind(this, this._onWindowDemandsAttention));
-                                    
+            this._urgent_signal = null;
+            global.settings.connect('changed::window-list-applet-alert', Lang.bind(this, this._updateAttentionGrabber));
+            this._updateAttentionGrabber();
             // this._container.connect('allocate', Lang.bind(Main.panel, this._allocateBoxes)); 
-            
             global.settings.connect('changed::panel-edit-mode', Lang.bind(this, this.on_panel_edit_mode_changed));
         }
         catch (e) {
             global.logError(e);
         }
     },
-    
+
+    on_orientation_changed: function(orientation) {
+        let box_list = this.actor.get_children();
+        if (orientation == St.Side.TOP) {
+            for (let i = 0; i < box_list.length; i++) {
+                box_list[i].add_style_class_name('window-list-box-top');
+                box_list[i].set_style('margin-top: 0px;');
+                box_list[i].set_style('padding-top: 0px;');
+            }
+            this.actor.set_style('margin-top: 0px;');
+            this.actor.set_style('padding-top: 0px;');
+        }
+        else {
+            for (let i = 0; i < box_list.length; i++) {
+                box_list[i].add_style_class_name('window-list-box-bottom');
+                box_list[i].set_style('margin-bottom: 0px;');
+                box_list[i].set_style('padding-bottom: 0px;');
+            }
+            this.actor.set_style('margin-bottom: 0px;');
+            this.actor.set_style('padding-bottom: 0px;');
+        }
+    },
+
+    _updateAttentionGrabber: function() {
+        let active = global.settings.get_boolean('window-list-applet-alert');
+        if (active) {
+            this._urgent_signal = global.display.connect('window-marked-urgent', Lang.bind(this, this._onWindowDemandsAttention));
+        } else {
+            if (this._urgent_signal) {
+                global.display.disconnect(this._urgent_signal);
+            }
+        }
+    },
+
     on_applet_clicked: function(event) {
-            
     },
 
     on_panel_edit_mode_changed: function() {
         this.actor.reactive = global.settings.get_boolean("panel-edit-mode");
     }, 
-    
-           
+
     _onWindowDemandsAttention : function(display, window) {
         for ( let i=0; i<this._windows.length; ++i ) {
-            if ( this._windows[i].metaWindow == window ) {                
-                this._windows[i].actor.add_style_class_name("window-list-item-demands-attention");                
+            if ( this._windows[i].metaWindow == window ) {
+                if (this._windows[i].actor._delegate.getAttention()) {
+                    this._alertWindows = this._alertWindows.filter(function(alertWindow) {
+                        return alertWindow.metaWindow != window; // we don't want duplicates
+                    }, this);
+                    let alertButton = new AppMenuButton(this, window, true, this.orientation, this._panelHeight, false);
+                    this._alertWindows.push(alertButton);
+                    this.calculate_alert_positions();
+                }
+                return;
             }
         }
     },
 
-    _onFocus: function() {
+    _clean_alert_boxes: function() {
+        let left_box_items = this.leftAlertBox.actor.get_children();
+        for (let i = 0; i < left_box_items.length; i++) {
+            this.leftAlertBox.actor.remove_actor(left_box_items[i]);
+        }
+        let right_box_items = this.rightAlertBox.actor.get_children();
+        for (let i = 0; i < right_box_items.length; i++) {
+            this.rightAlertBox.actor.remove_actor(right_box_items[i]);
+        }
+    },
+
+    calculate_alert_positions: function() {
+        this._clean_alert_boxes();
+
+        // purge destroyed alert windows
+        this._alertWindows = this._alertWindows.filter(function(alertWindow) {
+            return alertWindow.metaWindow.get_workspace() != null;
+        }, this);
+
+        let cur_ws_index = global.screen.get_active_workspace_index();
+
+        for (let i = 0; i < this._alertWindows.length; i++ ) {
+            let window_ws_index = this._alertWindows[i].metaWindow.get_workspace().index();
+            if (window_ws_index < cur_ws_index) {
+                this.leftAlertBox.actor.add(this._alertWindows[i].actor);
+            } else if (window_ws_index > cur_ws_index) {
+                this.rightAlertBox.actor.add(this._alertWindows[i].actor);
+            }
+            if (!this._alertWindows[i]._needsAttention) {
+                this._alertWindows[i].getAttention();
+            }
+        }
+    },
+
+    _onFocus: function() {        
         for ( let i = 0; i < this._windows.length; ++i ) {
-            let window = this._windows[i];
+            let window = this._windows[i];                        
             window.set_icon(this._panelHeight);
             window.doFocus();
         }
     },
 
+    _onFocusWindow: function(display) {
+        let currentWindow = display.focus_window;
+        if (currentWindow && this._alertWindows.length) {
+            this._alertWindows = this._alertWindows.filter(function(alertWindow) {
+                return alertWindow.metaWindow != currentWindow;
+            }, this);
+            this.calculate_alert_positions();
+        }
+    },
+
     on_panel_height_changed: function() {
         this._refreshItems();
     },
@@ -809,50 +1057,20 @@ MyApplet.prototype = {
             else
                 this._windows[i].actor.hide();
         }
-
+        this.calculate_alert_positions();
         this._onFocus();
     },
 
-    _onWindowStateChange: function(state, actor) {
+    _onWindowStateChange: function(cinnamonwm, actor) {
         for ( let i=0; i<this._windows.length; ++i ) {
             if ( this._windows[i].metaWindow == actor.get_meta_window() ) {
                 let windowReference = this._windows[i];
                 let title = windowReference.getDisplayTitle();
-                
-                if (state == 'minimize') {
-                    windowReference._label.set_text("["+ title +"]");
-                    return;
-                } else if (state == 'map') {
-                    windowReference._label.set_text(title);
-                    return;
-                }
+                windowReference._label.set_text(title);                
             }
         }
     },
-    
-    _onMinimize: function(cinnamonwm, actor) {
-        this._onWindowStateChange('minimize', actor);
-    },
-    
-    _onMaximize: function(cinnamonwm, actor) {
-        this._onWindowStateChange('maximize', actor);
-    },
-    
-    _onMap: function(cinnamonwm, actor) {
-    	/* Note by Clem: The call to this._refreshItems() below doesn't look necessary. 
-    	 * When a window is mapped in a quick succession of times (for instance if 
-    	 * the user repeatedly minimize/unminimize the window by clicking on the window list, 
-    	 * or more often when the showDesktop button maps a lot of minimized windows in a quick succession.. 
-    	 * when this happens, many calls to refreshItems are made and this creates a memory leak. 
-    	 * It also slows down all the mapping and so it takes time for all windows to get unminimized after showDesktop is clicked.
-    	 * 
-    	 * For now this was removed. If it needs to be put back, this isn't the place. 
-    	 * If showDesktop needs it, then it should only call it once, not once per window.
-    	 */ 
-        //this._refreshItems();
-        this._onWindowStateChange('map', actor);
-    },
-   
+       
     getOriginFromWindow: function(metaWindow) {
         for ( let i=0; i<this._windows.length; ++i ) {
             if ( this._windows[i].metaWindow == metaWindow ) {
@@ -872,7 +1090,7 @@ MyApplet.prototype = {
             }
         }
 
-        let appbutton = new AppMenuButton(this, metaWindow, true, this.orientation, this._panelHeight);
+        let appbutton = new AppMenuButton(this, metaWindow, true, this.orientation, this._panelHeight, true);
         this._windows.push(appbutton);
         this.myactor.add(appbutton.actor);
         if (metaWorkspace.index() != global.screen.get_active_workspace_index())
@@ -888,6 +1106,7 @@ MyApplet.prototype = {
                 break;
             }
         }
+        this.calculate_alert_positions();
     },
     
     _changeWorkspaces: function() {
