@@ -24,8 +24,8 @@ const POINTER_ENTER_MILLISECONDS_GRACE = 150;
 const RESCALE_ANIMATION_TIME = 0.2;
 const SLIDE_ANIMATION_TIME = 0.3;
 const INACTIVE_OPACITY = 120;
-const REARRANGE_TIME_ON = 0.3;
-const REARRANGE_TIME_OFF = 0.3 * 2;
+const REARRANGE_TIME_ON = 0.1;
+const REARRANGE_TIME_OFF = 0.3;
 const ICON_OPACITY = Math.round(255 * 0.9);
 const ICON_SIZE = 128;
 const ICON_OFFSET = -5;
@@ -48,7 +48,7 @@ ExpoWindowClone.prototype = {
         this.actor._delegate = this;
         this.realWindow = realWindow;
         this.metaWindow = realWindow.meta_window;
-
+        this.refreshClone();
         let positionChangedId = this.realWindow.connect('position-changed',
                                                           Lang.bind(this, this.onPositionChanged));
         let sizeChangedId = this.realWindow.connect('position-changed',
@@ -148,7 +148,7 @@ ExpoWindowClone.prototype = {
         this.actor.add_actor(this.clone);
         let [pwidth, pheight] = [this.realWindow.width, this.realWindow.height];
         let clones = WindowUtils.createWindowClone(this.metaWindow, 0, 0, withTransients);
-        for (i in clones) {
+        for (let i in clones) {
             let clone = clones[i].actor;
             this.clone.add_actor(clone);
             let [width, height] = clone.get_size();
@@ -259,6 +259,10 @@ ExpoWindowClone.prototype = {
         {
             this.emit('selected', event.get_time());
         }
+        if ((Cinnamon.get_event_state(event) & Clutter.ModifierType.BUTTON2_MASK))
+        {
+            this.emit('middle-button-release', event.get_time());
+        }
         return true;
     },
 
@@ -380,23 +384,6 @@ ExpoWorkspaceThumbnail.prototype = {
 
         this.actor.connect('scroll-event', Lang.bind(this, this.onScrollEvent));
         
-        this.closeWindowButton = new St.Button({ style_class: 'window-close' });
-        this.actor.add_actor(this.closeWindowButton);
-        this.closeWindowButton.connect('clicked', Lang.bind(this, function(actor, event) {
-            if (this.lastHoveredClone) {
-                this.lastHoveredClone.metaWindow.delete(global.get_current_time());
-                this.resetCloneHover();
-            }
-        }));
-        this.closeWindowButton.connect('enter-event', Lang.bind(this, function(actor, event) {
-            this.closeWindowButton._oldStyle = this.closeWindowButton.style;
-            this.closeWindowButton.style = "border: 1px solid rgba(255,0,0,0.3);"
-        }));
-        this.closeWindowButton.connect('leave-event', Lang.bind(this, function(actor, event) {
-            this.closeWindowButton.style = this.closeWindowButton._oldStyle;
-        }));
-        this.closeWindowButton.hide();
-
         this.title = new St.Entry({ style_class: 'expo-workspaces-name-entry',                                     
                                      track_hover: true,
                                      can_focus: true });                
@@ -736,6 +723,9 @@ ExpoWorkspaceThumbnail.prototype = {
                 this.box.emit('sticky-detected', clone.metaWindow);
             }
         }));
+        clone.connect('middle-button-release', Lang.bind(this, function(sender, time) {
+            clone.metaWindow.delete(time);
+        }));
         clone.connect('hovering', Lang.bind(this, this.onCloneHover));
         clone.connect('demanding-attention', Lang.bind(this, function() {this.overviewModeOn();}));
         clone.connect('selected', Lang.bind(this, this.activate));
@@ -765,7 +755,6 @@ ExpoWorkspaceThumbnail.prototype = {
     },
 
     resetCloneHover : function () {
-        this.closeWindowButton.hide();
         this.lastHoveredClone = null;
         if (this.tooltip) {
             this.tooltip.destroy();
@@ -789,16 +778,6 @@ ExpoWorkspaceThumbnail.prototype = {
                     this.resetCloneHover();
                     return;
                 }
-                let [x,y] = clone.actor.get_position();
-                let [scaleX, scaleY] = clone.actor.get_scale();
-                let iboxScale = 1/this.box.scale;
-                let themeNode = this.closeWindowButton.get_theme_node();
-                let overlap = (themeNode.get_length('-cinnamon-close-overlap') / 2) * iboxScale;
-                let xOffset = overlap + Math.round((-this.closeWindowButton.width) * iboxScale + clone.actor.width * scaleX);
-                let yOffset = -overlap;
-                this.closeWindowButton.set_scale(iboxScale, iboxScale);
-                this.closeWindowButton.set_position(x + xOffset, y + yOffset);
-                this.closeWindowButton.show();
                 if (this.tooltip) {
                     this.tooltip.destroy();
                 }
@@ -987,9 +966,7 @@ ExpoWorkspaceThumbnail.prototype = {
         if (dropping) {
             let draggable = source._draggable;
             actor.opacity = draggable._dragOrigOpacity;
-            // Can't use reparent here, it produces strange warnings about widget not being in the stage
-            actor.get_parent().remove_actor(actor);
-            draggable._dragOrigParent.add_actor(actor);
+            global.reparentActor(actor, draggable._dragOrigParent);
         }
 
         if (source == Main.xdndHandler) {
