@@ -16,7 +16,7 @@ const MessageTray = imports.ui.messageTray;
 const KEYBOARD_SCHEMA = 'org.cinnamon.keyboard';
 const KEYBOARD_TYPE = 'keyboard-type';
 
-const A11Y_APPLICATIONS_SCHEMA = 'org.gnome.desktop.a11y.applications';
+const A11Y_APPLICATIONS_SCHEMA = 'org.cinnamon.desktop.a11y.applications';
 const SHOW_KEYBOARD = 'screen-keyboard-enabled';
 
 // Key constants taken from Antler
@@ -288,7 +288,7 @@ Keyboard.prototype = {
         if (focus && (focus._extended_keys || (focus._key && focus._key.extended_key)))
             return;
 
-        let time = global.current_event_time();
+        let time = global.get_current_time();
         if (focus instanceof Clutter.Text)
             this.Show(time);
         else
