@@ -444,6 +444,10 @@ LayoutManager.prototype = {
 
     findMonitorForActor: function(actor) {
         return this._chrome.findMonitorForActor(actor);
+    },
+
+    isTrackingChrome: function(actor) {
+        return this._chrome._findActor(actor) != -1;
     }
 };
 Signals.addSignalMethods(LayoutManager.prototype);
@@ -655,7 +659,6 @@ Chrome.prototype = {
     _relayout: function() {
         this._monitors = this._layoutManager.monitors;
         this._primaryMonitor = this._layoutManager.primaryMonitor;
-
         this._updateFullscreen();
         this._updateVisibility();
         this._queueUpdateRegions();
@@ -808,97 +811,98 @@ Chrome.prototype = {
             this._updateRegionIdle = 0;
         }
 
-        for (i = 0; i < this._trackedActors.length; i++) {
-            let actorData = this._trackedActors[i];
-            if ((!actorData.affectsInputRegion && !actorData.affectsStruts) ||
-                 primary.inFullscreen)
-                continue;
+        if (!primary.inFullscreen) {
+            for (let i = 0; i < this._trackedActors.length; i++) {
+                let actorData = this._trackedActors[i];
+                if (!actorData.affectsInputRegion && !actorData.affectsStruts)
+                    continue;
 
-            let [x, y] = actorData.actor.get_transformed_position();
-            let [w, h] = actorData.actor.get_transformed_size();
-            x = Math.round(x);
-            y = Math.round(y);
-            w = Math.round(w);
-            h = Math.round(h);
-            let rect = new Meta.Rectangle({ x: x, y: y, width: w, height: h});
+                let [x, y] = actorData.actor.get_transformed_position();
+                let [w, h] = actorData.actor.get_transformed_size();
+                x = Math.round(x);
+                y = Math.round(y);
+                w = Math.round(w);
+                h = Math.round(h);
+                let rect = new Meta.Rectangle({ x: x, y: y, width: w, height: h});
 
-            if (actorData.affectsInputRegion &&
-                actorData.actor.get_paint_visibility() &&
-                !Main.uiGroup.get_skip_paint(actorData.actor))
-                rects.push(rect);
+                if (actorData.affectsInputRegion &&
+                    actorData.actor.get_paint_visibility() &&
+                    !Main.uiGroup.get_skip_paint(actorData.actor))
+                    rects.push(rect);
 
-            if (!actorData.affectsStruts)
-                continue;
+                if (!actorData.affectsStruts)
+                    continue;
 
-            // Limit struts to the size of the screen
-            let x1 = Math.max(x, 0);
-            let x2 = Math.min(x + w, global.screen_width);
-            let y1 = Math.max(y, 0);
-            let y2 = Math.min(y + h, global.screen_height);
+                // Limit struts to the size of the screen
+                let x1 = Math.max(x, 0);
+                let x2 = Math.min(x + w, global.screen_width);
+                let y1 = Math.max(y, 0);
+                let y2 = Math.min(y + h, global.screen_height);
 
-            // NetWM struts are not really powerful enought to handle
-            // a multi-monitor scenario, they only describe what happens
-            // around the outer sides of the full display region. However
-            // it can describe a partial region along each side, so
-            // we can support having the struts only affect the
-            // primary monitor. This should be enough as we only have
-            // chrome affecting the struts on the primary monitor so
-            // far.
-            //
-            // Metacity wants to know what side of the screen the
-            // strut is considered to be attached to. If the actor is
-            // only touching one edge, or is touching the entire
-            // border of the primary monitor, then it's obvious which
-            // side to call it. If it's in a corner, we pick a side
-            // arbitrarily. If it doesn't touch any edges, or it spans
-            // the width/height across the middle of the screen, then
-            // we don't create a strut for it at all.
-            let side;
-            if (x1 <= primary.x && x2 >= primary.x + primary.width) {
-                if (y1 <= primary.y)
+                // NetWM struts are not really powerful enought to handle
+                // a multi-monitor scenario, they only describe what happens
+                // around the outer sides of the full display region. However
+                // it can describe a partial region along each side, so
+                // we can support having the struts only affect the
+                // primary monitor. This should be enough as we only have
+                // chrome affecting the struts on the primary monitor so
+                // far.
+                //
+                // Metacity wants to know what side of the screen the
+                // strut is considered to be attached to. If the actor is
+                // only touching one edge, or is touching the entire
+                // border of the primary monitor, then it's obvious which
+                // side to call it. If it's in a corner, we pick a side
+                // arbitrarily. If it doesn't touch any edges, or it spans
+                // the width/height across the middle of the screen, then
+                // we don't create a strut for it at all.
+                let side;
+                if (x1 <= primary.x && x2 >= primary.x + primary.width) {
+                    if (y1 <= primary.y)
+                        side = Meta.Side.TOP;
+                    else if (y2 >= primary.y + primary.height)
+                        side = Meta.Side.BOTTOM;
+                    else
+                        continue;
+                } else if (y1 <= primary.y && y2 >= primary.y + primary.height) {
+                    if (x1 <= 0)
+                        side = Meta.Side.LEFT;
+                    else if (x2 >= global.screen_width)
+                        side = Meta.Side.RIGHT;
+                    else
+                        continue;
+                } else if (x1 <= 0)
+                    side = Meta.Side.LEFT;
+                else if (y1 <= 0)
                     side = Meta.Side.TOP;
-                else if (y2 >= primary.y + primary.height)
+                else if (x2 >= global.screen_width)
+                    side = Meta.Side.RIGHT;
+                else if (y2 >= global.screen_height)
                     side = Meta.Side.BOTTOM;
                 else
                     continue;
-            } else if (y1 <= primary.y && y2 >= primary.y + primary.height) {
-                if (x1 <= 0)
-                    side = Meta.Side.LEFT;
-                else if (x2 >= global.screen_width)
-                    side = Meta.Side.RIGHT;
-                else
-                    continue;
-            } else if (x1 <= 0)
-                side = Meta.Side.LEFT;
-            else if (y1 <= 0)
-                side = Meta.Side.TOP;
-            else if (x2 >= global.screen_width)
-                side = Meta.Side.RIGHT;
-            else if (y2 >= global.screen_height)
-                side = Meta.Side.BOTTOM;
-            else
-                continue;
 
-            // Ensure that the strut rects goes all the way to the screen edge,
-            // as this really what muffin expects.
-            switch (side) {
-            case Meta.Side.TOP:
-                y1 = 0;
-                break;
-            case Meta.Side.BOTTOM:
-                y2 = global.screen_height;
-                break;
-            case Meta.Side.LEFT:
-                x1 = 0;
-                break;
-            case Meta.Side.RIGHT:
-                x2 = global.screen_width;
-                break;
+                // Ensure that the strut rects goes all the way to the screen edge,
+                // as this really what muffin expects.
+                switch (side) {
+                case Meta.Side.TOP:
+                    y1 = 0;
+                    break;
+                case Meta.Side.BOTTOM:
+                    y2 = global.screen_height;
+                    break;
+                case Meta.Side.LEFT:
+                    x1 = 0;
+                    break;
+                case Meta.Side.RIGHT:
+                    x2 = global.screen_width;
+                    break;
+                }
+
+                let strutRect = new Meta.Rectangle({ x: x1, y: y1, width: x2 - x1, height: y2 - y1});
+                let strut = new Meta.Strut({ rect: strutRect, side: side });
+                struts.push(strut);
             }
-
-            let strutRect = new Meta.Rectangle({ x: x1, y: y1, width: x2 - x1, height: y2 - y1});
-            let strut = new Meta.Strut({ rect: strutRect, side: side });
-            struts.push(strut);
         }
 
         let enable_stage = true;
