@@ -1,7 +1,44 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
+/**
+ * FILE:main.js
+ * @placesManager (PlacesManager.PlacesManager): The places manager
+ * @overview (Overview.Overview): The "scale" overview 
+ * @expo (Expo.Expo): The "expo" overview
+ * @runDialog (RunDialog.RunDialog): The run dialog
+ * @lookingGlass (LookingGlass.LookingGlass): The looking glass
+ * @wm (WindowManager.WindowManager): The window manager
+ * @messageTray (MessageTray.MessageTray): The mesesage tray
+ * @notificationDaemon (NotificationDaemon.NotificationDaemon): The notification daemon
+ * @windowAttentionHandler (WindowAttentionHandler.WindowAttentionHandler): The window attention handler
+ * @recorder (Cinnamon.Recorder): The recorder
+ * @cinnamonDBusService (CinnamonDBus.Cinnamon): The cinnamon dbus object
+ * @modalCount (int): The number of modals "pushed"
+ * @modalActorFocusStack (array): Array of pushed modal actors
+ * @uiGroup (Cinnamon.GenericContainer): The group containing all Cinnamon
+ *                                       and Muffin actors
+ * @magnifier (Magnifier.Magnifier): The magnifier
+ * @xdndHandler (XdndHandler.XdndHandler): The X DND handler
+ * @statusIconDispatcher (StatusIconDispatcher.StatusIconDispatcher): The status icon dispatcher
+ * @keyboard (Keyboard.Keyboard): The keyboard object
+ * @layoutManager (Layout.LayoutManager): The layout manager
+ * @themeManager (ThemeManager.ThemeManager): The theme manager
+ * @dynamicWorkspaces (boolean): Whether dynamic workspaces are to be used.
+ *                               This is not yet implemented
+ * @nWorks (int): Number of workspaces
+ * @tracker (Cinnamon.WindowTracker): The window tracker
+ * @workspace_names (array): Names of workspace
+ * @background (null): Unused
+ * @deskletContainer (DeskletManager.DeskletContainer): The desklet container 
+ * @software_rendering (boolean): Whether software rendering is used
+ * @lg_log_file (Gio.FileOutputStream): The stream used to log looking messages
+ *                                      to ~/.cinnamon/glass.log
+ * @can_log (boolean): Whether looking glass log to file can be used
+ *
+ * The main file is responsible for launching Cinnamon as well as creating its components. Most components of Cinnamon can be accessed through main
+ */
 
 const Clutter = imports.gi.Clutter;
-const DBus = imports.dbus;
+const DBus= imports.dbus;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
@@ -11,12 +48,10 @@ const Cinnamon = imports.gi.Cinnamon;
 const St = imports.gi.St;
 const PointerTracker = imports.misc.pointerTracker;
 
+const SoundManager = imports.ui.soundManager;
+const BackgroundManager = imports.ui.backgroundManager;
 const AppletManager = imports.ui.appletManager;
-const AutomountManager = imports.ui.automountManager;
-const AutorunManager = imports.ui.autorunManager;
 const DeskletManager = imports.ui.deskletManager;
-const EndSessionDialog = imports.ui.endSessionDialog;
-const PolkitAuthenticationAgent = imports.ui.polkitAuthenticationAgent;
 const ExtensionSystem = imports.ui.extensionSystem;
 const Keyboard = imports.ui.keyboard;
 const MessageTray = imports.ui.messageTray;
@@ -27,7 +62,6 @@ const PlacesManager = imports.ui.placesManager;
 const RunDialog = imports.ui.runDialog;
 const Layout = imports.ui.layout;
 const LookingGlass = imports.ui.lookingGlass;
-const NetworkAgent = imports.ui.networkAgent;
 const NotificationDaemon = imports.ui.notificationDaemon;
 const WindowAttentionHandler = imports.ui.windowAttentionHandler;
 const Scripting = imports.ui.scripting;
@@ -39,6 +73,9 @@ const Magnifier = imports.ui.magnifier;
 const XdndHandler = imports.ui.xdndHandler;
 const StatusIconDispatcher = imports.ui.statusIconDispatcher;
 const Util = imports.misc.util;
+const Keybindings = imports.ui.keybindings;
+const Settings = imports.ui.settings;
+const Systray = imports.ui.systray;
 
 const DEFAULT_BACKGROUND_COLOR = new Clutter.Color();
 DEFAULT_BACKGROUND_COLOR.from_pixel(0x2266bbff);
@@ -49,12 +86,11 @@ const LAYOUT_CLASSIC = "classic";
 
 const CIN_LOG_FOLDER = GLib.get_home_dir() + '/.cinnamon/';
 
-let automountManager = null;
-let autorunManager = null;
-
 let panel = null;
 let panel2 = null;
 
+let soundManager = null;
+let backgroundManager = null;
 let placesManager = null;
 let overview = null;
 let expo = null;
@@ -76,7 +112,7 @@ let statusIconDispatcher = null;
 let keyboard = null;
 let layoutManager = null;
 let themeManager = null;
-let networkAgent = null;
+let keybindingManager = null;
 let _errorLogStack = [];
 let _startDate;
 let _defaultCssStylesheet = null;
@@ -84,7 +120,8 @@ let _cssStylesheet = null;
 let dynamicWorkspaces = null;
 let nWorks = null;
 let tracker = null;
-let desktopShown;
+let settingsManager = null;
+let systrayManager = null;
 
 let workspace_names = [];
 
@@ -96,12 +133,9 @@ let deskletContainer = null;
 
 let software_rendering = false;
 
-
 let lg_log_file;
 let can_log = false;
 
-
-
 // Override Gettext localization
 const Gettext = imports.gettext;
 Gettext.bindtextdomain('cinnamon', '/usr/share/cinnamon/locale');
@@ -141,6 +175,8 @@ function _initUserSession() {
 
     global.screen.override_workspace_layout(Meta.ScreenCorner.TOPLEFT, false, 1, -1);
 
+    systrayManager = new Systray.SystrayManager();
+    
     ExtensionSystem.init();
 
     Meta.keybindings_set_custom_handler('panel-run-dialog', function() {
@@ -153,7 +189,22 @@ function _initUserSession() {
     
 }
 
+function _reparentActor(actor, newParent) {
+    let parent = actor.get_parent();
+    if (parent)
+      parent.remove_actor(actor);
+    if(newParent)
+        newParent.add_actor(actor);
+}
+
+/**
+ * start:
+ *
+ * Starts cinnamon. Should not be called in JavaScript code
+ */
 function start() {
+    global.reparentActor = _reparentActor;
+
     // Monkey patch utility functions into the global proxy;
     // This is easier and faster than indirecting down into global
     // if we want to call back up into JS.
@@ -190,9 +241,9 @@ function start() {
     }
 
     // Chain up async errors reported from C
-    global.connect('notify-error', function (global, msg, detail) { notifyError(msg, detail); });
+    global.connect('notify-error', function (global, msg, detail) { notifyError(msg, detail); });    
 
-    Gio.DesktopAppInfo.set_desktop_env('GNOME');
+    Gio.DesktopAppInfo.set_desktop_env('X-Cinnamon');
 
     cinnamonDBusService = new CinnamonDBus.Cinnamon();
     lookingGlassDBusService = new LookingGlassDBus.CinnamonLookingGlass();
@@ -211,7 +262,7 @@ function start() {
     // races for now we initialize it here.  It's better to
     // be predictable anyways.
     tracker = Cinnamon.WindowTracker.get_default();
-    Cinnamon.AppUsage.get_default();
+    Cinnamon.AppSystem.get_default();
 
     // The stage is always covered so Clutter doesn't need to clear it; however
     // the color is used as the default contents for the Muffin root background
@@ -228,9 +279,16 @@ function start() {
     }
     
     Gtk.IconTheme.get_default().append_search_path("/usr/share/cinnamon/icons/");
-    _defaultCssStylesheet = global.datadir + '/theme/cinnamon.css';
+    _defaultCssStylesheet = global.datadir + '/theme/cinnamon.css';    
+
+    soundManager = new SoundManager.SoundManager();
 
     themeManager = new ThemeManager.ThemeManager();
+
+    settingsManager = new Settings.SettingsManager();
+
+    backgroundManager = new BackgroundManager.BackgroundManager();
+    
     deskletContainer = new DeskletManager.DeskletContainer();
 
     // Set up stage hierarchy to group all UI actors under one container.
@@ -243,31 +301,14 @@ function start() {
                     });
     St.set_ui_root(global.stage, uiGroup);
 
-    let parent = global.background_actor.get_parent();
-    if (parent) {
-      parent.remove_child(global.background_actor);
-    }
-    parent = global.bottom_window_group.get_parent();
-    if (parent) {
-      parent.remove_child(global.bottom_window_group);
-    }
-    parent = global.window_group.get_parent();
-    if (parent) {
-      parent.remove_child(global.window_group);
-    }
-    parent = global.overlay_group.get_parent();
-    if (parent) {
-      parent.remove_child(global.overlay_group);
-    }
-
-    uiGroup.add_actor(global.background_actor);
-    uiGroup.add_actor(global.bottom_window_group);
+    global.reparentActor(global.background_actor, uiGroup);
+    global.reparentActor(global.bottom_window_group, uiGroup);
     uiGroup.add_actor(deskletContainer.actor);
-    uiGroup.add_actor(global.window_group);
-    uiGroup.add_actor(global.overlay_group);
+    global.reparentActor(global.window_group, uiGroup);
+    global.reparentActor(global.overlay_group, uiGroup);
 
     global.stage.add_actor(uiGroup);
-    global.top_window_group.reparent(global.stage);
+    global.reparentActor(global.top_window_group, global.stage);
 
     layoutManager = new Layout.LayoutManager();
     let pointerTracker = new PointerTracker.PointerTracker();
@@ -278,7 +319,7 @@ function start() {
     // This overview object is just a stub for non-user sessions
     overview = new Overview.Overview();
     expo = new Expo.Expo();
-    magnifier = new Magnifier.Magnifier();
+
     statusIconDispatcher = new StatusIconDispatcher.StatusIconDispatcher();  
                     
     if (desktop_layout == LAYOUT_TRADITIONAL) {
@@ -291,7 +332,8 @@ function start() {
         panel.actor.add_style_class_name('panel-top');
         layoutManager.panelBox.add(panel.actor);
     }
-    else if (desktop_layout == LAYOUT_CLASSIC) {
+    else {
+        desktop_layout == LAYOUT_CLASSIC;
         panel = new Panel.Panel(false, true);
         panel2 = new Panel.Panel(true, false);
         panel.actor.add_style_class_name('panel-top');
@@ -308,9 +350,9 @@ function start() {
     windowAttentionHandler = new WindowAttentionHandler.WindowAttentionHandler();
 
     placesManager = new PlacesManager.PlacesManager();    
-    automountManager = new AutomountManager.AutomountManager();
-    //autorunManager = new AutorunManager.AutorunManager();
-    //networkAgent = new NetworkAgent.NetworkAgent();
+
+    keybindingManager = new Keybindings.KeybindingManager();
+    magnifier = new Magnifier.Magnifier();
 
     Meta.later_add(Meta.LaterType.BEFORE_REDRAW, _checkWorkspaces);
 
@@ -333,9 +375,6 @@ function start() {
     // initiate logouts.
     //EndSessionDialog.init();
 
-    // Attempt to become a PolicyKit authentication agent
-    PolkitAuthenticationAgent.init()
-
     _startDate = new Date();
 
     global.stage.connect('captured-event', _globalKeyPressHandler);
@@ -358,12 +397,29 @@ function start() {
     global.screen.connect('window-left-monitor', _windowLeftMonitor);
     global.screen.connect('restacked', _windowsRestacked);
 
-    global.window_manager.connect('map', _onWindowMapped);
-
     _nWorkspacesChanged();
-    
+
     AppletManager.init();
     DeskletManager.init();
+
+    if (software_rendering && !GLib.getenv('CINNAMON_2D')) {
+        notifyCinnamon2d();
+    }
+}
+
+function notifyCinnamon2d() {
+    let icon = new St.Icon({ icon_name: 'display',
+                             icon_type: St.IconType.FULLCOLOR,
+                             icon_size: 36 });
+    criticalNotify(_("Running in software rendering mode"),
+                   _("Cinnamon is currently running without video hardware acceleration and, as a result, you may observe much higher than normal CPU usage.\n\n") +
+                   _("There could be a problem with your drivers or some other issue.  For the best experience, it is recommended that you only use this mode for") +
+                   _(" troubleshooting purposes."), icon);
+}
+
+function loadSoundSettings() {
+
+
 }
 
 function enablePanels() {
@@ -376,6 +432,15 @@ function disablePanels() {
     if (panel2) panel2.disable();
 }
 
+function getPanels() {
+    let panels = [];
+    if(panel)
+        panels.push(panel);
+    if(panel2)
+        panels.push(panel2);
+    return panels;
+}
+
 let _workspaces = [];
 let _checkWorkspacesId = 0;
 
@@ -407,6 +472,13 @@ function _makeDefaultWorkspaceName(index) {
     return _("WORKSPACE") + " " + (index + 1).toString();
 }
 
+/**
+ * setWorkspaceName:
+ * @index (int): index of workspace
+ * @name (string): name of workspace
+ *
+ * Sets the name of the workspace @index to @name
+ */
 function setWorkspaceName(index, name) {
     name.trim();
     if (name != getWorkspaceName(index)) {
@@ -419,6 +491,14 @@ function setWorkspaceName(index, name) {
     }
 }
 
+/**
+ * getWorkspaceName:
+ * @index (int): index of workspace
+ *
+ * Retrieves the name of the workspace @index
+ *
+ * Returns (string): name of workspace
+ */
 function getWorkspaceName(index) {
     let wsName = index < workspace_names.length ?
         workspace_names[index] :
@@ -429,6 +509,14 @@ function getWorkspaceName(index) {
         _makeDefaultWorkspaceName(index);
 }
 
+/**
+ * hasDefaultWorkspaceName:
+ * @index (int): index of workspace
+ *
+ * Whether the workspace uses the default name
+ *
+ * Returns (boolean): whether the workspace uses the default name
+ */
 function hasDefaultWorkspaceName(index) {
     return getWorkspaceName(index) == _makeDefaultWorkspaceName(index);
 }
@@ -457,6 +545,17 @@ function _removeWorkspace(workspace) {
     return true;
 }
 
+/**
+ * moveWindowToNewWorkspace:
+ * @metaWindow (Meta.Window): the window to be moved
+ * @switchToNewWorkspace (boolean): whether or not to switch to the
+ *                                  new created workspace
+ *
+ * Moves the window to a new workspace.
+ *
+ * If @switchToNewWorkspace is true, it will switch to the new workspace
+ * after moving the window
+ */
 function moveWindowToNewWorkspace(metaWindow, switchToNewWorkspace) {
     if (switchToNewWorkspace) {
         let targetCount = global.screen.n_workspaces + 1;
@@ -475,12 +574,12 @@ function _staticWorkspaces() {
     let i;
     let dif = nWorks - global.screen.n_workspaces;
     if (dif > 0) {
-        for (i = 0; i < dif; i++)
+        for (let i = 0; i < dif; i++)
             global.screen.append_new_workspace(false, global.get_current_time());
     } else {
         if (nWorks == 0)
             return false;
-        for (i = 0; i > dif; i--){
+        for (let i = 0; i > dif; i--){
             let removeWorkspaceIndex = global.screen.n_workspaces - 1;
             let removeWorkspace = global.screen.get_workspace_by_index(removeWorkspaceIndex);
             let lastRemoved = removeWorkspace._lastRemovedWindow;
@@ -496,7 +595,7 @@ function _checkWorkspaces() {
     let i;
     let emptyWorkspaces = [];
 
-    for (i = 0; i < _workspaces.length; i++) {
+    for (let i = 0; i < _workspaces.length; i++) {
         let lastRemoved = _workspaces[i]._lastRemovedWindow;
         if (lastRemoved &&
             (lastRemoved.get_window_type() == Meta.WindowType.SPLASHSCREEN ||
@@ -508,7 +607,7 @@ function _checkWorkspaces() {
     }
 
     let windows = global.get_window_actors();
-    for (i = 0; i < windows.length; i++) {
+    for (let i = 0; i < windows.length; i++) {
         let win = windows[i];
 
         if (win.get_meta_window().is_on_all_workspaces())
@@ -537,7 +636,7 @@ function _checkWorkspaces() {
     }
 
     // Delete other empty workspaces; do it from the end to avoid index changes
-    for (i = emptyWorkspaces.length - 2; i >= 0; i--) {
+    for (let i = emptyWorkspaces.length - 2; i >= 0; i--) {
         if (emptyWorkspaces[i])
             global.screen.remove_workspace(_workspaces[i], global.get_current_time());
     }
@@ -583,10 +682,6 @@ function _windowsRestacked() {
     global.sync_pointer();
 }
 
-function _onWindowMapped() {
-    desktopShown = false;
-}
-
 function _queueCheckWorkspaces() {
     if (!dynamicWorkspaces)
         return false;
@@ -608,13 +703,11 @@ function _nWorkspacesChanged() {
 
     let lostWorkspaces = [];
     if (newNumWorkspaces > oldNumWorkspaces) {
-        let w;
-
         // Assume workspaces are only added at the end
-        for (w = oldNumWorkspaces; w < newNumWorkspaces; w++)
+        for (let w = oldNumWorkspaces; w < newNumWorkspaces; w++)
             _workspaces[w] = global.screen.get_workspace_by_index(w);
 
-        for (w = oldNumWorkspaces; w < newNumWorkspaces; w++) {
+        for (let w = oldNumWorkspaces; w < newNumWorkspaces; w++) {
             let workspace = _workspaces[w];
             workspace._windowAddedId = workspace.connect('window-added', _queueCheckWorkspaces);
             workspace._windowRemovedId = workspace.connect('window-removed', _windowRemoved);
@@ -650,8 +743,8 @@ function _nWorkspacesChanged() {
  *
  * Get the theme CSS file that Cinnamon will load
  *
- * Returns: A file path that contains the theme CSS,
- *          null if using the default
+ * Returns (string): A file path that contains the theme CSS,
+ *                   null if using the default
  */
 function getThemeStylesheet()
 {
@@ -660,8 +753,8 @@ function getThemeStylesheet()
 
 /**
  * setThemeStylesheet:
- * @cssStylesheet: A file path that contains the theme CSS,
- *                  set it to null to use the default
+ * @cssStylesheet (string): A file path that contains the theme CSS,
+ *                         set it to null to use the default
  *
  * Set the theme CSS file that Cinnamon will load
  */
@@ -677,21 +770,27 @@ function setThemeStylesheet(cssStylesheet)
  */
 function loadTheme() {
     let themeContext = St.ThemeContext.get_for_stage (global.stage);
-
-    let cssStylesheet = _defaultCssStylesheet;
-    if (_cssStylesheet != null)
-        cssStylesheet = _cssStylesheet;
-
     let theme = new St.Theme ();
-    theme.load_stylesheet(cssStylesheet);
-    
+    let stylesheetLoaded = false;
+    if (_cssStylesheet != null) {
+        stylesheetLoaded = theme.load_stylesheet(_cssStylesheet);
+    }
+    if (!stylesheetLoaded) {
+        theme.load_stylesheet(_defaultCssStylesheet);
+        if (_cssStylesheet != null) {
+            global.logError("There was some problem parsing the theme: " + _cssStylesheet + ".  Falling back to the default theme.");
+        }
+    }
+
     themeContext.set_theme (theme);
 }
 
 /**
  * notify:
- * @msg: A message
- * @details: Additional information
+ * @msg (string): A message
+ * @details (string): Additional information to be
+ *
+ * Sends a notification
  */
 function notify(msg, details) {
     let source = new MessageTray.SystemNotificationSource();
@@ -702,9 +801,23 @@ function notify(msg, details) {
 }
 
 /**
- * notifyError:
- * @msg: An error message
+ * criticalNotify:
+ * @msg: A critical message
  * @details: Additional information
+ */
+function criticalNotify(msg, details, icon) {
+    let source = new MessageTray.SystemNotificationSource();
+    messageTray.add(source);
+    let notification = new MessageTray.Notification(source, msg, details, { icon: icon });
+    notification.setTransient(true);
+    notification.setUrgency(MessageTray.Urgency.CRITICAL);
+    source.notify(notification);
+}
+
+/**
+ * notifyError:
+ * @msg (string): An error message
+ * @details (string): Additional information
  *
  * See cinnamon_global_notify_problem().
  */
@@ -719,10 +832,10 @@ function notifyError(msg, details) {
 
 /**
  * _log:
- * @category: string message type ('info', 'error')
- * @msg: A message string
- * ...: Any further arguments are converted into JSON notation,
- *      and appended to the log message, separated by spaces.
+ * @category (string): string message type ('info', 'error')
+ * @msg (string): A message string
+ * @...: Any further arguments are converted into JSON notation,
+ *       and appended to the log message, separated by spaces.
  *
  * Log a message into the LookingGlass error
  * stream.  This is primarily intended for use by the
@@ -747,16 +860,42 @@ function _log(category, msg) {
     if (can_log) lg_log_file.write(renderLogLine(out), null);
 }
 
+/**
+ * isError:
+ * @obj (Object): the object to be tested
+ * 
+ * Tests whether @obj is an error object
+ * 
+ * Returns (boolean): whether @obj is an error object
+ */
 function isError(obj) {
     return typeof(obj) == 'object' && 'message' in obj && 'stack' in obj;
 }
 
+/**
+ * _LogTraceFormatted:
+ * @stack (string): the stack trace
+ * 
+ * Prints the stack trace to the LookingGlass
+ * error stream in a predefined format
+ */
 function _LogTraceFormatted(stack) {
     _log('trace', '\n<----------------\n' + stack + '---------------->');
 }
 
-// If msg is an Error, its stack-trace will be printed, otherwise a stack-trace will be generated from this call
-// If you want to print the message of an Error as well, use the other log functions instead.
+/**
+ * _logTrace:
+ * @msg (Error): An error object
+ *
+ * Prints a stack trace of the given object.
+ *
+ * If msg is an error, its stack-trace will be
+ * printed. Otherwise, a stack-trace of the call
+ * will be generated
+ *
+ * If you want to print the message of an Error
+ * as well, use the other log functions instead.
+ */
 function _logTrace(msg) {
     if(isError(msg)) {
         _LogTraceFormatted(msg.stack);
@@ -768,12 +907,21 @@ function _logTrace(msg) {
             // _logTrace() (which we strip off), and the second being
             // our caller.
             let trace = e.stack.substr(e.stack.indexOf('\n') + 1);
-            _LogTraceFormatted(stack);
+            _LogTraceFormatted(trace);
         }
     }
 }
 
-// If msg is an Error, its message will be printed as 'warning' and its stack-trace will be printed as 'trace'
+/**
+ * _logWarning:
+ * @msg (Error/string): An error object or the message string
+ *
+ * Logs the message to the LookingGlass error
+ * stream.
+ *
+ * If msg is an error, its stack-trace will be
+ * printed.
+ */
 function _logWarning(msg) {
     if(isError(msg)) {
         _log('warning', msg.message);
@@ -783,7 +931,20 @@ function _logWarning(msg) {
     }
 }
 
-// If msg is an Error, its message will be printed as 'error' and its stack-trace will be printed as 'trace'
+/**
+ * _logError:
+ * @msg (string): (optional) The message string
+ * @error (Error): (optional) The error object
+ * 
+ * Logs the following (if present) to the
+ * LookingGlass error stream:
+ * - The message from the error object
+ * - The stack trace of the error object
+ * - The message @msg
+ * 
+ * It can be called in the form of either _logError(msg),
+ * _logError(error) or _logError(msg, error).
+ */
 function _logError(msg, error) {
     if(error && isError(error)) {
         _log('error', error.message);
@@ -798,6 +959,15 @@ function _logError(msg, error) {
 }
 
 // If msg is an Error, its message will be printed as 'info' and its stack-trace will be printed as 'trace'
+/**
+ * _logInfo:
+ * @msg (Error/string): The error object or the message string
+ * 
+ * Logs the message to the LookingGlass
+ * error stream. If @msg is an Error object, 
+ * its stack trace will also be printed
+ */
+
 function _logInfo(msg) {
     if(isError(msg)) {
         _log('info', msg.message);
@@ -807,14 +977,36 @@ function _logInfo(msg) {
     }
 }
 
+/**
+ * formatTime:
+ * @d (Date): date object to be formatted
+ *
+ * Formats a date object into a ISO-8601 format (YYYY-MM-DDTHH:MM:SSZ) in UTC+0
+ *
+ * Returns (string): a formatted string showing the date
+ */
 function formatTime(d) {
     return d.toISOString();
 }
 
+/**
+ * renderLogLine:
+ * @line (dictionary): a log line
+ * 
+ * Converts a log line object into a string
+ *
+ * Returns (string): line in the format CATEGORY t=TIME MESSAGE
+ */
 function renderLogLine(line) {
     return line.category + ' t=' + formatTime(new Date(parseInt(line.timestamp))) + ' ' + line.message + '\n';
 }
 
+/**
+ * logStackTrace:
+ * @msg (string): message
+ *
+ * Logs the message @msg to stdout with backtrace
+ */
 function logStackTrace(msg) {
     try {
         throw new Error();
@@ -827,6 +1019,15 @@ function logStackTrace(msg) {
     }
 }
 
+/**
+ * isWindowActorDisplayedOnWorkspace:
+ * @win (Meta.WindowActor): window actor
+ * @workspaceIndex (int): index of workspace
+ *
+ * Determines whether the window actor belongs to a specific workspace
+ *
+ * Returns (boolean): whether the window is on the workspace
+ */
 function isWindowActorDisplayedOnWorkspace(win, workspaceIndex) {
     if (win.get_workspace() == workspaceIndex) {return true;}
     let mwin = win.get_meta_window();
@@ -835,6 +1036,14 @@ function isWindowActorDisplayedOnWorkspace(win, workspaceIndex) {
     );
 }
 
+/**
+ * getWindowActorsForWorkspace:
+ * @workspaceIndex (int): index of workspace
+ *
+ * Gets a list of actors on a workspace
+ *
+ * Returns (array): the array of window actors
+ */
 function getWindowActorsForWorkspace(workspaceIndex) {
     return global.get_window_actors().filter(function (win) {
         return isWindowActorDisplayedOnWorkspace(win, workspaceIndex);
@@ -859,6 +1068,10 @@ function _globalKeyPressHandler(actor, event) {
     // This relies on the fact that Clutter.ModifierType is the same as Gdk.ModifierType
     let action = global.display.get_keybinding_action(keyCode, modifierState);
 
+    if (action == Meta.KeyBindingAction.CUSTOM) {
+        global.display.keybinding_action_invoke_by_code(keyCode, modifierState);
+    }
+
     // Other bindings are only available when the overview is up and no modal dialog is present
     if (((!overview.visible && !expo.visible) || modalCount > 1))
         return false;
@@ -916,8 +1129,8 @@ function _findModal(actor) {
 
 /**
  * pushModal:
- * @actor: #ClutterActor which will be given keyboard focus
- * @timestamp: optional timestamp
+ * @actor (Clutter.Actor): actor which will be given keyboard focus
+ * @timestamp (int): optional timestamp
  *
  * Ensure we are in a mode where all keyboard and mouse input goes to
  * the stage, and focus @actor. Multiple calls to this function act in
@@ -932,7 +1145,7 @@ function _findModal(actor) {
  * initiated event.  If not provided then the value of
  * global.get_current_time() is assumed.
  *
- * Returns: true iff we successfully acquired a grab or already had one
+ * Returns (boolean): true iff we successfully acquired a grab or already had one
  */
 function pushModal(actor, timestamp) {
     if (timestamp == undefined)
@@ -943,6 +1156,7 @@ function pushModal(actor, timestamp) {
             log('pushModal: invocation of begin_modal failed');
             return false;
         }
+        Meta.disable_unredirect_for_screen(global.screen);
     }
 
     global.set_stage_input_mode(Cinnamon.StageInputMode.FULLSCREEN);
@@ -973,8 +1187,8 @@ function pushModal(actor, timestamp) {
 
 /**
  * popModal:
- * @actor: #ClutterActor passed to original invocation of pushModal().
- * @timestamp: optional timestamp
+ * @actor (Clutter.Actor): actor passed to original invocation of pushModal().
+ * @timestamp (int): optional timestamp
  *
  * Reverse the effect of pushModal().  If this invocation is undoing
  * the topmost invocation, then the focus will be restored to the
@@ -1023,8 +1237,16 @@ function popModal(actor, timestamp) {
 
     global.end_modal(timestamp);
     global.set_stage_input_mode(Cinnamon.StageInputMode.NORMAL);
+    Meta.enable_unredirect_for_screen(global.screen);
 }
 
+/**
+ * createLookingGlass:
+ *
+ * Obtains the looking glass object. Create if it does not exist
+ *
+ * Returns (LookingGlass.LookingGlass): looking glass object
+ */
 function createLookingGlass() {
     if (lookingGlass == null) {
         lookingGlass = new LookingGlass.LookingGlass();
@@ -1032,6 +1254,13 @@ function createLookingGlass() {
     return lookingGlass;
 }
 
+/**
+ * getRunDialog:
+ *
+ * Obtains the run dialog object. Create if it does not exist
+ *
+ * Returns (RunDialog.RunDialog): run dialog object
+ */
 function getRunDialog() {
     if (runDialog == null) {
         runDialog = new RunDialog.RunDialog();
@@ -1041,9 +1270,9 @@ function getRunDialog() {
 
 /**
  * activateWindow:
- * @window: the Meta.Window to activate
- * @time: (optional) current event time
- * @workspaceNum: (optional) window's workspace number
+ * @window (Meta.Window): the Meta.Window to activate
+ * @time (int): (optional) current event time
+ * @workspaceNum (int): (optional) window's workspace number
  *
  * Activates @window, switching to its workspace first if necessary,
  * and switching out of the overview if it's currently active
@@ -1123,8 +1352,8 @@ function _queueBeforeRedraw(workId) {
 
 /**
  * initializeDeferredWork:
- * @actor: A #ClutterActor
- * @callback: Function to invoke to perform work
+ * @actor (Clutter.Actor): A #ClutterActor
+ * @callback (function): Function to invoke to perform work
  *
  * This function sets up a callback to be invoked when either the
  * given actor is mapped, or after some period of time when the machine
@@ -1137,7 +1366,7 @@ function _queueBeforeRedraw(workId) {
  * initialization as well, under the assumption that new actors
  * will need it.
  *
- * Returns: A string work identifer
+ * Returns (string): A string work identifer
  */
 function initializeDeferredWork(actor, callback, props) {
     // Turn into a string so we can use as an object property
@@ -1161,7 +1390,7 @@ function initializeDeferredWork(actor, callback, props) {
 
 /**
  * queueDeferredWork:
- * @workId: work identifier
+ * @workId (string): work identifier
  *
  * Ensure that the work identified by @workId will be
  * run on map or timeout.  You should call this function
@@ -1188,7 +1417,19 @@ function queueDeferredWork(workId) {
     }
 }
 
+/**
+ * isInteresting:
+ * @metaWindow (Meta.Window): the window to be tested
+ *
+ * Determines whether a window is "interesting", i.e.
+ * ones to be displayed in alt-tab, window list etc.
+ *
+ * Returns (boolean): whether the window is interesting
+ */
 function isInteresting(metaWindow) {
+    if (metaWindow.get_title() == "JavaEmbeddedFrame")
+        return false;
+
     if (tracker.is_window_interesting(metaWindow)) {
         // The nominal case.
         return true;
@@ -1205,12 +1446,14 @@ function isInteresting(metaWindow) {
 
 /**
  * getTabList:
- * @workspaceOpt (optional) workspace, defaults to global.screen.get_active_workspace()
- * @screenOpt: (optional) screen, defaults to global.screen
+ * @workspaceOpt (Meta.Workspace): (optional) workspace, defaults to global.screen.get_active_workspace()
+ * @screenOpt (Meta.Screen): (optional) screen, defaults to global.screen
  *
  * Return a list of the interesting windows on a workspace (by default,
  * the active workspace).
  * The list will include app-less dialogs.
+ *
+ * Returns (array): list of windows
  */
 function getTabList(workspaceOpt, screenOpt) {
     let screen = screenOpt || global.screen;
@@ -1235,16 +1478,3 @@ function getTabList(workspaceOpt, screenOpt) {
     }
     return windows;
 }
-
-/**
- * toggleDesktop:
- *
- * Shows or unshows desktop
- */
-function toggleDesktop() {
-    if (desktopShown)
-        global.screen.unshow_desktop();
-    else
-        global.screen.show_desktop(global.get_current_time());
-    desktopShown = !desktopShown;
-}
