@@ -441,12 +441,11 @@ Notification.prototype = {
         this._titleFitsInBannerMode = true;
         this._titleDirection = St.TextDirection.NONE;
         this._spacing = 0;
-        this._scrollPolicy = Gtk.PolicyType.AUTOMATIC;
+
         this._imageBin = null;
         this._timestamp = new Date();
         this._inNotificationBin = false;
-        let calendarSettings = new Gio.Settings({ schema: 'org.cinnamon.calendar' });
-        this.dateFormat = calendarSettings.get_string('date-format');
+        this.dateFormat = _("%l:%M %p");
 
         source.connect('destroy', Lang.bind(this,
             function (source, reason) {
@@ -607,16 +606,10 @@ Notification.prototype = {
         this._icon.visible = visible;
     },
 
-    enableScrolling: function(enableScrolling) {
-        this._scrollPolicy = enableScrolling ? Gtk.PolicyType.AUTOMATIC : Gtk.PolicyType.NEVER;
-        if (this._scrollArea)
-            this._scrollArea.vscrollbar_policy = this._scrollPolicy;
-    },
-
     _createScrollArea: function() {
         this._table.add_style_class_name('multi-line-notification');
         this._scrollArea = new St.ScrollView({ name: 'notification-scrollview',
-                                               vscrollbar_policy: this._scrollPolicy,
+                                               vscrollbar_policy: Gtk.PolicyType.NEVER,
                                                hscrollbar_policy: Gtk.PolicyType.NEVER,
                                                style_class: 'vfade' });
         this._table.add(this._scrollArea, { row: 1,
@@ -759,11 +752,11 @@ Notification.prototype = {
         if (!this._buttonBox) {
 
             let box = new St.BoxLayout({ name: 'notification-actions' });
-            this.setActionArea(box, { x_expand: false,
+            this.setActionArea(box, { x_expand: true,
                                       y_expand: false,
-                                      x_fill: false,
+                                      x_fill: true,
                                       y_fill: false,
-                                      x_align: St.Align.END });
+                                      x_align: St.Align.START });
             this._buttonBox = box;
         }
 
@@ -1041,9 +1034,7 @@ Source.prototype = {
                                         child: this._counterLabel });
         this._counterBin.hide();
 
-        this._iconBin = new St.Bin({ width: this.ICON_SIZE,
-                                     height: this.ICON_SIZE,
-                                     x_fill: true,
+        this._iconBin = new St.Bin({ x_fill: true,
                                      y_fill: true });
 
         this.actor.add_actor(this._iconBin);
@@ -1318,7 +1309,6 @@ SummaryItem.prototype = {
             if (notification.actor.get_parent() == this.notificationStack)
                 this.notificationStack.remove_actor(notification.actor);
             notification.setIconVisible(true);
-            notification.enableScrolling(true);
         }
         this._stackedNotifications = [];
     },
@@ -1335,8 +1325,6 @@ SummaryItem.prototype = {
         stackedNotification.notificationDoneDisplayingId = notification.connect('done-displaying', Lang.bind(this, this._notificationDoneDisplaying));
         stackedNotification.notificationDestroyedId = notification.connect('destroy', Lang.bind(this, this._notificationDestroyed));
         this._stackedNotifications.push(stackedNotification);
-        if (!this.source.isChat)
-            notification.enableScrolling(false);
         if (this.notificationStack.get_children().length > 0)
             notification.setIconVisible(false);
         this.notificationStack.add(notification.actor);
@@ -1421,10 +1409,6 @@ MessageTray.prototype = {
         this._notificationState = State.HIDDEN;
         this._notificationTimeoutId = 0;
         this._notificationExpandedId = 0;
-        if (Main.overview.visible || Main.expo.visible)
-            this._overviewVisible = true;
-        else
-            this._overviewVisible = false;
         this._notificationRemoved = false;
         this._reNotifyAfterHideNotification = null;
         
@@ -1433,45 +1417,26 @@ MessageTray.prototype = {
 
         Main.layoutManager.connect('monitors-changed', Lang.bind(this, this._setSizePosition));
 
+        let onNotificationEnabledUpdated = Lang.bind(this, function() {
+            this._notificationsEnabled = global.settings.get_boolean("display-notifications");
+        });
+        global.settings.connect('changed::display-notifications', onNotificationEnabledUpdated);
+        onNotificationEnabledUpdated();
+
         this._setSizePosition();
 
-        Main.overview.connect('showing', Lang.bind(this,
-            function() {
-                this._overviewVisible = true;
-                if (this._locked) {
-                    this._unlock();
-                } else {
-                    this._updateState();
-                }
-            }));
-        Main.overview.connect('hiding', Lang.bind(this,
-            function() {
-                this._overviewVisible = false;
-                if (this._locked) {
-                    this._unlock();
-                } else {
-                    this._updateState();
-                }
-            }));
+        let updateLockState = Lang.bind(this, function() {
+            if (this._locked) {
+                this._unlock();
+            } else {
+                this._updateState();
+            }
+        });
 
-        Main.expo.connect('showing', Lang.bind(this,
-            function() {
-                this._overviewVisible = true;
-                if (this._locked) {
-                    this._unlock();
-                } else {
-                    this._updateState();
-                }
-            }));
-        Main.expo.connect('hiding', Lang.bind(this,
-            function() {
-                this._overviewVisible = false;
-                if (this._locked) {
-                    this._unlock();
-                } else {
-                    this._updateState();
-                }
-            }));
+        Main.overview.connect('showing', updateLockState);
+        Main.overview.connect('hiding', updateLockState);
+        Main.expo.connect('showing', updateLockState);
+        Main.expo.connect('hiding', updateLockState);
     },
 
     _setSizePosition: function() {
@@ -1607,27 +1572,30 @@ MessageTray.prototype = {
         let notificationsPending = this._notificationQueue.length > 0 && (!this._busy || notificationUrgent);
         let notificationExpanded = this._notificationBin.y < 0;
 
-        let notificationExpired = (this._notificationTimeoutId == 0 && !(this._notification && this._notification.urgency == Urgency.CRITICAL) && !this._pointerInTray && !this._locked && !(this._pointerInKeyboard && notificationExpanded)) || this._notificationRemoved;
-        let notificationsEnabled = global.settings.get_boolean("display-notifications");
-        let canShowNotification = notificationsPending && notificationsEnabled;
+        let notificationExpired = (this._notificationTimeoutId == 0 &&
+                !(this._notification && this._notification.urgency == Urgency.CRITICAL) &&
+                !this._pointerInTray &&
+                !this._locked &&
+                !(this._pointerInKeyboard && notificationExpanded)
+            ) || this._notificationRemoved;
+        let canShowNotification = notificationsPending && this._notificationsEnabled;
 
         if (this._notificationState == State.HIDDEN) {
-            if (canShowNotification)
+            if (canShowNotification) {
                 this._showNotification();
-            else if (!notificationsEnabled) {
-                this._notification = this._notificationQueue.shift();
-                if (AppletManager.get_role_provider_exists(AppletManager.Roles.NOTIFICATIONS)) {
-                    this.emit('notify-applet-update', this._notification);
+            }
+            else if (!this._notificationsEnabled) {
+                if (notificationsPending) {
+                    this._notification = this._notificationQueue.shift();
+                    if (AppletManager.get_role_provider_exists(AppletManager.Roles.NOTIFICATIONS)) {
+                        this.emit('notify-applet-update', this._notification);
+                    }
                 }
             }
         } else if (this._notificationState == State.SHOWN) {
             if (notificationExpired)
                 this._hideNotification();
         }
-
-        let notificationsVisible = (this._notificationState == State.SHOWING ||
-                                    this._notificationState == State.SHOWN);
-        let notificationsDone = !notificationsVisible && !notificationsPending;
     },
 
     _tween: function(actor, statevar, value, params) {
@@ -1689,15 +1657,7 @@ MessageTray.prototype = {
     _updateShowingNotification: function() {
         Tweener.removeTweens(this._notificationBin);
 
-        // We auto-expand notifications with CRITICAL urgency.
-        // We use Tweener.removeTweens() to remove a tween that was hiding the notification we are
-        // updating, in case that notification was in the process of being hidden. However,
-        // Tweener.removeTweens() would also remove a tween that was updating the position of the
-        // notification we are updating, in case that notification was already expanded and its height
-        // changed. Therefore we need to call this._expandNotification() for expanded notifications
-        // to make sure their position is updated.
-        if (this._notification.urgency == Urgency.CRITICAL || this._notification.expanded)
-            this._expandNotification(true);		
+        this._expandNotification(true);
 
         // We tween all notifications to full opacity. This ensures that both new notifications and
         // notifications that might have been in the process of hiding get full opacity.
