@@ -4,9 +4,6 @@ const Lang = imports.lang;
 const Cinnamon = imports.gi.Cinnamon;
 const Main = imports.ui.main;
 const Signals = imports.signals;
-const AppletManager = imports.ui.appletManager;
-const DeskletManager = imports.ui.deskletManager;
-const ExtensionSystem = imports.ui.extensionSystem;
 const Extension = imports.ui.extension;
 const Mainloop = imports.mainloop;
 
@@ -42,6 +39,13 @@ var STRING_TYPES = {
             "description"
         ]
     },
+    "textview" : {
+        "required-fields": [
+            "type",
+            "default",
+            "description"
+        ]
+    },
     "colorchooser" : {
         "required-fields": [
             "type",
@@ -79,6 +83,13 @@ var STRING_TYPES = {
             "options"
         ]
     },
+    "keybinding" : {
+        "required-fields": [
+            "type",
+            "default",
+            "description"
+        ]
+    },
     "generic" : {
         "required-fields": [
             "type",
@@ -154,8 +165,7 @@ var NON_SETTING_TYPES = {
     }
 };
 
-
-function _provider(xlet, uuid, instanceId) {
+function _provider(xlet, uuid, instanceId, type, string) {
     this._init(xlet, uuid, instanceId, type, string);
 }
 
@@ -166,35 +176,35 @@ _provider.prototype = {
                 this.xlet_str = string;
             }
             if (!xlet) {
-                global.logError(_("%s constructor arguments invalid").format(this.xlet_str));
-                global.logError(_("First argument MUST be a(n) %s object (use 'this' as the first argument").format(this.xlet_str));
+                global.logError(this.xlet_str + " constructor arguments invalid");
+                global.logError("First argument MUST be a(n)" + this.xlet_str + " object (use 'this' as the first argument");
                 return;
             }
             if (!uuid) {
-                global.logError(_("%sSettings constructor arguments invalid").format(this.xlet_str));
-                global.logError(_("Missing required UUID as second argument - should be a string:"));
-                global.logError(_("<%s-name>@<your-id>.org or something similar").format(this.xlet_str));
+                global.logError(this.xlet_str + "Settings constructor arguments invalid");
+                global.logError("Missing required UUID as second argument - should be a string:");
+                global.logError("<xlet-name>@<your-id>.org or something similar");
                 return;
             }
             this.uuid = uuid;
             this.xlet = xlet;
             if (!instanceId && this.ext_type != Extension.Type.EXTENSION) {
-                global.logWarning(_("%sSettings constructor arguments warning").format(this.xlet_str));
-                global.logWarning(_("Missing instance ID as third argument"));
-                global.logWarning(_("The %s UUID is %s").format(this.xlet_str, this.uuid));
+                global.logWarning(this.xlet_str + "Settings constructor arguments warning");
+                global.logWarning("Missing instance ID as third argument");
+                global.logWarning("The UUID is " + this.uuid);
             }
             this.instanceId = instanceId;
             this.valid = false;
             this.applet_dir = Extension.findExtensionDirectory(this.uuid, this.ext_type);
             if (!this.applet_dir) {
-                global.logError(_("Could not find %s installation directory for %s.").format(this.xlet_str, this.uuid));
+                global.logError("Could not find installation directory for " + this.uuid);
                 return;
             }
             this.multi_instance = this._get_is_multi_instance_xlet(this.uuid);
             if (this.multi_instance && !this.instanceId) {
-                global.logError(_("%sSettings fatal error!").format(this.xlet_str));
-                global.logError(_("Multi-instanciable %s with no instance ID supplied").format(this.xlet_str));
-                global.logError(_("The %s UUID is %s").format(this.xlet_str, this.uuid));
+                global.logError(this.xlet_str + "Settings fatal error!");
+                global.logError("Multi-instanciable xlet with no instance ID supplied");
+                global.logError("The UUID is " + this.uuid);
                 return;
             }
 
@@ -210,23 +220,24 @@ _provider.prototype = {
             // If it already exists, check for updates to it (new keys, etc..)
             if (!this.settings_file.query_exists(null)) {
                 if (!this._create_settings_file()) {
-                    global.logError(_("Problem initializing settings for %s").format(this.uuid));
+                    global.logError("Problem initializing settings for " + this.uuid);
                     return;
                 }
             } else {
                 if (!this._maybe_update_settings_file()) {
-                    global.logError(_("Problem updating settings for %s").format(this.uuid));
+                    global.logError("Problem updating settings for " + this.uuid);
                     return;
                 }
             }
 
             this.metaBindings = {}
-            if (!this.instanceId) {
-                this.instanceId = 0; 
+            if (!this.multi_instance) {
+                this.instanceId = this.uuid; 
             }
             this.settings_obj = new SettingObj(this, this.settings_file, this.uuid, this.instanceId);
 
             this.valid = true;
+            Main.settingsManager.register(this.uuid, this.instanceId, this);
         },
 
         _get_is_multi_instance_xlet: function(uuid) {
@@ -239,7 +250,7 @@ _provider.prototype = {
             }
             let orig_file = this.applet_dir.get_child(SETTING_SCHEMA_FILE);
             if (!orig_file.query_exists(null)) {
-                global.logError(_("Failed to locate %s for %s %s").format(SETTING_SCHEMA_FILE, this.xlet_str, this.uuid));
+                global.logError("Failed to locate settings schema file for " + this.uuid);
                 return false;
             }
             let init_file_contents = Cinnamon.get_file_contents_utf8_sync(orig_file.get_path());
@@ -249,11 +260,11 @@ _provider.prototype = {
             try {
                 init_json = JSON.parse(init_file_contents);
             } catch (e) {
-                global.logError(_("Cannot parse %s file for %s %s.  Check the structure for missing commas, etc... Error is: %s").format(SETTING_SCHEMA_FILE, this.xlet_str, this.uuid, e));
+                global.logError("Cannot parse settings schema file for %s" + this.uuid + ".  Check the structure for missing commas, etc... Error is: " + e);
                 return false;
             }
             if (!this._json_validity_check(init_json)) {
-                global.logError(_("Initial %s file is not valid for %s %s").format(SETTING_SCHEMA_FILE, this.xlet_str, this.uuid));
+                global.logError("Initial settings schema file is not valid for " + this.uuid);
                 return false;
             }
 
@@ -261,19 +272,19 @@ _provider.prototype = {
                 init_json[key]["value"] = init_json[key]["default"]
             }
             init_json["__md5__"] = checksum;
-            let out_file = JSON.stringify(init_json, null, 4);
 
-            let fp = this.settings_file.create(0, null);
-            fp.write(out_file, null);
-            fp.close(null);
+            let f = Gio.file_new_for_path(this.settings_file.get_path());
+            let raw = f.replace(null, false, Gio.FileCreateFlags.NONE, null);
+            let out_file = Gio.BufferedOutputStream.new_sized (raw, 4096);
+            Cinnamon.write_string_to_stream(out_file, JSON.stringify(init_json, null, 4));
+            out_file.close(null);
 
             return true;
         },
 
         _json_validity_check: function (init_json) {
-            let primary_key;
             let valid = false;
-            for (primary_key in init_json) {
+            for (let primary_key in init_json) {
                 valid = this._check_for_min_props(init_json[primary_key])
                 if (!valid)
                     break;
@@ -326,8 +337,8 @@ _provider.prototype = {
         _maybe_update_settings_file: function () {
             let orig_file = this.applet_dir.get_child(SETTING_SCHEMA_FILE);
             if (!orig_file.query_exists(null)) {
-                global.logWarning(_("Failed to locate %s for %s %s to check for updates").format(SETTING_SCHEMA_FILE, this.xlet_str, this.uuid));
-                global.logWarning(_("Something may not be right"));
+                global.logWarning("Failed to locate settings schema file to check for updates: " + this.uuid);
+                global.logWarning("Something may not be right");
                 return false;
             }
             let init_file_contents = Cinnamon.get_file_contents_utf8_sync(orig_file.get_path());
@@ -337,15 +348,22 @@ _provider.prototype = {
             let existing_json;
             let new_json;
             try {
-                existing_json = JSON.parse(existing_settings_file);
                 new_json = JSON.parse(init_file_contents);
             } catch (e) {
-                global.logError(_("Problem parsing settings files for %s %s while preparing to perform upgrade").format(this.xlet_str, this.uuid));
-                global.logError(_("Skipping upgrade for now - something may be wrong with the new %s file.").format(SETTING_SCHEMA_FILE));
+                global.logError("Problem parsing " + orig_file.get_path() + " while preparing to perform upgrade.");
+                global.logError("Skipping upgrade for now - something may be wrong with the new settings schema file.");
                 return false;
             }
+            try {
+                existing_json = JSON.parse(existing_settings_file);
+            } catch (e) {
+                global.logError("Problem parsing " + this.settings_file.get_path() + " while preparing to perform upgrade.");
+                global.log("Re-creating settings file.");   
+                this.settings_file.delete(null, null);
+                return this._create_settings_file();
+            }           
             if (existing_json["__md5__"] != checksum) {
-                global.log(_("Updated settings file detected for %s %s.  Beginning upgrade of existing settings").format(this.xlet_str, this.uuid));
+                global.log("Updated settings file detected for " + this.uuid + ".  Beginning upgrade of existing settings");
                 return this._do_upgrade(new_json, existing_json, checksum);
             } else {
                 return true;
@@ -355,8 +373,8 @@ _provider.prototype = {
         _do_upgrade: function(new_json, old_json, checksum) {
             // First, check the new json for validity
             if (!this._json_validity_check(new_json)) {
-                global.logError(_("Upgraded %s file is NOT valid for %s %s.").format(SETTING_SCHEMA_FILE, this.xlet_str, this.uuid));
-                global.logError(_("Aborting settings upgrade."));
+                global.logError("Upgraded settings schema file is NOT valid for " + this.uuid);
+                global.logError("Aborting settings upgrade.");
                 return false;
             }
             /* We're going to iterate through all the keys in the new settings file
@@ -383,16 +401,16 @@ _provider.prototype = {
             }
             new_json["__md5__"] = checksum;
 
-            let out_file = JSON.stringify(new_json, null, 4);
-
             if (this.settings_file.delete(null, null)) {
-                let fp = this.settings_file.create(0, null);
-                fp.write(out_file, null);
-                fp.close;
-                global.log(_("Upgrade complete"));
+                let f = Gio.file_new_for_path(this.settings_file.get_path());
+                let raw = f.replace(null, false, Gio.FileCreateFlags.NONE, null);
+                let out_file = Gio.BufferedOutputStream.new_sized (raw, 4096);
+                Cinnamon.write_string_to_stream(out_file, JSON.stringify(new_json, null, 4));
+                out_file.close(null);
+                global.log("Upgrade complete");
                 return true;
             } else {
-                global.logError(_("Failed to gain write access to save updated settings for %s %s, instance %s").format(this.xlet_str, this.uuid, this.instanceId));
+                global.logError("Failed to gain write access to save updated settings for " + this.uuid + "..." + this.instanceId)
                 return false;
             }
         },
@@ -431,6 +449,10 @@ _provider.prototype = {
             return true;
         },
 
+        remote_set: function (key, payload) {
+            let node = JSON.parse(payload);
+            this.settings_obj.set_node_from_dbus(key, node);
+        },
 
 /* _settings_file_changed:  For convenience only, if you want to handle updating your applet props yourself,
  * connect to this signal on your AppletSettings object to get notified when the json file changes, then
@@ -463,6 +485,8 @@ _provider.prototype = {
                 return false;
             }
             let type = this.settings_obj.get_key_exists_and_type(key_name);
+            if (!applet_callback)
+                applet_callback = function() {};
             if (type) {
                 if (type in BOOLEAN_TYPES || type in STRING_TYPES || type in NUMBER_TYPES) {
                     this.metaBindings[key_name] = new _setting(sync_type, this.xlet, key_name, this.settings_obj, applet_var, Lang.bind (this.xlet, applet_callback), user_data);
@@ -494,6 +518,7 @@ _provider.prototype = {
             }
             this.metaBindings = undefined;
             this.settings_obj = undefined;
+            Main.settingsManager.unregister(this.uuid, this.instanceId);
         },
 
         getValue: function (key_name) {
@@ -554,11 +579,12 @@ SettingObj.prototype = {
 
     save: function() {
         this.settings_file_monitor.disconnect(this.monitor_id);
-        let raw_file = JSON.stringify(this.json, null, 4);
         if (this.file.delete(null, null)) {
-            let fp = this.file.create(0, null);
-            fp.write(raw_file, null);
-            fp.close;
+            let f = Gio.file_new_for_path(this.file.get_path());
+            let raw = f.replace(null, false, Gio.FileCreateFlags.NONE, null);
+            let out_file = Gio.BufferedOutputStream.new_sized (raw, 4096);
+            Cinnamon.write_string_to_stream(out_file, JSON.stringify(this.json, null, 4));
+            out_file.close(null);
         } else {
             global.logError("Failed gain write access to settings file for applet/desklet '" + this.uuid + "', instance ") + this.instanceId;
         }
@@ -582,6 +608,19 @@ SettingObj.prototype = {
         this.save();
     },
 
+    set_node_from_dbus: function(key, node) {
+        let different = false;
+        let old_val = this.json[key]['value'];
+        let new_val = node['value']
+        if (old_val != new_val) {
+            this.json[key] = node;
+            this.provider._value_changed_notify(key, old_val, new_val);
+            this.emit("setting-file-changed");
+            this.provider._setting_file_changed_notify();
+            this.save(); // TODO: This is probably wrong to have here....should be earlier..but i'd rather do everything else first, for responsiveness
+        }
+    },
+
     _on_file_changed: function() {
         if (this.file_changed_timeout) {
             Mainloop.source_remove(this.file_changed_timeout);
@@ -665,17 +704,20 @@ _setting.prototype = {
 
     _setting_file_changed: function () {
         if (this.sync_type != BindingDirection.OUT) {
-            let new_val = this.get_val();
-            if (new_val != this.obj[this.applet_var]) {
-                this._monitor_applet_var(false);
-                this.obj[this.applet_var] = this.get_val();
-                if (this.user_data) {
-                    this.cb(user_data);
-                } else {
-                    this.cb();
-                }
-                this._monitor_applet_var(true);
+            this.set_applet_var_and_cb(this.get_val());
+        }
+    },
+
+    set_applet_var_and_cb: function (new_val) {
+        if (new_val != this.obj[this.applet_var]) {
+            this._monitor_applet_var(false);
+            this.obj[this.applet_var] = new_val;
+            if (this.user_data) {
+                this.cb(user_data);
+            } else {
+                this.cb();
             }
+            this._monitor_applet_var(true);
         }
     },
 
@@ -719,9 +761,7 @@ AppletSettings.prototype = {
     },
 
     _get_is_multi_instance_xlet: function(uuid) {
-        let num = -1;
-        num = AppletManager.get_num_instances_for_applet(uuid);
-        return num > 1 || num == -1;
+        return Extension.get_max_instances(uuid) > 1;
     },
 };
 
@@ -738,9 +778,7 @@ DeskletSettings.prototype = {
     },
 
     _get_is_multi_instance_xlet: function(uuid) {
-        let num = -1;
-        num = DeskletManager.get_num_instances_for_desklet(uuid);
-        return num > 1 || num == -1;
+        return Extension.get_max_instances(uuid) > 1;
     }
 };
 
@@ -760,3 +798,23 @@ ExtensionSettings.prototype = {
         return false;
     }
 };
+
+function SettingsManager() {
+    this._init();
+}
+
+SettingsManager.prototype = {
+        _init: function () {
+            this.uuids = {};
+        },
+
+        register: function (uuid, instance_id, obj) {
+            if (!(uuid in this.uuids))
+                this.uuids[uuid] = {}
+            this.uuids[uuid][instance_id] = obj;
+        },
+
+        unregister: function (uuid, instance_id) {
+            this.uuids[uuid][instance_id] = null;
+        }
+};
