@@ -3,7 +3,6 @@
 #include "config.h"
 
 #include "cinnamon-app-system.h"
-#include "cinnamon-app-usage.h"
 #include <string.h>
 
 #include <gio/gio.h>
@@ -84,6 +83,27 @@ static void cinnamon_app_system_class_init(CinnamonAppSystemClass *klass)
   g_type_class_add_private (gobject_class, sizeof (CinnamonAppSystemPrivate));
 }
 
+static void
+setup_merge_dir_symlink(void)
+{
+    gchar *user_config = g_get_user_config_dir();
+    gchar *merge_path = g_build_filename (user_config, "menus", "applications-merged", NULL);
+    GFile *merge_file = g_file_new_for_path (merge_path);
+
+    g_file_make_directory_with_parents (merge_file, NULL, NULL);
+
+    gchar *sym_path = g_build_filename (user_config, "menus", "cinnamon-applications-merged", NULL);
+    GFile *sym_file = g_file_new_for_path (sym_path);
+    if (!g_file_query_exists (sym_file, NULL)) {
+        g_file_make_symbolic_link (sym_file, merge_path, NULL, NULL);
+    }
+
+    g_free (merge_path);
+    g_free (sym_path);
+    g_object_unref (merge_file);
+    g_object_unref (sym_file);
+}
+
 static void
 cinnamon_app_system_init (CinnamonAppSystem *self)
 {
@@ -97,6 +117,14 @@ cinnamon_app_system_init (CinnamonAppSystem *self)
   priv->id_to_app = g_hash_table_new_full (g_str_hash, g_str_equal,
                                            NULL,
                                            (GDestroyNotify)g_object_unref);
+
+/* According to desktop spec, since our menu file is called 'cinnamon-applications', our
+ * merged menu folders need to be called 'cinnamon-applications-merged'.  We'll setup the folder
+ * 'applications-merged' if it doesn't exist yet, and a symlink pointing to it in the
+ * ~/.config/menus directory
+ */
+  setup_merge_dir_symlink();
+
   /* For now, we want to pick up Evince, Nemo, etc.  We'll
    * handle NODISPLAY semantics at a higher level or investigate them
    * case by case.
@@ -679,34 +707,6 @@ cinnamon_app_system_get_running (CinnamonAppSystem *self)
   return ret;
 }
 
-
-static gint
-compare_apps_by_usage (gconstpointer a,
-                       gconstpointer b,
-                       gpointer      data)
-{
-  CinnamonAppUsage *usage = cinnamon_app_usage_get_default ();
-
-  CinnamonApp *app_a = (CinnamonApp*)a;
-  CinnamonApp *app_b = (CinnamonApp*)b;
-
-  return cinnamon_app_usage_compare (usage, "", app_a, app_b);
-}
-
-static GSList *
-sort_and_concat_results (CinnamonAppSystem *system,
-                         GSList         *prefix_matches,
-                         GSList         *substring_matches)
-{
-  prefix_matches = g_slist_sort_with_data (prefix_matches,
-                                           compare_apps_by_usage,
-                                           system);
-  substring_matches = g_slist_sort_with_data (substring_matches,
-                                              compare_apps_by_usage,
-                                              system);
-  return g_slist_concat (prefix_matches, substring_matches);
-}
-
 /**
  * normalize_terms:
  * @terms: (element-type utf8): Input search terms
@@ -752,8 +752,7 @@ search_tree (CinnamonAppSystem *self,
   g_slist_foreach (normalized_terms, (GFunc)g_free, NULL);
   g_slist_free (normalized_terms);
 
-  return sort_and_concat_results (self, prefix_results, substring_results);
-
+  return g_slist_concat (prefix_results, substring_results);
 }
 
 /**
@@ -809,7 +808,7 @@ cinnamon_app_system_subsearch (CinnamonAppSystem   *system,
   /* Note that a shorter term might have matched as a prefix, but
      when extended only as a substring, so we have to redo the
      sort rather than reusing the existing ordering */
-  return sort_and_concat_results (system, prefix_results, substring_results);
+  return g_slist_concat (prefix_results, substring_results);
 }
 
 /**
@@ -827,4 +826,4 @@ cinnamon_app_system_search_settings (CinnamonAppSystem  *self,
 {
   GSList *null_list;
   return null_list;
-}
\ No newline at end of file
+}
