@@ -7,7 +7,7 @@
 #endif
 #include <stdlib.h>
 #include <string.h>
-
+#include <X11/Xlib.h>
 #include <clutter/clutter.h>
 #include <clutter/x11/clutter-x11.h>
 #include <dbus/dbus-glib.h>
@@ -31,8 +31,6 @@ extern GType gnome_cinnamon_plugin_get_type (void);
 #define CINNAMON_DBUS_SERVICE "org.Cinnamon"
 #define MAGNIFIER_DBUS_SERVICE "org.gnome.Magnifier"
 
-#define OVERRIDES_SCHEMA "org.cinnamon.overrides"
-
 static void
 cinnamon_dbus_init (gboolean replace)
 {
@@ -129,19 +127,6 @@ cinnamon_dbus_init (gboolean replace)
   g_object_unref (bus);
 }
 
-static void
-cinnamon_prefs_init (void)
-{
-  meta_prefs_override_preference_schema ("attach-modal-dialogs",
-                                         OVERRIDES_SCHEMA);
-  meta_prefs_override_preference_schema ("workspaces-only-on-primary",
-                                         OVERRIDES_SCHEMA);
-  meta_prefs_override_preference_schema ("button-layout",
-                                         OVERRIDES_SCHEMA);
-  meta_prefs_override_preference_schema ("edge-tiling",
-                                         OVERRIDES_SCHEMA);
-}
-
 static void
 malloc_statistics_callback (CinnamonPerfLog *perf_log,
                             gpointer      data)
@@ -238,13 +223,15 @@ main (int argc, char **argv)
   GOptionContext *ctx;
   GError *error = NULL;
   int ecode;
-
+  g_setenv ("CLUTTER_DISABLE_XINPUT", "1", TRUE);
   g_type_init ();
 
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
+  g_setenv ("GDK_SCALE", "1", TRUE);
+
   ctx = meta_get_option_context ();
   g_option_context_add_main_entries (ctx, gnome_cinnamon_options, GETTEXT_PACKAGE);
   if (!g_option_context_parse (ctx, &argc, &argv, &error))
@@ -263,19 +250,22 @@ main (int argc, char **argv)
   meta_init ();
   g_unsetenv ("NO_GAIL");
   g_unsetenv ("NO_AT_BRIDGE");
+  g_unsetenv ("CLUTTER_DISABLE_XINPUT");
 
   /* FIXME: Add gjs API to set this stuff and don't depend on the
    * environment.  These propagate to child processes.
    */
   g_setenv ("GJS_DEBUG_OUTPUT", "stderr", TRUE);
   g_setenv ("GJS_DEBUG_TOPICS", "JS ERROR;JS LOG", TRUE);
-  
+
+  g_setenv ("CINNAMON_VERSION", VERSION, TRUE);
+
+
   center_pointer_on_screen();
 
   cinnamon_dbus_init (meta_get_replace_current_wm ());
   cinnamon_a11y_init ();
   cinnamon_perf_log_init ();
-  cinnamon_prefs_init ();
 
   g_irepository_prepend_search_path (CINNAMON_PKGLIBDIR);
 #if HAVE_BLUETOOTH
@@ -293,6 +283,8 @@ main (int argc, char **argv)
   /* Initialize the global object */
   _cinnamon_global_init (NULL);
 
+  g_unsetenv ("GDK_SCALE");
+
   ecode = meta_run ();
 
   if (g_getenv ("CINNAMON_ENABLE_CLEANUP"))
