@@ -24,6 +24,7 @@
 #include <gdk/gdk.h>
 
 #include "st-theme-node.h"
+#include "st-types.h"
 
 G_BEGIN_DECLS
 
@@ -43,7 +44,10 @@ struct _StThemeNode {
 
   int background_position_x;
   int background_position_y;
-  gboolean background_position_set : 1;
+
+  StBackgroundSize background_size;
+  gint background_size_w;
+  gint background_size_h;
 
   ClutterColor foreground_color;
   ClutterColor border_color[4];
@@ -73,8 +77,8 @@ struct _StThemeNode {
 
   GType element_type;
   char *element_id;
-  char *element_class;
-  char *pseudo_class;
+  GStrv element_classes;
+  GStrv pseudo_classes;
   char *inline_style;
 
   CRDeclaration **properties;
@@ -83,6 +87,9 @@ struct _StThemeNode {
   /* We hold onto these separately so we can destroy them on finalize */
   CRDeclaration *inline_properties;
 
+  guint background_position_set : 1;
+  guint background_repeat : 1;
+
   guint properties_computed : 1;
   guint geometry_computed : 1;
   guint background_computed : 1;
