@@ -34,6 +34,7 @@
 
 #include "st-widget.h"
 
+#include "st-background-effect.h"
 #include "st-label.h"
 #include "st-marshal.h"
 #include "st-private.h"
@@ -285,7 +286,6 @@ st_widget_dispose (GObject *gobject)
 
   if (priv->theme_node)
     {
-      g_object_run_dispose (G_OBJECT (priv->theme_node));
       g_object_unref (priv->theme_node);
       priv->theme_node = NULL;
     }
@@ -426,6 +426,21 @@ st_widget_paint (ClutterActor *actor)
                                     opacity);
   else
     st_theme_node_paint (theme_node, &allocation, opacity);
+
+  ClutterEffect *effect = clutter_actor_get_effect (actor, "background-effect");
+
+  if (effect == NULL)
+    {
+      effect = st_background_effect_new ();
+      clutter_actor_add_effect_with_name (actor, "background-effect", effect);
+    }
+
+  const char *bumpmap_path = st_theme_node_get_background_bumpmap(theme_node);
+
+  g_object_set (effect,
+                "bumpmap",
+                bumpmap_path,
+                NULL);
 }
 
 static void
@@ -578,6 +593,8 @@ st_widget_get_theme_node (StWidget *widget)
       StThemeNode *parent_node = NULL;
       ClutterStage *stage = NULL;
       ClutterActor *parent;
+      StThemeContext *context;
+      StThemeNode *tmp_node;
       char *pseudo_class, *direction_pseudo_class;
 
       parent = clutter_actor_get_parent (CLUTTER_ACTOR (widget));
@@ -595,7 +612,7 @@ st_widget_get_theme_node (StWidget *widget)
         {
           g_critical ("st_widget_get_theme_node called on the widget %s which is not in the stage.",
                     st_describe_actor (CLUTTER_ACTOR (widget)));
-          return NULL;
+          return g_object_new (ST_TYPE_THEME_NODE, NULL);
         }
 
       if (parent_node == NULL)
@@ -616,16 +633,20 @@ st_widget_get_theme_node (StWidget *widget)
       else
         pseudo_class = direction_pseudo_class;
 
-      priv->theme_node = st_theme_node_new (st_theme_context_get_for_stage (stage),
-                                            parent_node, priv->theme,
-                                            G_OBJECT_TYPE (widget),
-                                            clutter_actor_get_name (CLUTTER_ACTOR (widget)),
-                                            priv->style_class,
-                                            pseudo_class,
-                                            priv->inline_style);
+      context = st_theme_context_get_for_stage (stage);
+      tmp_node = st_theme_node_new (context, parent_node, priv->theme,
+                                    G_OBJECT_TYPE (widget),
+                                    clutter_actor_get_name (CLUTTER_ACTOR (widget)),
+                                    priv->style_class,
+                                    pseudo_class,
+                                    priv->inline_style);
 
       if (pseudo_class != direction_pseudo_class)
         g_free (pseudo_class);
+
+      priv->theme_node = g_object_ref (st_theme_context_intern_node (context,
+                                                                     tmp_node));
+      g_object_unref (tmp_node);
     }
 
   return priv->theme_node;
@@ -1448,6 +1469,12 @@ st_widget_recompute_style (StWidget    *widget,
   int transition_duration;
   gboolean paint_equal;
 
+  if (new_theme_node == old_theme_node)
+    {
+      widget->priv->is_style_dirty = FALSE;
+      return;
+    }
+
   if (!old_theme_node ||
       !st_theme_node_geometry_equal (old_theme_node, new_theme_node))
     clutter_actor_queue_relayout ((ClutterActor *) widget);
@@ -1828,8 +1855,10 @@ st_widget_start_tooltip_timeout (StWidget *widget)
   GtkSettings *settings = gtk_settings_get_default ();
   guint timeout;
 
-  if (priv->tooltip_timeout_id)
+  if (priv->tooltip_timeout_id) {
     g_source_remove (priv->tooltip_timeout_id);
+    priv->tooltip_timeout_id = 0;
+  }
 
   g_object_get (settings, "gtk-tooltip-timeout", &timeout, NULL);
   priv->tooltip_timeout_id = g_timeout_add (timeout, tooltip_timeout, widget);
